--[[
Hero :
	Maintains and saves all character specific base stats and qualities. Calculates and provides a snapshot of character specific
	derived stats for use by other systems.

	A
--]]

use_safeglobals()

local function SumValueThroughItems(items, method, ...)
	local itemset = {}
	for _, item in ipairs(items) do		-- stops two handed weapons being counted twice
		itemset[item] = item
	end
	local value = 0
	for item in pairs(itemset) do
		value = value + item[method](item, ...)
	end
	return value
end

class "Hero" (GameObject)

Hero.isHero = true

Hero.AttributeDescriptions = AttributeDescriptionList()
Hero.AttributeDescriptions:AddAttribute('string',	'name',					{serialize=1, default="name"})
Hero.AttributeDescriptions:AddAttribute('string',	'class',				{serialize=1, default="class"})
Hero.AttributeDescriptions:AddAttribute('int',		'level',				{serialize=1, default=1})
Hero.AttributeDescriptions:AddAttribute('string',	'portrait',				{serialize=1, default="portrait_hero"})
Hero.AttributeDescriptions:AddAttribute('string',	'sex',					{serialize=1, default="M", fixed=1})
Hero.AttributeDescriptions:AddAttribute('int',		'wounds',				{serialize=1, default=0})			-- The amount of wounds you have. In battle, your health is health - wounds
Hero.AttributeDescriptions:AddAttribute('int',		'gold',					{serialize=1, default=0})			-- Current gold
Hero.AttributeDescriptions:AddAttribute('int',		'xp',					{serialize=1, default=0})			-- Current XP
Hero.AttributeDescriptions:AddAttribute('int',		'strength',				{serialize=1, default=0})			-- Player chosen levelup bonus
Hero.AttributeDescriptions:AddAttribute('int',		'agility',				{serialize=1, default=0})			-- Player chosen levelup bonus
Hero.AttributeDescriptions:AddAttribute('int',		'stamina',				{serialize=1, default=0})			-- Player chosen levelup bonus
Hero.AttributeDescriptions:AddAttribute('int',		'intelligence',			{serialize=1, default=0})			-- Player chosen levelup bonus
Hero.AttributeDescriptions:AddAttribute('int',		'morale',				{serialize=1, default=0})			-- Player chosen levelup bonus

Hero.AttributeDescriptions:AddAttribute('int',		'play_time',			{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute('int',		'difficulty',			{serialize=1, default=1})

Hero.AttributeDescriptions:AddAttribute("GameObject", "mainHandItem",		{serialize=1})
Hero.AttributeDescriptions:AddAttribute("GameObject", "offHandItem",		{serialize=1})

Hero.AttributeDescriptions:AddAttribute("GameObject", "helmItem",			{serialize=1})
Hero.AttributeDescriptions:AddAttribute("GameObject", "armourItem",			{serialize=1})
Hero.AttributeDescriptions:AddAttribute("GameObject", "jewelleryItem",		{serialize=1})
Hero.AttributeDescriptions:AddAttribute("GameObject", "feetItem",			{serialize=1})

Hero.AttributeDescriptions:AddAttribute('int',		'wood',					{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute('int',		'metal',				{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute('int',		'leather',				{serialize=1, default=0})

Hero.AttributeDescriptions:AddAttribute('int',		'amber',				{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute('int',		'pearl',				{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute('int',		'emerald',				{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute('int',		'ruby',					{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute('int',		'sapphire',				{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute('int',		'crystal',				{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute('int',		'fang',					{serialize=1, default=0})

Hero.AttributeDescriptions:AddAttributeCollection("GameObject",	"inventory",{serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string",	"knownSpells",	{serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string",	"activeSpells",	{serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string",	"friends",		{serialize=1})

Hero.AttributeDescriptions:AddAttributeCollection("string", "traits",		{serialize=1})

Hero.AttributeDescriptions:AddAttribute("int", 		"bashGameCount",		{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute("int", 		"knockGameCount",		{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute("int", 		"pickGameCount",		{serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute("int", 		"numItemActivations",	{serialize=1, default=0})

--Tutorial Checks
Hero.AttributeDescriptions:AddAttribute("bool", 	"bashTutorialShown",			{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"knockTutorialShown",		{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"pickTutorialShown",			{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"captureTutorialShown",		{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"disarmTutorialShown",		{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"hoardTutorialShown",		{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"searchTutorialShown",		{serialize=1, default=false})

Hero.AttributeDescriptions:AddAttribute("bool", 	"inventoryTutorialShown",	{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"levelUpTutorialShown",		{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"spellbookTutorialShown",	{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"questLogTutorialShown",	{serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", 	"portalTutorialShown",		{serialize=1, default=false})


--[[---------------------------------------------------------------------------

About the hero world state:

 - lockedNodes.  All nodes in the world are unlocked by default however, some
   nodes should only be made visible later (after discovering a secret door,
   for example).  The lockedNodes list contains all nodes that have yet to be
   unlocked.

 - currentLocation, currentNode.  The id's of the players current level and
   the node on which they are standing.

 - questItems.  Quest items do not exist in the normal inventory, the player
   cannot have more than one of a particular type of quest item.  This is a
   list of all the quest item id's the player has.

 - clearedEncounters.  Static encounters should not respawn after the player
   has killed the monster (stuff like bosses).  Once an encounter is cleared
   its node should should be added to this list.  The encounter spawning code
   will not spawn a static encounter at a node in this list.

 - visitedRooms.  A list of all rooms visited by the player.  Once the player
   enters a room for the first time it is added to this list.

 - detectedTraps.  A list of all trap nodes that have been detected by the
   player.  Note, that the trap may also appear in "disarmedTraps".

 - disarmedTraps.  A list of all trap nodes that have been triggered by the
   player.

 - doorsUnlocked.  A list of locked door nodes that have been unlocked by the
   player using one of the minigames.

-----------------------------------------------------------------------------]]

Hero.AttributeDescriptions:AddAttributeCollection("string", "lockedNodes", {fixed=4, serialize=1})
Hero.AttributeDescriptions:AddAttribute("string", "currentLocation", {fixed=4, serialize=1, default="R909"})
Hero.AttributeDescriptions:AddAttribute("string", "currentNode", {fixed=4, serialize=1, default="9090"})
Hero.AttributeDescriptions:AddAttribute("string", "currentDir", {fixed=2, default="se", serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string", "questItems", {serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string", "clearedEncounters", {fixed=4, serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string", "visitedRooms", {fixed=4, serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string", "detectedTraps", {fixed=4, serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string", "disarmedTraps", {fixed=4, serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string", "doorsUnlocked", {fixed=4, serialize=1})
Hero.AttributeDescriptions:AddAttributeCollection("string", "lootedTreasures", {fixed=4, serialize=1})
Hero.AttributeDescriptions:AddAttribute("bool", "introductionCutsceneShown", {serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("bool", "introductionConversationShown", {serialize=1, default=false})
Hero.AttributeDescriptions:AddAttribute("int", "activeQuest", {serialize=1, default=0})
Hero.AttributeDescriptions:AddAttribute("int", "dummy", {serialize=1, default=11})

Hero.AttributeDescriptions:AddAttribute("int",	"seed", {serialize=1})

-- Intelligence is the unpredictability of the best move selection.  It is
-- a number by which an individual move might be biased by.  That is, for
-- every potential move, the game adds -int>x>int to the score.

Hero.AttributeDescriptions:AddAttribute("int",	"move_intelligence", {default=0}) -- Not saved, we only use this for monsters

local hero_stats_constants = require( "Hero/StatsConstants" )
local MAX_INV_ITEMS = 34 -- 24 items + 10 trade items

function Hero:__init(classid)
	super(classid or "HERO")
	MakeQuestable(self)
	MakeTutorialised(self)
	self.questMessages = {}
	
	-- session data
	self.sessionData = {}
	self.sessionData.monsters = {}
	self.sessionData.loot = {}
	self.sessionData.randomEncounters = {}
	self.sessionData.portal = {}
	self.sessionData.shops = {}
end

function Hero:PreDestroy()

	if self.sessionData and self.sessionData.randomEncounters then
		for node, data in pairs(self.sessionData.randomEncounters) do
			if data.hero then
				DestroyMonsterHero(data.hero)
			end
		end
	end
	
	if self.sessionData and self.sessionData.monsters then
		for node, monster in pairs(self.sessionData.monsters) do
			DestroyMonsterHero(monster)
		end
	end
	
	self.sessionData.randomEncounter = nil
	self.sessionData.monsters = nil
end

-- functions to use the dummy attribute to store the rebuilding tutorials
function Hero:GetRebuildUnlockTutorialSeen()
	local dummy = string.format("%02d", self:GetAttribute("dummy"))
	local tut1 = string.sub(dummy, 1, 1)
	
	return tonumber(tut1) == 0
end

function Hero:SetRebuildUnlockTutorialSeen()
	local dummy = string.format("%02d", self:GetAttribute("dummy"))
	local tut2 = string.sub(dummy, 2, 2)
	
	self:SetAttribute("dummy", tonumber(string.format("0%s", tut2)))
end

function Hero:GetRebuildTutorialSeen()
	local dummy = string.format("%02d", self:GetAttribute("dummy"))
	local tut2 = string.sub(dummy, 2, 2)
	
	return tonumber(tut2) == 0
end

function Hero:SetRebuildTutorialSeen()
	local dummy = string.format("%02d", self:GetAttribute("dummy"))
	local tut1 = string.sub(dummy, 1, 1)
	
	self:SetAttribute("dummy", tonumber(string.format("%s0", tut1)))
end

function Hero:InitialiseCampaign()
	self:ClearAttributeCollection("lockedNodes")
	self:ClearAttributeCollection("questItems")
	self:ClearAttributeCollection("clearedEncounters")
	self:ClearAttributeCollection("detectedTraps")
	self:ClearAttributeCollection("disarmedTraps")
	self:ClearAttributeCollection("doorsUnlocked")
	self:ClearAttributeCollection("lootedTreasures")

	self:SetAttribute("seed", GetSystemTime())

	for _, v in ipairs(require("LevelData.LevelsLocked")) do
		self:PushAttribute("lockedNodes", v)
	end

	self:SetAttribute("currentLocation", "R909")
	self:SetAttribute("currentNode", "9090")
	self:SetAttribute("currentDir", "ne")

	InitQuests(self, 124)
	-- StartQuest(self, 1)
end

function Hero:PrintDebug(verbose)
	LOGF("HERO DEBUG INFO: %s (%s)", self:GetAttribute("name"), translate_text(self:GetAttribute("name")))
	LOGF("    Seed:        %d", self:GetAttribute("seed"))
	LOGF("    XP:          %d", self:GetAttribute("xp"))
	LOGF("    Gold:        %d", self:GetAttribute("gold"))
	LOGF("    Level:       %d", self:GetAttribute("level"))
	LOGF("")
	LOGF("  Attributes:")
	LOGF("    Strength:     %d (r%d.%s)", self.stats.base.strength.points, self.stats.base.strength.rank, tostring(self.stats.base.strength.rankname))
	LOGF("    Agility:      %d (r%d.%s)", self.stats.base.agility.points, self.stats.base.agility.rank, tostring(self.stats.base.agility.rankname))
	LOGF("    Stamina:      %d (r%d.%s)", self.stats.base.stamina.points, self.stats.base.stamina.rank, tostring(self.stats.base.stamina.rankname))
	LOGF("    Intelligence: %d (r%d.%s)", self.stats.base.intelligence.points, self.stats.base.intelligence.rank, tostring(tostring(self.stats.base.intelligence.rankname)))
	LOGF("    Morale:       %d (r%d.%s)", self.stats.base.morale.points, self.stats.base.morale.rank, tostring(self.stats.base.morale.rankname))
	LOGF("")
	LOGF("  Attack Rating:  %d", self:GetAttack())
	LOGF("  Defence Rating: %d", self:GetDefence())
	LOGF("  Health:         %d (%d)", self:GetHealth(), self:GetMaxHealth())
	LOGF("")
	LOGF("  Trade Items:")
	LOGF("    Wood:         %d", self:GetCommodityAmount("wood"))
	LOGF("    Metal:        %d", self:GetCommodityAmount("metal"))
	LOGF("    Leather:      %d", self:GetCommodityAmount("leather"))
	LOGF("    Amber:        %d", self:GetCommodityAmount("amber"))
	LOGF("    Pearl:        %d", self:GetCommodityAmount("pearl"))
	LOGF("    Emerald:      %d", self:GetCommodityAmount("emerald"))
	LOGF("    Ruby:         %d", self:GetCommodityAmount("ruby"))
	LOGF("    Sapphire:     %d", self:GetCommodityAmount("sapphire"))
	LOGF("    Fang:         %d", self:GetCommodityAmount("fang"))
	LOGF("    Crystal:      %d", self:GetCommodityAmount("crystal"))
	LOGF("")
	LOGF("  Stats:")
	LOGF("    Item Uses:    %d", self:GetAttribute("numItemActivations"))
	LOGF("    Bash Count:   %d", self:GetAttribute("bashGameCount"))
	LOGF("    Knock Count:  %d", self:GetAttribute("knockGameCount"))
	LOGF("    Pick Count:   %d", self:GetAttribute("pickGameCount"))

	if self:GetAttribute("mainHandItem") then
		LOG("=== Main Hand ===")
		self:GetAttribute("mainHandItem"):DebugPrint(self)
	end

	if self:GetAttribute("offHandItem") then
		LOG("=== Off Hand ===")
		self:GetAttribute("offHandItem"):DebugPrint(self)
	end

	LOG("=== Passive ===")
	
	if self:GetAttribute("helmItem") then
		self:GetAttribute("helmItem"):DebugPrint(self)
	end
	if self:GetAttribute("armourItem") then
		self:GetAttribute("armourItem"):DebugPrint(self)
	end
	if self:GetAttribute("jewelleryItem") then
		self:GetAttribute("jewelleryItem"):DebugPrint(self)
	end
	if self:GetAttribute("feetItem") then
		self:GetAttribute("feetItem"):DebugPrint(self)
	end

	LOG("Known spells:")

	for i, spell in IterateCollection(self, "knownSpells") do
		LOGF("\t%s", spell)
	end

	LOG("Active spells:")

	for i, spell in IterateCollection(self, "activeSpells") do
		LOGF("\t%s", spell)
	end

	LOG("Friends:")

	for i, friend in IterateCollection(self, "friends") do
		LOGF("\t%s", friend)
	end
	
	LOG("Inventory items:")

	for i, item in IterateCollection(self, "inventory") do
		item:DebugPrint(self)
	end

	local function DumpCollection(object, collection, heading)
		LOGF("%s:", heading)
		for i, node in IterateCollection(self, collection) do
			LOGF("\t%s", node)
		end
	end

	if verbose then
		DumpCollection(self, "lockedNodes", "Locked Nodes")
		DumpCollection(self, "questItems", "Quest Items")
		DumpCollection(self, "clearedEncounters", "Encounters Defeated")
		DumpCollection(self, "visitedRooms", "Rooms Visited")
		DumpCollection(self, "detectedTraps", "Traps Detected")
		DumpCollection(self, "disarmedTraps", "Traps Triggered or Disarmed")
		DumpCollection(self, "doorsUnlocked", "Doors Unlocked")
		DumpCollection(self, "lootedTreasures", "Treasures Looted")
	end

	LOGF("Active quest index: %d", self:GetAttribute("activeQuest"))
	
	if self:NumAttributes("quest_state") > 0 then
		LOG("Active Portals:")
		
		for room, room_name, node in IterateActivePortals(self) do
			LOGF("\troom: %s, node: %s", tostring(room), tostring(node))
		end
	end

    DumpQuests(self)
end

function Hero:GetGameDifficulty()
	return self:GetAttribute("difficulty")
end

function Hero:SetGameDifficulty(level)
	assert(type(level)=="number")
	
	self:SetAttribute("difficulty", level)
end

function Hero:HasIntroConversationBeenPlayed()
	return self:GetAttribute("introductionConversationShown")
end

function Hero:SetIntroConversationCompleted()
	return self:SetAttribute("introductionConversationShown", true)
end

function Hero:GetCurrentLocationLevelName()
	return ROOMS[self:GetAttribute("currentLocation").."_meta"].level
end

function Hero:GetCurrentRoom()
	return self:GetAttribute("currentLocation")
end

function Hero:GetCurrentNode()
	return self:GetAttribute("currentNode")
end

function Hero:GetNumberOfItems()
	local num_items = 0
	for index, item in IterateCollection(self, "inventory") do
		if not item:IsTrade() then
			num_items = num_items + 1
		end
	end
	return num_items
end

function Hero:GetMaxInvItems()
	return MAX_INV_ITEMS
end

function Hero:GetMaxResources()
	--return #(require("Items.ItemConfiguration").CATALYST_ITEMS) + 3 -- 3 for metal, leather, wood
	return 10 -- normally a Very Bad Thing(tm) but it's not going to change at this stage of the game - MS 7-Apr-10
end
	
-- AddItemWouldExceedCapLimit:
--     Returns true if adding the specified number of items would exceed the max items cap (takes missing 
--     commodities into account)
-- NB: The number passed in should NEVER include trade items (commodities)!
-- Alogorithm: Get a list of all commodity types then remove any that are already in the inventory.
--     Add however many are left to the size of the inventory and requested quantity to add and compare 
--     to the cap.
function Hero:AddItemWouldExceedCapLimit(num_to_add)
	assert("Need to make sure calls to this from other than Hero:AddItem() only happen for non-commodity items!")
	num_to_add = num_to_add or 1
	local current_size = self:NumAttributes("inventory")
	local file_commodities = require("Items.ItemConfiguration").CATALYST_ITEMS
	local commodities = {}
	for _,v in ipairs(file_commodities) do
		table.insert(commodities, v)
	end
	table.insert(commodities, 1, "metal")
	table.insert(commodities, 1, "wood")
	table.insert(commodities, 1, "leather")
	file_commodities = nil
	local num_commodities = #commodities
	
	for index, item in IterateCollection(self, "inventory") do
		if item.IS_TRADE then
			local item_name = string.lower(item:GetAttribute("resource"))
			for i, commodity in ipairs(commodities) do
				if item_name == commodity then
					table.remove(commodities, i)
					num_commodities = #commodities
					break
				end
			end
		end
		if num_commodities == 0 then
			break
		end
	end
	local newSize = current_size + num_commodities + num_to_add
	local result = newSize > MAX_INV_ITEMS
	
	return result  
end

-- AddItem:
--     Adds an item to the players inventory.  The ownership of the game object (item) is
--     transferred to the hero at this point.
function Hero:AddItem(item)
   assert(item)

	if item:IsTrade() then
		self:AddCommodityItem(item)
		GameObjectManager:Destroy(item)
	else
		if not self:AddItemWouldExceedCapLimit() then
			self:PushAttribute("inventory", item)
			self:AddChild(item)
			self:AutoEquipItem(item)
		else
			open_message_menu("[INV_FULL]", "[INV_FULL_MSG]")
			return
		end
	end
	
	return true
end

-- RemoveItem:
--     Removes an item from the heroes inventory and returns it.  The hero relinquishes its
--     ownership of the object.  This method automatically unequips the item.

function Hero:RemoveItem(item)
    assert(item)
    assert(CollectionContainsAttribute(self, "inventory", item))

	self:UnequipItem(item, true)

    if CollectionContainsAttribute(self, "inventory", item) then
        self:EraseAttribute("inventory", item)
    end

    self:RemoveChild(item)

    return item
end

function Hero:IsMonster()
	return self.def ~= nil
end

-- GetName:
--     Returns the hero name, already translated.  If this is a monster hero then this is
--     the translated name of the monster type (for example, "[TROLL_NAME]").  If it is a
--     player hero then it is the players name they entered.

function Hero:GetName()
	if self.def and self.def.texttag then -- this is a monster hero
		return translate_text(string.format("[%s_NAME]", self.def.texttag))
	else
		return self:GetAttribute("name")
	end
end

-- GetDescriptionTag:
--     Returns a text tag for the hero.  If the hero is a monster then it returns the description
--     for the monster.  If the hero is a player hero then it returns the classes description.

function Hero:GetDescriptionTag()
	if self.def and self.def.texttag then -- this is a monster hero
		return string.format("[%s_LONG]", self.def.texttag)
	else
		return string.format("[%s_DESC]", string.upper(self:GetAttribute("class")))
	end
end

function Hero:GetToolTipFormatAndData()
	return GetToolTipFormatAndData(self:GetName(), self:GetAttribute("level"), self:GetDescriptionTag())
end

function Hero:GetStealthToolTipFormatAndData(format, data)
	return GetStealthToolTipFormatAndData(format, data)
end

function Hero:GetSmallPortrait()
	return self:GetAttribute("portrait")
end

function Hero:GetBigPortrait()
	return self:GetAttribute("portrait") .. "_VS"
end

function Hero:GetConversationPortrait()
	return self:GetAttribute("portrait") .. "_M"
end

function Hero:GetWorldMapPortrait()
	return self:GetAttribute("portrait") .. "_WM"
end

function Hero:GetGenderForConversation()
	if self:GetAttribute("sex") == "M" then
		return "male"
	else
		return "female"
	end
end

function Hero:AddCommodityItem(item)
	-- little more than a macro to simplify adding already-constructed trade items
	self:AddCommodity(item:GetAttribute("resource"), item:GetAttribute("stacksize"))
end

function Hero:AddCommodity(resource, amount)
	assert(resource)
	assert(amount)
	for i, item in IterateCollection(self, "inventory") do
		if item.IS_TRADE and item:GetAttribute("resource") == resource and item:GetAttribute("stacksize") < TradeItem.MAX_STACK_SIZE then
			local stack = item:GetAttribute("stacksize")
			amount = math.min(stack + amount, TradeItem.MAX_STACK_SIZE)
			item:SetAttribute("stacksize", amount)
			amount = 0
		end
	end
	if amount > 0 then
		-- NOTE: this needs to be fixed so that it does not add an extra stack if the players
		-- inventory is full.  It should return the number of the resource that could not be
		-- assigned to the player

		local item = GameObjectManager:ConstructLocal("ITTR")
		local stacksize = math.min(amount, TradeItem.MAX_STACK_SIZE)

		item:SetResource(resource)
		item:SetAttribute("stacksize", stacksize)

		self:PushAttribute("inventory", item)
		self:AddChild(item)

		amount = amount - stacksize
	end
end

function Hero:RemoveCommodity(resource, amount)
	assert(resource)
	assert(amount)
	local itemsToRemove = {}
	local amountToGo = amount
	for i, item in IterateCollection(self, "inventory") do
		if item.IS_TRADE and item:GetAttribute("resource") == resource then
			local stackSize = item:GetAttribute("stacksize")
			local reduction = math.min(stackSize, amountToGo)

			if item:GetAttribute("stacksize") - reduction == 0 then
				-- remove this item completely
				table.insert(itemsToRemove, item)
			else
				item:SetAttribute("stacksize", stackSize - reduction)
			end
			amountToGo = amountToGo - reduction
		end
	end
	for _, item in ipairs(itemsToRemove) do
		self:EraseAttribute("inventory", item)
	end
end

function Hero:GetCommodityAmount(resource)
	local amount = 0
	for i, item in IterateCollection(self, "inventory") do
		if item.IS_TRADE and string.lower(item:GetAttribute("resource")) == string.lower(resource) then -- string.lower is a safeguard against "Wood" vs "wood" vs "WOOD"
			if item:HasAttribute("stacksize") then
				amount = amount + item:GetAttribute("stacksize")
			else
				amount = amount + 1
			end
		end
	end
	return amount
end

function Hero:GetMapSprite()
	return string.format("%s%s", self:GetAttribute("class"), self:GetAttribute("sex"))
end

function Hero:GetMinimapIcon()
	return string.format("img_mini_icon_%s%s", self:GetAttribute("class"), self:GetAttribute("sex"))
end

function Hero:IsQuestStartable(index)
	return _G.IsQuestStartable(self, tonumber(index))
end

function Hero:IsQuestActive(index)
	return _G.IsQuestRunning(self, tonumber(index))
end

function Hero:IsQuestComplete(index)
	return _G.IsQuestCompleted(self, tonumber(index))
end

function Hero:IsEncounterDefeated(node)
	return CollectionContainsAttribute(self, "clearedEncounters", assert(node))
end

function Hero:OnEncounterDefeated(node)
	if not CollectionContainsAttribute(self, "clearedEncounters", node) then
		self:PushAttribute("clearedEncounters", node)
	end
end

function Hero:IsNodeVisible(node)
	return not CollectionContainsAttribute(self, "lockedNodes", assert(node))
end

function Hero:SetNodeVisible(node)
	self:EraseAttribute("lockedNodes", node)
end

function Hero:IsTrapDetected(node)
	return CollectionContainsAttribute(self, "detectedTraps", assert(node))
end

function Hero:IsTrapDisarmed(node)
	return CollectionContainsAttribute(self, "disarmedTraps", assert(node))
end

function Hero:SetTrapDetected(node)
	if not CollectionContainsAttribute(self, "detectedTraps", node) then
		self:PushAttribute("detectedTraps", node)
	end
end

function Hero:SetTrapDisarmed(node)
	if not CollectionContainsAttribute(self, "disarmedTraps", node) then
		self:PushAttribute("disarmedTraps", node)
	end
end

function Hero:IsTreasureLooted(node)
	return CollectionContainsAttribute(self, "lootedTreasures", assert(node))
end

function Hero:SetTreasureLooted(node)
	if not CollectionContainsAttribute(self, "lootedTreasures", assert(node)) then
		self:PushAttribute("lootedTreasures", node)
	end
end

function Hero:SetDoorUnlocked(node)
	self:PushAttribute("doorsUnlocked", node)
end

function Hero:IsDoorUnlocked(node)
	return CollectionContainsAttribute(self, "doorsUnlocked", assert(node))
end

function Hero:HasVisitedRoom(node)
	return CollectionContainsAttribute(self, "visitedRooms", assert(node))
end

function Hero:GetNumberOfRoomsVisited()
	return self:NumAttributes("visitedRooms")
end

function Hero:SetRoomVisited(node)
	assert(type(node) == "string")
	if not self:HasVisitedRoom(node) then
		self:PushAttribute("visitedRooms", node)
	end
end

function Hero:CanDetectHiddenEncounter(node)
	-- temp (duh)
	return math.random(1,1) == 2
end

function Hero:CanDetectHiddenNode(node)
	return math.random(1,1) == 1
end

function Hero:IsHiddenTrapDetected(node)
	return math.random(1,1) == 1
end

local function GetLevelFromExperience(xp)
	local Thresholds = require("Hero/HeroLevelThresholds")
	local i = 1
	while Thresholds[i+1] and xp > Thresholds[i+1] do
		i = i + 1
	end
	return i
end

function Hero:GetExperienceData()
	local Thresholds = require("Hero/HeroLevelThresholds")
	local i = 1
	local xp = self:GetAttribute("xp")
	while Thresholds[i+1] do
		if (xp < Thresholds[i+1]) and (xp >= Thresholds[i]) then
			local xp_to_level = Thresholds[i+1] - Thresholds[i]
			local curr_xp_level = self:GetAttribute("xp") - Thresholds[i]
			return { curr_xp = xp, percent_progress = ((curr_xp_level / xp_to_level) * 100), prev_lvl = i, prev_lvl_xp = Thresholds[i], next_lvl = (i+1), next_lvl_xp = Thresholds[i+1], }
		else
			i = i + 1
		end
	end
	return { curr_xp = xp, percent_progress = 100, prev_lvl = i, prev_lvl_xp = Thresholds[i], }
end

function Hero:GetClassForStats()
	return string.upper(self:GetAttribute("class"))
end

function Hero:GetGenderForStats()
	if self:GetAttribute("sex") == "M" then
		return "MALE"
	else
		return "FEMALE"
	end
end

function Hero:AwardExperience(amount)
	Stats.AdjustSPStat("OVERALL", "XP", amount)

	local prev_xp = self:GetAttribute("xp")
	local new_xp = prev_xp + amount
	self:SetAttribute("xp", new_xp)
	local newLevel = GetLevelFromExperience(new_xp)
	local prevLevel = self:GetAttribute("level")
	while prevLevel < newLevel do
		prevLevel = prevLevel + 1
		local currLevel = prevLevel
		self:PushQuestFeedback({ -- this is not shown because it has no subtitle
			modal = true,
			title = "[LEVELUP_HEAD]",
			type="gain",
			action = function ()
				SCREENS.LevelUpMenu:Open()
				SCREENS.LevelUpMenu:SetLevelGainData(self, currLevel)
			end
		})
	end
	self:SetAttribute("level", newLevel)
	self:CalculateStats()

	if IsOpen("WorldMapMenu") then
		SCREENS.WorldMapMenu:set_text("str_level", tostring(self:GetAttribute("level")))
		SCREENS.WorldMapMenu:set_progress("prog_xp", math.floor(self:GetExperienceData().percent_progress))
	end
end

function Hero:AwardGold(amount)
	Stats.AdjustSPStat("OVERALL", "GOLD", amount)

	self:SetAttribute("gold", self:GetAttribute("gold") + amount)
end

function Hero:GetStatBonus(stat_bonus_table, bonus_type)
	assert(stat_bonus_table)
	assert(bonus_type)
	local stat_bonus = 0
	for stat,bonus in pairs(stat_bonus_table) do
		stat_bonus = stat_bonus + self.stats.base[stat][bonus_type] * bonus
	end
	return stat_bonus
end

function Hero:GetStatPointBonus(stat_bonus_table)
	assert(stat_bonus_table)
	return self:GetStatBonus(stat_bonus_table, "points")
end

function Hero:GetStatRankBonus(stat_bonus_table)
	assert(stat_bonus_table)
	return self:GetStatBonus(stat_bonus_table, "rank")
end

function Hero:GetStatPointsRanking(stat_points)
	local stat_ranking = { rank = 0, name = hero_stats_constants.ranks.name }

	for tier, rank_detail in ipairs( hero_stats_constants.ranks ) do
		--print(string.format("Hero:GetStatPointsRanking - stat_points = %d  tier = %d  max_points = %d", stat_points, tier, rank_detail.max_points ))
		if stat_points <= rank_detail.max_points then
			stat_ranking.rank = tier - 1
			stat_ranking.name = rank_detail.name
			--print(string.format("Hero:GetStatPointsRanking - returning rank = %d", tier - 1 ))
			break
		end
	end

	return stat_ranking
end

function Hero:CalculateStatsBaseRank(stat)
	local ranking = self:GetStatPointsRanking( self.stats.base[stat].points )
	self.stats.base[stat].rank = ranking.rank
	self.stats.base[stat].rankname = ranking.name
end

function Hero:CalculateStatsBase(class_bonus_table, level)
	class_bonus_table = class_bonus_table or hero_stats_constants.stat_bonus[string.lower( self:GetAttribute( "class" ) )]
	level = level or self:GetAttribute( "level" )

	for stat,stat_detail in pairs( self.stats.base ) do
        local character_stat =
			self:GetAttribute( stat ) +
			class_bonus_table.start[stat] +
			class_bonus_table.per_level_interval[stat] * math.floor( level / hero_stats_constants.level_bonus_interval )
		local equipment_stat = SumValueThroughItems( self:GetEquippedItems(), "GetStatBuff", stat )
		self.stats.base[stat].points = character_stat + equipment_stat
        if equipment_stat ~= 0 then
            self.stats.base[stat].modified = equipment_stat
        else
            self.stats.base[stat].modified = nil
        end
		self:CalculateStatsBaseRank( stat )
	end
end

function Hero:CalculateStatsHealth(class_base_table, level)
	class_base_table = class_base_table or require( "Hero/" .. self:GetAttribute( "class" ) )
	level = level or self:GetAttribute( "level" )

	self.stats.derived.health.maximum =
		class_base_table.start_data.health  + 
		( class_base_table.hit_points_per_level + self:GetStatRankBonus( hero_stats_constants.stat_bonus.health_per_level ) ) * level +
		SumValueThroughItems( self:GetEquippedItems(), "GetPassiveHealthBuff" )
	self.stats.derived.health.wounds = self:GetAttribute( "wounds" )

	print( string.format( "Hero:CalculateStatsHealth() - health = %s, wounds = %s", self.stats.derived.health.maximum, self.stats.derived.health.wounds ) )
end

function Hero:CalculateStatsMana(class_base_table)
	class_base_table = class_base_table or require( "Hero/" .. self:GetAttribute( "class" ) )

	for pool,value in pairs( self.stats.derived.mana.start ) do
		self.stats.derived.mana.start[pool] = math.floor(
			self:GetStatRankBonus( hero_stats_constants.stat_bonus.mana.start[pool] ) +
			SumValueThroughItems( self:GetEquippedItems(), "GetStartingManaBuff", pool ))
	end

	for pool,value in pairs( self.stats.derived.mana.maximum ) do
		self.stats.derived.mana.maximum[pool] =
			class_base_table.start_data.max_mana[pool] +
			self:GetStatRankBonus( hero_stats_constants.stat_bonus.mana.maximum[pool] ) +
			self.stats.base["intelligence"]["rank"] +
			SumValueThroughItems( self:GetEquippedItems(), "GetMaxManaBuff", pool )
	end
end

function Hero:CalculateStatsMatchBonus()
	for match_type,value in pairs( self.stats.derived.match_bonus ) do
		self.stats.derived.match_bonus[match_type] = self:GetStatRankBonus( hero_stats_constants.stat_bonus.match_score[match_type] )
	end

	local item_attack_bonus = SumValueThroughItems( self:GetEquippedItems(), "GetAttackBonus" )
	local endurance_attack_bonus = self:GetEnduranceAttackBonus()

	self.stats.derived.match_bonus.skull = self.stats.derived.match_bonus.skull + item_attack_bonus + endurance_attack_bonus
	self.stats.derived.match_bonus.skull5 = self.stats.derived.match_bonus.skull5 + item_attack_bonus + endurance_attack_bonus
end

function Hero:CalculateStatsCriticals()
	self.stats.derived.weapon_critical =
		self:GetStatPointBonus( hero_stats_constants.stat_bonus.weapon_critical ) +
		SumValueThroughItems( self:GetEquippedItems(), "GetWeaponCriticalBonus" )
	self.stats.derived.shield_critical =
		self:GetStatPointBonus( hero_stats_constants.stat_bonus.shield_critical ) +
		SumValueThroughItems( self:GetEquippedItems(), "GetShieldCriticalBonus" )
end

function Hero:CalculateStatsMinigameBonusTurns()
	self.stats.derived.minigame_bonus_turns = self:GetStatPointBonus( hero_stats_constants.stat_bonus.minigame_bonus_turns )
end

function Hero:CalculateStatsDerived(class_base_table, level)
	class_base_table = class_base_table or require( "Hero/" .. self:GetAttribute( "class" ) )
	level = level or self:GetAttribute( "level" )

	self:CalculateStatsHealth( class_base_table, level )

	self:CalculateStatsMana( class_base_table )

	self:CalculateStatsMatchBonus()

	self:CalculateStatsCriticals()

	self.stats.derived.weapon_bonus = self:GetStatRankBonus( hero_stats_constants.stat_bonus.weapon_damage )

	self.stats.derived.spell_resist =
		self:GetStatPointBonus( hero_stats_constants.stat_bonus.spell_resist ) +
		SumValueThroughItems( self:GetEquippedItems(), "GetSpellResistanceBonus" )

	self.stats.derived.spell_penetration =
		self:GetStatPointBonus( hero_stats_constants.stat_bonus.spell_penetration ) +
		SumValueThroughItems( self:GetEquippedItems(), "GetSpellPenetrationBonus" )

	self:CalculateStatDefenceDerived()

	self.stats.derived.blocking_effectiveness = hero_stats_constants.blocking_effectiveness_base + SumValueThroughItems( self:GetEquippedItems(), "GetBlockingEffectivenessBonus" )
	
	self:CalculateStatsMinigameBonusTurns()
end

function Hero:CalculateStatDefenceDerived()
	local new_rating = 
		self:GetStatPointBonus( hero_stats_constants.stat_bonus.defence_rating ) +
		SumValueThroughItems( self:GetEquippedItems(), "GetDefenseBonus" )
	
	self.stats.derived.defence_rating = new_rating
	return new_rating
end

-- Hero stats are fixed from battle begin to end; based on class, level, and passive item effects. No active status effect
-- directly changes a stat, they can only change attributes that are derived froms stats (things like hitpoints, weaponcrit
-- chance, spell resist etc.)
function Hero:InitializeStats()
	self.stats = table.clone( require( "Hero/StatsVariable" ) ) -- its a large table, so we keep it separate from our hero source
end

function Hero:CalculateStats(class_base_table, class_bonus_table, level)
	class_base_table = class_base_table or require( "Hero/" .. self:GetAttribute( "class" ) )
	class_bonus_table = class_bonus_table or hero_stats_constants.stat_bonus[string.lower( self:GetAttribute( "class" ) )]
	level = level or self:GetAttribute( "level" )

	self:CalculateStatsBase( class_bonus_table, level )

	self:CalculateStatsDerived( class_base_table, level )
end

-- Here we return a table of state data based on a battle ready hero
function Hero:GetInvariantStats()
	return self.stats
end

function Hero:GetStat(stat_name)
    local stat = self.stats.base[stat_name]
    return stat.points, stat.modified
end

-- Hero: Health
function Hero:GetHealth()
	return self.stats.derived.health.maximum - self.stats.derived.health.wounds
end

-- Hero: MaxHealth
function Hero:GetMaxHealth()
	return self.stats.derived.health.maximum
end

-- Hero: MaxMana
function Hero:GetMaxMana(pool)
	local translated_pool = string.lower( pool )
	return self.stats.derived.mana.maximum[translated_pool]
end

-- Hero: Attack (skull bonus damage)
function Hero:GetAttack()
	return self.stats.derived.match_bonus.skull
end

-- Hero: SetEnduranceAttackBonus (skull bonus damage)
function Hero:SetEnduranceAttackBonus(bonus)
	self.endurance_attack_bonus = self.endurance_attack_bonus or 0
	self.endurance_attack_bonus = self.endurance_attack_bonus + bonus
end

-- Hero: GetEnduranceAttackBonus (skull bonus damage)
function Hero:GetEnduranceAttackBonus()
	return self.endurance_attack_bonus or 0
end

-- Hero: Defence (% chance to half received damage)
function Hero:GetDefence()
	return self.stats.derived.defence_rating
end

-- Hero: WeaponBonusDamage
function Hero:GetWeaponBonusDamage()
	return self.stats.derived.weapon_bonus
end

-- Hero: WeaponCritical
function Hero:GetWeaponCriticalChance()
	return self.stats.derived.weapon_critical
end

-- Hero: ShieldCritical
function Hero:GetShieldCriticalChance()
	return self.stats.derived.shield_critical
end

-- Hero: SpellResist
function Hero:GetSpellResistChance()
	return self.stats.derived.spell_resist
end

-- Hero: SpellPenetration
function Hero:GetSpellPenetration()
	return self.stats.derived.spell_penetration
end

-- Hero: BlockingEffectiveness
function Hero:GetBlockingEffectiveness()
	return self.stats.derived.blocking_effectiveness
end

-- Hero: MinigameBonusTurns
function Hero:GetMinigameBonusTurns()
	return self.stats.derived.minigame_bonus_turns
end

function Hero:GetWounds()
	return self.stats.derived.health.wounds
end

function Hero:ApplyTraitsToHeroState(state)
	-- normally overridden per class
end

function Hero:Cheat_ClearVisitedRooms()
	self:ClearAttributeCollection("visitedRooms")
end

function Hero:GetEnemyGold()
	assert(self.def, "Hero is not an enemy")

	local MINION_LOWER_BRACKET = 50
	local MINION_UPPER_BRACKET = 50
	local BOSS_LOWER_BRACKET   = 150
	local BOSS_UPPER_BRACKET   = 450

	if self.def.gold then
		local minRandom, maxRandom
		local level = self:GetAttribute("level")
		local level_bracket = math.floor((level-1)/10)
		if self.def.boss then
			minRandom = BOSS_LOWER_BRACKET + (level_bracket * BOSS_LOWER_BRACKET)
			maxRandom = BOSS_UPPER_BRACKET + (level_bracket * BOSS_UPPER_BRACKET)
		else
			minRandom = math.max(level_bracket * MINION_LOWER_BRACKET, 1)
			maxRandom = MINION_UPPER_BRACKET + (level_bracket * MINION_UPPER_BRACKET)
		end
		return level + math.random(minRandom, maxRandom)
	end

	return 0
end

function Hero:GetRandomEncounterSessionData(node)
	local sessionData = self.sessionData.randomEncounters[node]

	if not sessionData then
		sessionData = {}
		self.sessionData.randomEncounters[node] = sessionData
	end

	return sessionData
end

function Hero:ResetRandomEncounterSessionData(node)
	if self.sessionData.randomEncounters[node].hero then
		DestroyMonsterHero(self.sessionData.randomEncounters[node].hero)
		self.sessionData.randomEncounters[node].hero = nil
	end
	self.sessionData.randomEncounters[node].defeated = true
end

local monsterSessionDice = cDice()

function Hero:GetSessionDataForEnemy(node, enemyNumber, enemyName)
	local monsterHero = self.sessionData.monsters[node]

	if monsterHero then
		RefreshMoveIntelligence(monsterHero, self:GetAttribute("difficulty"))
		
		return monsterHero
	end

	monsterSessionDice:Seed(self:GetAttribute("seed") + enemyNumber)

	--Generate data
	monsterHero = CreateMonsterHero(enemyName, self, node, monsterSessionDice)
	self.sessionData.monsters[node] = monsterHero
	monsterHero:PrintDebug(true)
	return monsterHero
end

function Hero:ResetSessionDataForEnemy(node)
	if self.sessionData.monsters[node] then
		DestroyMonsterHero(self.sessionData.monsters[node])
		self.sessionData.monsters[node] = nil
	end
end

-- AddSessionLootForNode:
--     When the player leaves items on a corpse (perhaps because they need to free up space
--     by selling items in town) then a container is created in the world to hold the remaining
--     items for when the player returns.  This container is stored in the session data for
--     the hero (so it is not persistent across sessions).
--
--     This method adds any items from "container" (normally a monster hero) to an existing
--     container associated with this node, or a new container if there is no existing corpse
--     at this node.

function Hero:AddSessionLootForNode(node, container, name, portrait)
    assert(type(node) == "string")
    assert(container)
    assert(container:NumAttributes("inventory") > 0)

    local nodeLoot = self.sessionData.loot[node]

    if not nodeLoot then
        assert(type(name) == "string")
        assert(type(portrait) == "string")

        nodeLoot = GameObjectManager:ConstructLocal("LOOT")
        nodeLoot:SetAttribute("name", name)
        nodeLoot:SetAttribute("portrait", portrait)
		
		self:AddChild(nodeLoot)
    end

    while container:NumAttributes("inventory") > 0 do
        local item = container:GetAttributeAt("inventory", 1)
        if item:IsLootable() then
        	nodeLoot:AddItem(container:RemoveItem(item))
        else
        	container:RemoveItem(item)
        end
    end

    nodeLoot:SetAttribute("gold", nodeLoot:GetAttribute("gold") + container:GetAttribute("gold"))

    self.sessionData.loot[node] = nodeLoot
end

-- GetSessionLootForNode:
--     Returns the container holding loot from a corpse if there is a corpse at this node,
--     otherwise it returns nil.  See AddSessionLootForNode for more info.

function Hero:GetSessionLootForNode(node)
    return self.sessionData.loot[node]
end

-- ResetSessionLootForNode:
--     Destroys the container/corpse for a node.  You would normally do this when there were
--     no more items on the corpse/container.  See AddSessionLootForNode for more info.

function Hero:ResetSessionLootForNode(node)
    local container = self.sessionData.loot[node]

    if container then
        GameObjectManager:Destroy(container)
    end

	self.sessionData.loot[node] = nil
end

function Hero:GetShopSessionData(node)
	assert(type(node) == "string")
	
	return self.sessionData.shops[node]
end

function Hero:SetShopSessionData(node, items)
	assert(type(node) == "string")
	assert(type(items) == "table")
	
	self:ResetShopSessionData(node)
	
	local item_data = {}
	for _, item in ipairs(items) do
		if item.IS_TRADE then
			table.insert(item_data, {kind = "trade", resource = item:GetAttribute("resource"), stack = item:GetAttribute("stacksize")})
		elseif item.IS_EQUIPPABLE then
			table.insert(item_data, {kind = "equip", baseType = item:GetBaseType(), race = item:GetRace(), rarity = item:GetRarity()})
		end
	end
	
	self.sessionData.shops[node] = item_data
	
	table.log(item_data)
end

function Hero:ResetShopSessionData(node)
	assert(type(node) == "string")
	
	self.sessionData.shops[node] = nil
end

function Hero:ClearAllShopSessionData()
	local shops = self.sessionData.shops

	if shops then
		for node_id in pairs(shops) do
			self:ResetShopSessionData(node_id)
		end
	end
end

-- GetPassiveItems
--    Returns all the hero's equipped passive items (helm, shoes, armour, jewellery)
--    as a table.
function Hero:GetPassiveItemsInOrder()
	local passive = {}

	if self:GetAttribute("helmItem") then
		table.insert(passive, 1, self:GetAttribute("helmItem"))
	end
	if self:GetAttribute("jewelleryItem") then
		table.insert(passive, 2, self:GetAttribute("jewelleryItem"))
	end
	if self:GetAttribute("armourItem") then
		table.insert(passive, 3, self:GetAttribute("armourItem"))
	end
	if self:GetAttribute("feetItem") then
		table.insert(passive, 4, self:GetAttribute("feetItem"))
	end

	return passive
end

function Hero:GetPassiveItems()
	local armour = { }
	if self:GetAttribute("helmItem") then
		table.insert(armour, self:GetAttribute("helmItem"))
	end
	if self:GetAttribute("armourItem") then
		table.insert(armour, self:GetAttribute("armourItem"))
	end
	if self:GetAttribute("jewelleryItem") then
		table.insert(armour, self:GetAttribute("jewelleryItem"))
	end
	if self:GetAttribute("feetItem") then
		table.insert(armour, self:GetAttribute("feetItem"))
	end

	return armour
end

function Hero:GetEquippedItems()
	local items = self:GetPassiveItems()
	if self:GetAttribute("mainHandItem") then
		table.insert(items, self:GetAttribute("mainHandItem"))
	end
	if self:GetAttribute("offHandItem") then
		table.insert(items, self:GetAttribute("offHandItem"))
	end
	return items
end

function Hero:ApplyItemMagicalEffects(event, params, ...)
	if self:GetAttribute("mainHandItem") then
		self:GetAttribute("mainHandItem"):ApplyMagicalEffects(event, params, ...)
	end
	if self:GetAttribute("offHandItem") then
		self:GetAttribute("offHandItem"):ApplyMagicalEffects(event, params, ...)
	end
	if self:GetAttribute("helmItem") then
		self:GetAttribute("helmItem"):ApplyMagicalEffects(event, params, ...)
	end
	if self:GetAttribute("armourItem") then
		self:GetAttribute("armourItem"):ApplyMagicalEffects(event, params, ...)
	end
	if self:GetAttribute("jewelleryItem") then
		self:GetAttribute("jewelleryItem"):ApplyMagicalEffects(event, params, ...)
	end
	if self:GetAttribute("feetItem") then
		self:GetAttribute("feetItem"):ApplyMagicalEffects(event, params, ...)
	end
end

function Hero:GetSpells()
	local spells = CollectionToTable(self, "activeSpells")
	
	return table.clone(spells)
end

-- EquipItem
--    Equips an item into the specified slot. If no slot is specified, the first valid
--    slot found is used. Any item already in the slot is automatically unequipped.
function Hero:EquipItem(item, slot)
	self:UnequipItem(item)
	if not slot then
		if item:IsCarriedInHand() then
			if item:IsTwoHanded() then
				slot = "bothHands"
			elseif item:IsEquippableInMainHand() then
				slot = "mainHand"
			elseif item:IsEquippableInOffHand() then
				slot = "offHand"
			end
		elseif item:IsHelm() then
			slot = "helm"
		elseif item:IsArmour() then
			slot = "armour"
		elseif item:IsShoes() then
			slot = "feet"
		elseif item:IsJewellery() then
			slot = "jewellery"
		end
	end

	if slot == "bothHands" then
		self:UnequipItem(self:GetAttribute("mainHandItem"), true)
		self:UnequipItem(self:GetAttribute("offHandItem"), true)
		self:SetAttribute("mainHandItem", item)
		self:SetAttribute("offHandItem", item)
	elseif slot then
		self:UnequipItem(self:GetAttribute(string.format("%sItem", slot)), true)
		self:SetAttribute(string.format("%sItem", slot), item)
	end
	
	self:CalculateStats()
end

function Hero:AutoEquipItem(item)
	if self:CanEquipItem(item) then
		if item:IsCarriedInHand() then
			if item:IsTwoHanded() then
				if not self:GetAttribute("mainHandItem") and not self:GetAttribute("offHandItem") then
					self:EquipItem(item, "bothHands")
				end
			elseif item:IsEquippableInMainHand() then
				if not self:GetAttribute("mainHandItem") then
					self:EquipItem(item, "mainHand")
				end
			elseif item:IsEquippableInOffHand() then
				if not self:GetAttribute("offHandItem") then
					self:EquipItem(item, "offHand")
				end
			end
		elseif item:IsHelm() then
			if not self:GetAttribute("helmItem") then
				self:EquipItem(item, "helm")
			end
		elseif item:IsArmour() then
			if not self:GetAttribute("armourItem") then
				self:EquipItem(item, "armour")
			end
		elseif item:IsShoes() then
			if not self:GetAttribute("feetItem") then
				self:EquipItem(item, "feet")
			end
		elseif item:IsJewellery() then
			if not self:GetAttribute("jewelleryItem") then
				self:EquipItem(item, "jewellery")
			end
		end
	end
end

function Hero:AddAndEquipItem(item, slot)
	if not CollectionContainsAttribute(self, "inventory", item) then
		self:PushAttribute("inventory", item)
	end
	return self:EquipItem(item, slot)
end

function Hero:UnequipItem(item, dont_calc_stats)
	if not item then
		return
	end

	if self:GetAttribute("mainHandItem") == item then
		self:SetAttribute("mainHandItem", nil)
		if item:IsTwoHanded() then
			self:SetAttribute("offHandItem", nil)
		end
	end

	if self:GetAttribute("offHandItem") == item then
		self:SetAttribute("offHandItem", nil)
		if item:IsTwoHanded() then
			self:SetAttribute("mainHandItem", nil)
		end
	end

	if self:GetAttribute("helmItem") == item then
		self:SetAttribute("helmItem", nil)
	end
	if self:GetAttribute("armourItem") == item then
		self:SetAttribute("armourItem", nil)
	end
	if self:GetAttribute("jewelleryItem") == item then
		self:SetAttribute("jewelleryItem", nil)
	end
	if self:GetAttribute("feetItem") == item then
		self:SetAttribute("feetItem", nil)
	end
	
	if dont_calc_stats == true then
		return
	end
	
	self:CalculateStats()
end

function Hero:IsItemEquipped(item)
	if self:GetAttribute("mainHandItem")  == item or
	   self:GetAttribute("offHandItem")   == item or
		self:GetAttribute("helmItem")      == item or
		self:GetAttribute("armourItem")    == item or
		self:GetAttribute("jewelleryItem") == item or
		self:GetAttribute("feetItem")      == item then
		return true
	end
	return false
end

function Hero:CanEquipItem(item)
	if item then
		return item:CanClassEquip(self:GetAttribute("class"))
	end
end

function Hero:GetActiveQuest()
	local activeQuest = self:GetAttribute("activeQuest")
	if activeQuest > 0 then
		return activeQuest
	end
end

function Hero:SetActiveQuest(index)
	LOG("QUEST: SetActiveQuest(%s)", tostring(index))
	self:SetAttribute("activeQuest", assert(index))

	if IsOpen("WorldMapMenu") then
		SCREENS.WorldMapMenu:ShowActiveQuest(self, index)
	end
end

function Hero:SetMinigamePlayed(game)
	LOGF("Hero:SetMinigamePlayed(%s)", game)

	local attributeName = string.format("%sGameCount", string.lower(game))

	self:SetAttribute(attributeName, self:GetAttribute(attributeName) + 1)
end

local function COPY(dest, source, name)
	assert(dest):SetAttribute(name, assert(source):GetAttribute(name))
end

local function COPY_COLLECTION(dest, source, name)
	assert(dest)
	assert(source)
	assert(name)

	dest:ClearAttributeCollection(name)

	for i=1,source:NumAttributes(name) do
		dest:PushAttribute(name, source:GetAttributeAt(name, i))
	end
end

function Hero:LoadFromEvent(heroEvent)
	assert(heroEvent)
	print( "Hero:LoadFromEvent()" )

	COPY(self, heroEvent, "name")
	COPY(self, heroEvent, "class")
	COPY(self, heroEvent, "level")
	COPY(self, heroEvent, "portrait")
	COPY(self, heroEvent, "sex")

	COPY(self, heroEvent, "gold")
	COPY(self, heroEvent, "xp")

	COPY(self, heroEvent, "strength")
	COPY(self, heroEvent, "agility")
	COPY(self, heroEvent, "stamina")
	COPY(self, heroEvent, "intelligence")
	COPY(self, heroEvent, "morale")
	LOGF("strength = %d", self:GetAttribute("strength"))
	LOGF("agility = %d", self:GetAttribute("agility"))
	LOGF("stamina = %d", self:GetAttribute("stamina"))
	LOGF("intelligence = %d", self:GetAttribute("intelligence"))
	LOGF("morale = %d", self:GetAttribute("morale"))
	
	
	COPY_COLLECTION(self, heroEvent, "activeSpells")
	COPY_COLLECTION(self, heroEvent, "traits")
	COPY_COLLECTION(self, heroEvent, "knownSpells")

	self:CalculateStats()
	print( "Hero:LoadFromEvent() end" )
end

-- This does not create an event that describes a fully calculated and consistently valid hero (in terms of stats), all it does it create
-- an event containing the minimal data for a 'presentable' hero which can be used by the UI to be viewed on screen
function Hero:SaveToEvent(heroEvent)
	assert(heroEvent)
	print( "Hero:SaveToEvent() begin" )

	COPY(heroEvent, self, "name")
	COPY(heroEvent, self, "class")
	COPY(heroEvent, self, "level")
	COPY(heroEvent, self, "portrait")
	COPY(heroEvent, self, "sex")

	COPY(heroEvent, self, "gold")
	COPY(heroEvent, self, "xp")

	COPY(heroEvent, self, "strength")
	COPY(heroEvent, self, "agility")
	COPY(heroEvent, self, "stamina")
	COPY(heroEvent, self, "intelligence")
	COPY(heroEvent, self, "morale")

	COPY_COLLECTION(heroEvent, self, "activeSpells")
	COPY_COLLECTION(heroEvent, self, "traits")
	COPY_COLLECTION(heroEvent, self, "knownSpells")

	print( "Hero:SaveToEvent() end" )
end

dofile("Assets/Scripts/Hero/Hero_Quest.lua")

return ExportClass("Hero")
