local ItemConfiguration = {}

ItemConfiguration.RACE_LEVEL_LIMITS = 
{
	Standard						= {min=1,	max=35},
	Orcish							= {min=5,	max=50},
	BlackIron						= {min=11,	max=50},
	DarkElven						= {min=17,	max=50},
	Dwarven							= {min=24,	max=50},
	Elven							= {min=30,	max=50},
	Demonic							= {min=36,	max=50},
}

ItemConfiguration.RARITY_LEVEL_LIMITS = 
{
	Normal							= {min=1,	max=35},
	Magical							= {min=5,	max=40},
	Master							= {min=11,	max=45},
	Renowned						= {min=17,	max=50},
	Legendary						= {min=24,	max=50},
}

ItemConfiguration.BASETYPE_LEVEL_LIMITS = 
{
	-- WEAPONS
	Axe								= {min=5,	max=50},
	BastardSword					= {min=1,	max=35},
	BattleAxe						= {min=6,	max=50},
	ClawedGlove						= {min=10,	max=50},
	Claymore						= {min=7,	max=50},
	CompositeBow					= {min=17,	max=50},
	CrusadeSword					= {min=6,	max=50},
	Dagger							= {min=5,	max=50},
	Daikatana						= {min=11,	max=50},
	Falcion							= {min=17,	max=50},
	FlintlockPistol					= {min=21,	max=50},
	GreatAxe						= {min=12,	max=50},
	GreatClub						= {min=6,	max=50},
	GreatHammer						= {min=24,	max=50},
	Greatsword						= {min=19,	max=50},
	Halberd							= {min=18,	max=50},
	HolySword						= {min=29,	max=50},
	Katana							= {min=22,	max=50},
	Longbow							= {min=11,  max=50},
	LongSword						= {min=17,	max=50},
	Mace							= {min=5,	max=50},
	Morningstar						= {min=17,	max=50},
	Ninjato							= {min=4,	max=50},
	Rapier							= {min=5,	max=50},
	Scimitar						= {min=16,	max=50},
	Shortbow						= {min=10,	max=50},
	ShortSword						= {min=1,	max=35},
	SpellStaff						= {min=6,	max=50},
	ThrowingKnife					= {min=3,	max=35},
	Warhammer						= {min=12,	max=50},
	WitchBlade						= {min=23,	max=50},
	Zaltir							= {min=12,	max=50},
	Zweihander						= {min=11,	max=50},
	
	-- SHIELDS
	Buckler							= {min=5,	max=30},
	RoundShield						= {min=11,	max=50},
	MainGauche						= {min=17,	max=50},
	KiteShield						= {min=24,	max=50},
	TowerShield						= {min=30,	max=50},
	
	-- POTIONS
	MinorHealingPotion				= {min=5,	max=50},
	RegularHealingPotion			= {min=17,	max=50},
	MajorHealingPotion				= {min=30,	max=50},
	
	RedManaPotion					= {min=10,	max=50},
	BlueManaPotion					= {min=10,	max=50},
	GreenManaPotion					= {min=10,	max=50},
	YellowManaPotion				= {min=10,	max=50},
	BlackManaPotion					= {min=10,	max=50},
	
	RedManaAccelerationPotion		= {min=5,	max=50},
	BlueManaAccelerationPotion		= {min=5,	max=50},
	GreenManaAccelerationPotion		= {min=5,	max=50},
	YellowManaAccelerationPotion	= {min=5,	max=50},
	BlackManaAccelerationPotion		= {min=5,	max=50},
	
	-- POISONS
	ArachnaPoison					= {min=5,	max=50},
	ManticorePoison					= {min=17,	max=50},
	WyvernPoison					= {min=30,	max=50},
	
	-- ARMOURS
	ClothRobes						= {min=1,	max=35},
	PaddedClothArmour				= {min=5,	max=45},
	LeatherArmour					= {min=11,	max=50},
	HardenedLeatherArmour			= {min=17,	max=50},
	ChainmailArmour					= {min=17,	max=50},
	ScalemailArmour					= {min=24,	max=50},
	PlatemailArmour					= {min=30,	max=50},
	HideArmour						= {min=1,	max=50},
	
	-- BOOTS
	PaddedClothBoots				= {min=1,	max=35},
	LeatherBoots					= {min=5,	max=50},
	HardenedLeatherBoots			= {min=11,	max=50},
	ChainmailBoots					= {min=17,	max=50},
	ScalemailBoots					= {min=24,	max=50},
	PlateBoots						= {min=30,	max=50},
	
	-- HELMETS
	PaddedClothHelmet				= {min=1,	max=35},
	LeatherHelmet					= {min=5,	max=50},
	HardenedLeatherHelmet			= {min=11,	max=50},
	ChainmailHelmet					= {min=17,	max=50},
	ScalemailHelmet					= {min=24,	max=50},
	PlateHelmet						= {min=30,	max=50},
	
	-- JEWELLERY
	ArcaneNecklace					= {min=5,	max=50},
	LuminousNecklace				= {min=5,	max=50},
	StatRing						= {min=5,	max=50},
}

-- How each base type is held:
--   "either": 	single handed, can be held in either hand
--   "main": 	single handed, must be held in main hand
--   "off": 	off handed, must be held in off hand
--   "both": 	double handed, must be held in both hands
--   "armour":	assigned into armour slot
--   "helm":	assigned into helm slot
--   "boots":	assigned into boot slot
--   "misc":	assigned into jewellery slot
-- These values are for the generator ONLY.  

ItemConfiguration.BASETYPE_SLOT_LIMITS = 
{
	Axe								= "main",
	BastardSword					= "both",
	BattleAxe						= "both",
	ClawedGlove						= "either",
	Claymore						= "both",
	CompositeBow					= "both",
	CrusadeSword					= "main",
	Dagger							= "either",
	Daikatana						= "main",
	Falcion							= "main",
	FlintlockPistol					= "either",
	GreatAxe						= "both",
	GreatClub						= "both",
	GreatHammer						= "both",
	Greatsword						= "both",
	Halberd							= "both",
	HolySword						= "main",
	Katana							= "either",
	Longbow							= "both",
	LongSword						= "main",
	Mace							= "main",
	Morningstar						= "main",
	Ninjato							= "either",
	Rapier							= "main",
	Scimitar						= "either",
	Shortbow						= "both",
	ShortSword						= "main",
	SpellStaff						= "main",
	ThrowingKnife					= "either",
	Warhammer						= "both",
	WitchBlade						= "main",
	Zaltir							= "both",
	Zweihander						= "main",
	
	-- SHIELDS
	Buckler							= "off",
	RoundShield						= "off",
	KiteShield						= "off",
	TowerShield						= "off",
	MainGauche						= "off",
	
	-- POTIONS
	MinorHealingPotion				= "either",
	RegularHealingPotion			= "either",
	MajorHealingPotion				= "either",
	
	RedManaPotion					= "either",
	BlueManaPotion					= "either",
	GreenManaPotion					= "either",
	YellowManaPotion				= "either",
	BlackManaPotion					= "either",
	
	RedManaAccelerationPotion		= "either",
	BlueManaAccelerationPotion		= "either",
	GreenManaAccelerationPotion		= "either",
	YellowManaAccelerationPotion	= "either",
	BlackManaAccelerationPotion		= "either",
	
	-- POISONS
	ArachnaPoison					= "either",
	ManticorePoison					= "either",
	WyvernPoison					= "either",

	-- ARMOURS
	ClothRobes						= "armour",
	PaddedClothArmour				= "armour",
	LeatherArmour					= "armour",
	HardenedLeatherArmour			= "armour",
	ChainmailArmour					= "armour",
	ScalemailArmour					= "armour",
	PlatemailArmour					= "armour",
	HideArmour						= "armour",
	
	-- BOOTS
	PaddedClothBoots				= "boots",
	LeatherBoots					= "boots",
	HardenedLeatherBoots			= "boots",
	ChainmailBoots					= "boots",
	ScalemailBoots					= "boots",
	PlateBoots						= "boots",
	
	-- HELMETS
	PaddedClothHelmet				= "helm",
	LeatherHelmet					= "helm",
	HardenedLeatherHelmet			= "helm",
	ChainmailHelmet					= "helm",
	ScalemailHelmet					= "helm",
	PlateHelmet						= "helm",
	
	-- JEWELLERY (NECKLACES/RINGS)
	ArcaneNecklace					= "misc",
	LuminousNecklace				= "misc",
	StatRing						= "misc",
}

-- How each base type is held:
--   "either": 	single handed, can be held in either hand
--   "main": 	single handed, must be held in main hand
--   "off": 	off handed, must be held in off hand
--   "both": 	double handed, must be held in both hands
--   "armour":	assigned into armour slot
--   "helm":	assigned into helm slot
--   "boots":	assigned into boot slot
--   "misc":	assigned into jewellery slot
-- These values are for the generator ONLY.  

ItemConfiguration.BASETYPE_CLASS_LIMITS = 
{
	Axe								= "None",
	BastardSword					= "None",
	BattleAxe						= "None",
	ClawedGlove						= "None",
	Claymore						= "None",
	CompositeBow					= "None",
	CrusadeSword					= "None",
	Dagger							= "None",
	Daikatana						= "None",
	Falcion							= "None",
	FlintlockPistol					= "None",
	GreatAxe						= "Barbarian",
	GreatClub						= "Barbarian",
	GreatHammer						= "Barbarian",
	Greatsword						= "Barbarian",
	Halberd							= "None",
	HolySword						= "Inquisitor",
	Katana							= "Assassin",
	Longbow							= "None",
	LongSword						= "None",
	Mace							= "None",
	Morningstar						= "None",
	Ninjato							= "Assassin",
	Rapier							= "None",
	Scimitar						= "None",
	Shortbow						= "None",
	ShortSword						= "None",
	SpellStaff						= "WarMage",
	ThrowingKnife					= "None",
	Warhammer						= "None",
	WitchBlade						= "None",
	Zaltir							= "Barbarian",
	Zweihander						= "None",
	
	-- SHIELDS
	Buckler							= "None",
	RoundShield						= "None",
	KiteShield						= "None",
	TowerShield						= "Inquisitor",
	MainGauche						= "None",
	
	-- POTIONS
	MinorHealingPotion				= "None",
	RegularHealingPotion			= "None",
	MajorHealingPotion				= "None",
	
	RedManaPotion					= "None",
	BlueManaPotion					= "None",
	GreenManaPotion					= "None",
	YellowManaPotion				= "None",
	BlackManaPotion					= "None",
	
	RedManaAccelerationPotion		= "WarMage",
	BlueManaAccelerationPotion		= "WarMage",
	GreenManaAccelerationPotion		= "WarMage",
	YellowManaAccelerationPotion	= "WarMage",
	BlackManaAccelerationPotion		= "WarMage",
	
	-- POISONS
	ArachnaPoison					= "None",
	ManticorePoison					= "Assassin",
	WyvernPoison					= "Assassin",
	
	-- ARMOURS
	ClothRobes						= "None",
	PaddedClothArmour				= "None",
	LeatherArmour					= "None",
	HardenedLeatherArmour			= "None",
	ChainmailArmour					= "None",
	ScalemailArmour					= "None",
	PlatemailArmour					= "Inquisitor",
	HideArmour						= "None",
	
	-- BOOTS
	PaddedClothBoots				= "None",
	LeatherBoots					= "None",
	HardenedLeatherBoots			= "None",
	ChainmailBoots					= "None",
	ScalemailBoots					= "None",
	PlateBoots						= "Inquisitor",
	
	-- HELMETS
	PaddedClothHelmet				= "None",
	LeatherHelmet					= "None",
	HardenedLeatherHelmet			= "None",
	ChainmailHelmet					= "None",
	ScalemailHelmet					= "None",
	PlateHelmet						= "Inquisitor",
	
	-- JEWELLERY (NECKLACES/RINGS)
	ArcaneNecklace					= "None",
	LuminousNecklace				= "None",
	StatRing						= "None",
}

ItemConfiguration.RACE_PROBABILITIES = 
{
	Standard						= 1,
	Orcish							= 1,
	BlackIron						= 1,
	DarkElven						= 1,
	Dwarven							= 1,
	Elven							= 1,
	Demonic							= 1,
}

ItemConfiguration.RARITY_PROBABILITIES = 
{
	Normal							= 2500,
	Magical							= 500,
	Master							= 100,
	Renowned						= 10,
	Legendary						= 1,
}

ItemConfiguration.RARITY_RARECHEST_PROBABILITIES = 
{
	Normal							= 0,
	Magical							= 20,
	Master							= 10,
	Renowned						= 5,
	Legendary						= 1,
}

ItemConfiguration.BASETYPE_PROBABILITIES = 
{
	Axe								= 1,
	BastardSword					= 1,
	BattleAxe						= 1,
	ClawedGlove						= 1,
	Claymore						= 1,
	CompositeBow					= 1,
	CrusadeSword					= 1,
	Dagger							= 1,
	Daikatana						= 1,
	Falcion							= 1,
	FlintlockPistol					= 1,
	GreatAxe						= 1,
	GreatClub						= 1,
	GreatHammer						= 1,
	Greatsword						= 1,
	Halberd							= 1,
	HolySword						= 1,
	Katana							= 1,
	Longbow							= 1,
	LongSword						= 1,
	Mace							= 1,
	Morningstar						= 1,
	Ninjato							= 1,
	Rapier							= 1,
	Scimitar						= 1,
	Shortbow						= 1,
	ShortSword						= 1,
	SpellStaff						= 1,
	ThrowingKnife					= 1,
	Warhammer						= 1,
	WitchBlade						= 1,
	Zaltir							= 1,
	Zweihander						= 1,
	
	-- SHIELDS
	Buckler							= 1,
	RoundShield						= 1,
	KiteShield						= 1,
	TowerShield						= 1,
	MainGauche						= 1,
	
	-- POTIONS
	MinorHealingPotion				= 1,
	RegularHealingPotion			= 1,
	MajorHealingPotion				= 1,
	
	RedManaPotion					= 1,
	BlueManaPotion					= 1,
	GreenManaPotion					= 1,
	YellowManaPotion				= 1,
	BlackManaPotion					= 1,
	
	RedManaAccelerationPotion		= 1,
	BlueManaAccelerationPotion		= 1,
	GreenManaAccelerationPotion		= 1,
	YellowManaAccelerationPotion	= 1,
	BlackManaAccelerationPotion		= 1,
	
	-- POISONS
	ArachnaPoison					= 1,
	ManticorePoison					= 1,
	WyvernPoison					= 1,
	
	-- ARMOURS
	ClothRobes						= 1,
	PaddedClothArmour				= 1,
	LeatherArmour					= 1,
	HardenedLeatherArmour			= 1,
	ChainmailArmour					= 1,
	ScalemailArmour					= 1,
	PlatemailArmour					= 1,
	HideArmour						= 1,
	
	-- BOOTS
	PaddedClothBoots				= 1,
	LeatherBoots					= 1,
	HardenedLeatherBoots			= 1,
	ChainmailBoots					= 1,
	ScalemailBoots					= 1,
	PlateBoots						= 1,
	
	-- HELMETS
	PaddedClothHelmet				= 1,
	LeatherHelmet					= 1,
	HardenedLeatherHelmet			= 1,
	ChainmailHelmet					= 1,
	ScalemailHelmet					= 1,
	PlateHelmet						= 1,
	
	-- JEWELLERY (NECKLACES/RINGS)
	ArcaneNecklace					= 1,
	LuminousNecklace				= 1,
	StatRing						= 1,
}

ItemConfiguration.BASETYPE_RARITY_COSTS = 
{
	-- WEAPONS
	Axe								= {"metal",	"amber"},
	BastardSword					= {"metal",	"amber"},
	BattleAxe						= {"metal",	"emerald"},
	ClawedGlove						= {"metal",	"leather",	"emerald"},
	Claymore						= {"metal",	"amber"},
	CompositeBow					= {"wood",	"leather",	"emerald"},
	CrusadeSword					= {"metal",	"amber"},
	Dagger							= {"metal",	"amber"},
	Daikatana						= {"wood",	"leather",	"emerald"},
	Falcion							= {"wood",	"leather",	"sapphire"},
	FlintlockPistol					= {"wood",	"metal",	"leather",	"sapphire"},
	GreatAxe						= {"wood",	"metal",	"sapphire"},
	GreatClub						= {"wood",	"amber"},
	GreatHammer						= {"wood",	"metal",	"leather",	"sapphire"},
	Greatsword						= {"metal",	"leather",	"sapphire"},
	Halberd							= {"wood",	"metal",	"sapphire"},
	HolySword						= {"wood",	"metal",	"leather",	"sapphire"},
	Katana							= {"wood",	"metal",	"leather",	"sapphire"},
	Longbow							= {"wood",	"leather",	"emerald"},
	LongSword						= {"metal",	"leather",	"emerald"},
	Mace							= {"metal",	"amber"},
	Morningstar						= {"wood",	"metal",	"emerald"},
	Ninjato							= {"metal",	"amber"},
	Rapier							= {"metal",	"emerald"},
	Scimitar						= {"metal",	"leather",	"emerald"},
	Shortbow						= {"wood",	"amber"},
	ShortSword						= {"metal",	"amber"},
	SpellStaff						= {"wood",	"sapphire"},
	ThrowingKnife					= {"metal",	"amber"},
	Warhammer						= {"wood",	"metal",	"emerald"},
	WitchBlade						= {"wood",	"metal",	"leather",	"sapphire"},
	Zaltir							= {"metal",	"leather",	"emerald"},
	Zweihander						= {"wood",	"metal",	"emerald"},
	
	-- SHIELDS
	Buckler							= {"wood",	"pearl"},
	RoundShield						= {"wood",	"leather",	"pearl"},
	KiteShield						= {"wood",	"metal",	"leather",	"pearl"},
	TowerShield						= {"wood",	"metal",	"leather",	"pearl"},
	MainGauche						= {"wood",	"metal",	"pearl"},
	
	-- POTIONS
	MinorHealingPotion				= {"fang"},
	RegularHealingPotion			= {"fang"},
	MajorHealingPotion				= {"fang"},
	
	RedManaPotion					= {"fang"},
	BlueManaPotion					= {"fang"},
	GreenManaPotion					= {"fang"},
	YellowManaPotion				= {"fang"},
	BlackManaPotion					= {"fang"},
	
	RedManaAccelerationPotion		= {"fang"},
	BlueManaAccelerationPotion		= {"fang"},
	GreenManaAccelerationPotion		= {"fang"},
	YellowManaAccelerationPotion	= {"fang"},
	BlackManaAccelerationPotion		= {"fang"},
	
	-- POISONS
	ArachnaPoison					= {"fang"},
	ManticorePoison					= {"fang"},
	WyvernPoison					= {"fang"},
	
	-- ARMOURS
	ClothRobes						= {"leather",	"ruby"},
	PaddedClothArmour				= {"leather",	"ruby"},
	LeatherArmour					= {"metal",	"leather",	"ruby"},
	HardenedLeatherArmour			= {"metal",	"leather",	"ruby"},
	ChainmailArmour					= {"wood",	"metal",	"ruby"},
	ScalemailArmour					= {"wood",	"metal",	"leather",	"ruby"},
	PlatemailArmour					= {"wood",	"metal",	"leather",	"ruby"},
	HideArmour						= {"wood",	"leather",	"ruby"},
	
	-- BOOTS
	PaddedClothBoots				= {"leather",	"crystal"},
	LeatherBoots					= {"leather",	"crystal"},
	HardenedLeatherBoots			= {"metal",	"leather",	"crystal"},
	ChainmailBoots					= {"wood",	"metal",	"crystal"},
	ScalemailBoots					= {"wood",	"metal",	"leather",	"crystal"},
	PlateBoots						= {"wood",	"metal",	"leather",	"crystal"},
	
	-- HELMETS
	PaddedClothHelmet				= {"leather",	"crystal"},
	LeatherHelmet					= {"leather",	"crystal"},
	HardenedLeatherHelmet			= {"metal",	"leather",	"crystal"},
	ChainmailHelmet					= {"wood",	"metal",	"crystal"},
	ScalemailHelmet					= {"wood",	"metal",	"leather",	"crystal"},
	PlateHelmet						= {"wood",	"metal",	"leather",	"crystal"},
	
	-- JEWELLERY (NECKLACES/RINGS)
	ArcaneNecklace					= {"metal",	"crystal"},
	LuminousNecklace				= {"metal",	"crystal"},
	StatRing						= {"metal",	"crystal"},
}

ItemConfiguration.RARITY_UPGRADE_COMPONENT_MULTIPLIER = 
{
	Normal							= 1,
	Magical							= 1,
	Master							= 2,
	Renowned						= 5,
	Legendary						= 10,
}

ItemConfiguration.RARITY_COSTS_MULTIPLIER = 
{
	Normal							= 1.00,
	Magical							= 1.05,
	Master							= 1.10,
	Renowned						= 1.50,
	Legendary						= 1.90,
}

ItemConfiguration.RACE_COSTS_MULTIPLIER = 
{
	Standard						= 1.00,
	Orcish							= 1.05,
	BlackIron						= 1.10,
	DarkElven						= 1.25,
	Dwarven							= 1.65,
	Elven							= 2.20,
	Demonic							= 2.50,
}

ItemConfiguration.BASETYPE_VALUES = 
{
	Axe								= 350,
	BastardSword					= 100,
	BattleAxe						= 350,
	ClawedGlove						= 700,
	Claymore						= 350,
	CompositeBow					= 1700,
	CrusadeSword					= 350,
	Dagger							= 350,
	Daikatana						= 700,
	Falcion							= 1700,
	FlintlockPistol					= 4200,
	GreatAxe						= 700,
	GreatClub						= 350,
	GreatHammer						= 4200,
	Greatsword						= 1700,
	Halberd							= 1700,
	HolySword						= 6500,
	Katana							= 4200,
	Longbow							= 700,
	LongSword						= 1700,
	Mace							= 350,
	Morningstar						= 1700,
	Ninjato							= 350,
	Rapier							= 350,
	Scimitar						= 1700,
	Shortbow						= 700,
	ShortSword						= 100,
	SpellStaff						= 100,
	ThrowingKnife					= 350,
	Warhammer						= 700,
	WitchBlade						= 4200,
	Zaltir							= 700,
	Zweihander						= 700,
	
	-- SHIELDS
	Buckler							= 400,
	RoundShield						= 750,
	MainGauche						= 1750,
	KiteShield						= 4250,
	TowerShield						= 6550,
	
	-- POTIONS
	MinorHealingPotion				= 350,
	RegularHealingPotion			= 1700,
	MajorHealingPotion				= 6500,
	
	RedManaPotion					= 700,
	BlueManaPotion					= 700,
	GreenManaPotion					= 700,
	YellowManaPotion				= 700,
	BlackManaPotion					= 700,
	
	RedManaAccelerationPotion		= 350,
	BlueManaAccelerationPotion		= 350,
	GreenManaAccelerationPotion		= 350,
	YellowManaAccelerationPotion	= 350,
	BlackManaAccelerationPotion		= 350,
	
	-- POISONS
	ArachnaPoison					= 430,
	ManticorePoison					= 1780,
	WyvernPoison					= 6580,
	
	-- ARMOURS
	ClothRobes						= 150,
	PaddedClothArmour				= 400,
	LeatherArmour					= 750,
	HardenedLeatherArmour			= 1750,
	ChainmailArmour					= 1750,
	ScalemailArmour					= 4250,
	PlatemailArmour					= 6550,
	HideArmour						= 750,
	
	-- BOOTS
	PaddedClothBoots				= 140,
	LeatherBoots					= 390,
	HardenedLeatherBoots			= 740,
	ChainmailBoots					= 1740,
	ScalemailBoots					= 4240,
	PlateBoots						= 6540,
	
	-- HELMETS
	PaddedClothHelmet				= 115,
	LeatherHelmet					= 365,
	HardenedLeatherHelmet			= 715,
	ChainmailHelmet					= 1715,
	ScalemailHelmet					= 4215,
	PlateHelmet						= 6515,
	
	-- JEWELLERY (NECKLACES/RINGS)
	ArcaneNecklace					= 470,
	LuminousNecklace				= 470,
	StatRing						= 470,
}

ItemConfiguration.CATALYST_ITEMS 	= {
	"amber",
	"pearl",
	"emerald",
	"ruby",
	"sapphire",
	"fang",
	"crystal",
}

ItemConfiguration.TRADE_DISTRIBUTION = {
	fang = 64,
	amber = 32,
	pearl = 16,
	crystal = 8,
	emerald = 4,
	ruby = 2,
	sapphire = 1,
	wood = 40,
	metal = 40,
	leather = 40,
}

return ItemConfiguration
