use_safeglobals()

local DEFAULT_MUSIC_FADE_DOWN_TIME = 2000 			-- in milliseconds
local DEFAULT_MUSIC_FADE_UP_TIME = 2000				-- in milliseconds 
local DEFAULT_SWAP_TRACK_WAIT_INTERVALS = 5			-- in updates

local BATTLE_TRACK_PLAYLIST = {
	"pq2audio/music/battle01",
	"pq2audio/music/battle02",
	"pq2audio/music/battle03",
	"pq2audio/music/battle04",
	"pq2audio/music/battle05",
	"pq2audio/music/battle06",
}

local BOSS_TRACK_PLAYLIST = {
	"pq2audio/music/boss01",
	"pq2audio/music/boss02",
	"pq2audio/music/boss03",
	"pq2audio/music/boss04",
	"pq2audio/music/boss05",
	"pq2audio/music/boss06",
}

-- Returns the enemies health in the range 0(none)-1(full), this only works if
-- a battle game is opened
local function GetEnemyHealth()
	assert(SCREENS.BattleGameMenu:IsOpen())
	local snapshot = SCREENS[SCREENS.BattleGameMenu.clients[2]].snapshot
	if snapshot then
		return snapshot.health / snapshot.max_health
	end
	return 1
end

-- Returns the heroes health in the range 0(none)-1(full), this only works if
-- a battle game is opened
local function GetHeroHealth()
	assert(SCREENS.BattleGameMenu:IsOpen())
	local snapshot = SCREENS[SCREENS.BattleGameMenuself.clients[1]].snapshot
	if snapshot then
		return snapshot.health / snapshot.max_health
	end
	return 1
end

local function IsScreenOpen(x)
	return rawget(SCREENS, x) and SCREENS[x]:IsOpen()
end

local function LoopTrackForever(x)
	MusicManager:PlayTrack(x)
	
	while true do
		if not SoundSystem.IsMusicPlaying() then
			MusicManager:PlayTrack(x)
		end
		
		coroutine.yield()
	end
end


local MusicManager = {}

function MusicManager:Reset()
	self:SetState("BEGIN")
end

function MusicManager:Init()
	_G.RegisterResetFunction("MusicManager", function () self:Reset() end)

	self.no_music_sample_loop = nil
	
	self.params = {}
	self.state_machine = require("StateMachine").Start({
	
	
		-- BEGIN: This is the starting state for the state machine, it just
		-- jumps straight into the menu theme
		
		BEGIN = function ()
			return "MENU"
		end,
		
		
		
		-- MENU: This state runs while any menu is active but transitions
		-- to the BATTLE_IDLE or WORLD_IDLE states when those screens are 
		-- opened.
		
		MENU = function ()
			LoopTrackForever("pq2audio/music/menu")
		end,
		
		
		
		
		-- LOADING: This state runs while any loading menu is displayed
		-- instead of streaming music it loops a small sample to ensure
		-- the system doesnt need to switch between loading/streaming
		LOADING = function ()
			while true do
				if self.no_music_sample_loop == nil then
					-- We stop the music system and clear its state
					if self.music and SoundSystem.IsValid( self.music ) then
						SoundSystem.Stop( self.music )
					end
					self.current_track = nil
					
					-- We initiate the sample loop to occupy our ears temporarily
					self.no_music_sample_loop = SoundSystem.Acquire( "pq2audio/ui/loading" )
					if SoundSystem.IsValid( self.no_music_sample_loop ) then
						SoundSystem.Play( self.no_music_sample_loop )
					else
						self.no_music_sample_loop = nil
					end
				end
				coroutine.yield()
			end
		end,
		
		
		
		-- PREBATTLE: in this state while the pre battle menu is being
		-- displayed.  Transitions from this state to MENU if the user
		-- flee's, or BATTLE_IDLE if the user goes into battle
		
		PREBATTLE = function ()
			if self.params.boss_encounter then
				LoopTrackForever("pq2audio/music/prebattle_boss")
			else
				LoopTrackForever("pq2audio/music/prebattle")
			end
		end,
		
		
		-- BATTLE_START
		
		BATTLE_START = function ()
			if self.params.boss_encounter then
				MusicManager:PlayTrack(table.random(BOSS_TRACK_PLAYLIST))
			else
				MusicManager:PlayTrack(table.random(BATTLE_TRACK_PLAYLIST))
			end
			
			self.params.enemy_health = 100
			self.params.enemy_max_health = 100
			self.params.hero_health = 100
			self.params.hero_max_health = 100
			
			return "BATTLE_IDLE"
		end,
		
		-- BATTLE_IDLE:
		
		BATTLE_IDLE = function ()
			while true do
				if self.params.enemy_health == 0 then
					return "BATTLE_VICTORY"
				elseif self.params.hero_health == 0 then
					return "BATTLE_DEFEAT"
				elseif (self.params.enemy_health / self.params.enemy_max_health) < 0.2 then
					return "BATTLE_WINNING"
				elseif (self.params.hero_health / self.params.hero_max_health) < 0.2 then
					return "BATTLE_LOSING"
				elseif not SoundSystem.IsMusicPlaying() then
					if self.params.boss_encounter then
						MusicManager:PlayTrack(table.random(BOSS_TRACK_PLAYLIST))
					else
						MusicManager:PlayTrack(table.random(BATTLE_TRACK_PLAYLIST))
					end
				end
				coroutine.yield()
			end
		end,
		
		BATTLE_VICTORY = function ()
			MusicManager:PlayTrack("pq2audio/music/victory")
			return "BATTLE_FINISH"
		end,
		
		BATTLE_DEFEAT = function ()
			MusicManager:PlayTrack("pq2audio/music/defeat")
			return "BATTLE_FINISH"
		end,
		
		BATTLE_WINNING = function ()
			MusicManager:PlayTrack("pq2audio/music/battlewinning")
			
			while SoundSystem.IsMusicPlaying() do
				if self.params.enemy_health <= 0 then
					return "BATTLE_VICTORY"
				elseif self.params.hero_health <= 0 then
					return "BATTLE_DEFEAT"
				end
				coroutine.yield()
			end
			return "BATTLE_IDLE"
		end,
		
		BATTLE_LOSING = function ()
			MusicManager:PlayTrack("pq2audio/music/battlelosing")
			
			while SoundSystem.IsMusicPlaying() do
				if self.params.enemy_health <= 0 then
					return "BATTLE_VICTORY"
				elseif self.params.hero_health <= 0 then
					return "BATTLE_DEFEAT"
				elseif (self.params.enemy_health / self.params.enemy_max_health) < 0.2 then
					return "BATTLE_WINNING"
				end
				coroutine.yield()
			end
			return "BATTLE_IDLE"
		end,
		
		BATTLE_FINISH = function ()
			while true do 
				coroutine.yield()
			end
			return "MENU"
		end,
		
		
		WORLD_BEGIN = function ()
			if _G.IsQuestCompleted(self.params.town_cleared_hero, self.params.town_cleared_quest) then
				return "WORLD_IDLE"
			else
				return "WORLD_IDLE_TOWNINDANGER"
			end
			
			return "WORLD_IDLE"
		end,
		
		
		WORLD_IDLE = function ()
			if self.params.room_def_queue.music then
				local music = self.params.room_def_queue.music
				local event = string.format("pq2audio/%s", string.gsub(music, "_", "/", 1))
				--LOGF("~ WORLD_IDLE music event: %s", event)
				self.params.room_def_current = event
			else
				self.params.room_def_current = nil
			end
			
			if self.params.room_def_current then
				MusicManager:PlayTrack(self.params.room_def_current)
			end
			
			while true do
				if not SoundSystem.IsMusicPlaying() then
					MusicManager:PlayTrack(self.params.room_def_current)
				end
				coroutine.yield()
			end
		end,
		
		
		WORLD_IDLE_TOWNINDANGER = function ()
			LoopTrackForever("pq2audio/music/town_danger")
		end,
		
		MINIGAME = function ()
			LoopTrackForever("pq2audio/music/minigame")
		end,
		
		MINIGAME_VICTORY = function ()
			MusicManager:PlayTrack("pq2audio/music/victory")
			return "BATTLE_FINISH"
		end,
		
		MINIGAME_DEFEAT = function ()
			MusicManager:PlayTrack("pq2audio/music/defeat")
			return "BATTLE_FINISH"
		end,
		
	}, "BEGIN")
end

function MusicManager:Update()
	if self.state_machine and not self.paused then
		self.state_machine:Update()
	end
end

function MusicManager:SetState(n)
	self.state_machine:SetState(n)
end

function MusicManager:PlayTrack(track, fade_down_time)
	LOGF("MusicManager:PlayTrack(%s, %s)", tostring(track), tostring(fade_down_time))
	
	if self.no_music_sample_loop ~= nil then
		SoundSystem.Stop( self.no_music_sample_loop )
		self.no_music_sample_loop = nil
	end
	
	if SoundSystem.IsMusicPlaying() and self.current_track == track then
		LOGF(" ... MusicManager ignored this request, this track is already playing")
		return
	end

	-- instruct the Update function to play a new track, strangely enough, this
	-- call actually fades out the old track over 2 seconds and then starts the
	-- new one.
	
	if self.music and SoundSystem.IsValid(self.music) then
		SoundSystem.Stop(self.music)
	end
	
	self.music = SoundSystem.Acquire(track)
	SoundSystem.Play(self.music)
	
	-- keep this around, its useful!
	self.current_track = track
end

function MusicManager:SetParam(name, v)
	self.params[name] = v
end

function MusicManager:Stop()
	if self.music and SoundSystem.IsValid(self.music) then
		SoundSystem.Stop(self.music)
	end
	
	self.paused = true
end

function MusicManager:Resume()
	self.paused = false
end

return MusicManager
