use_safeglobals()

local PLAYER_CLASSES = {
	["[CLASS_BARBARIAN]"] 	= "Barbarian",
	["[CLASS_WARMAGE]"] 	= "WarMage",
	["[CLASS_ASSASSIN]"] 	= "Assassin",
	["[CLASS_INQUISITOR]"] 	= "Inquisitor",
}

class "CreateHeroMenu" (Menu);

function CreateHeroMenu:__init()
	super()
	
	self:Initialize("Assets\\Screens\\CreateHeroMenu.xml")
end

function CreateHeroMenu:OnOpen()
	self.lockInput = nil
	self.activate_ok = true
	
	local class_tag = "[CLASS_ASSASSIN]"
	self.selected_gender = "M"
	self.selected_class = PLAYER_CLASSES[class_tag]
	
	return Menu.OnOpen(self)
end

function CreateHeroMenu:OnAnimOpen(data)
	if (data == 1) then
		self:ActivateEditor()
	end
end

function CreateHeroMenu:OnRefresh()
	if IsGamepadActive() then
		self:set_alpha("icon_gp_bg", 0.6)
		self:activate_widget("grp_xboxbuttons")
		self:hide_widget("grp_pcbuttons")
	else
		self:hide_widget("grp_xboxbuttons")
		self:activate_widget("grp_pcbuttons")
	end
	
	if self.selected_gender and self.selected_class then
		local name_gender = "male"
		if self.selected_gender == "F" then
			name_gender = "female"
		end
		
		local name = self:GetName(name_gender)
		self:set_text_raw("edit_name", name)
		
		local icon_widget = string.format("icon_%s%s", string.lower(self.selected_class), string.lower(self.selected_gender))
		local icon_name = string.format("img_charselect_%s%s", self.selected_class, self.selected_gender)
		self:set_image(icon_widget, icon_name)
		
		local class_tag = string.format("[CLASS_%s]", string.upper(self.selected_class))
		self:set_text("str_charselect", class_tag)
		
		local portrait_name = string.format("portrait_%s%s_M", self.selected_class, self.selected_gender)
		self:set_image("icon_portrait", portrait_name)
	end
end

function CreateHeroMenu:OnKey(key)
	if key == Keys.SK_ESCAPE then
		self:StartAnimation("fadeout")
	end
	
	return Menu.OnKey(self, key)
end

function CreateHeroMenu:OnButtonAvailable(id, x, y, on)
	if self.lockInput then
		return Menu.MESSAGE_HANDLED
	end
	
	local icon_changed = false
	
	local selected_class = self.selected_class
	local selected_gender = self.selected_gender
	
	if not IsGamepadActive() then
		local function IsPCButton(button_id)
			if button_id == self:get_widget_id("butt_info") then
				return true
			elseif button_id == self:get_widget_id("butt_ok") then
				return true
			elseif button_id == self:get_widget_id("butt_cancel") then
				return true
			end
		end
		
		if IsPCButton(id) then
			local class_name = self.selected_class
			local gender = self.selected_gender
			
			if class_name and gender then
				-- class name
				local class_tag = string.format("[CLASS_%s]", string.upper(class_name))
				self:set_text("str_charselect", class_tag)
				
				-- portrait
				local portrait_name = string.format("portrait_%s%s_M", class_name, gender)
				self:set_image("icon_portrait", portrait_name)
			end
		end
	end
	
	for class_tag, class_name in pairs(PLAYER_CLASSES) do
		local lowerClass = string.lower(class_name)
		for _, gender in ipairs({"M", "F"}) do
			local lowerGender = string.lower(gender)
			
			local icon_widget = string.format("icon_%s%s", lowerClass, lowerGender)
			local icon_hi_widget = string.format("icon_%s%s_hi", lowerClass, lowerGender)
			
			local button_widget = string.format("butt_%s%s", lowerClass, lowerGender)
			
			if id == self:get_widget_id(button_widget) then
				local gamepad_active = IsGamepadActive()
				if gamepad_active then
					icon_changed = true
					self.activate_ok = true
					
					local name_gender = "male"
					if gender == "F" then
						name_gender = "female"
					end
					
					local name = self:GetName(name_gender)
					self:set_text_raw("edit_name", name)
				end
				
				if on then
					-- highlight
					self:set_image(icon_hi_widget, "img_charselect_HighlightBox")
					
					if gamepad_active then
						-- icon
						local icon_name = string.format("img_charselect_%s%s", class_name, gender)
						self:set_image(icon_widget, icon_name)
					end
					
					-- class name
					self:set_text("str_charselect", class_tag)
					
					-- portrait
					local portrait_name = string.format("portrait_%s%s_M", class_name, gender)
					self:set_image("icon_portrait", portrait_name)
					
					if gamepad_active then
						self.selected_class = class_name
						self.selected_gender = gender
					end
				end
			else
				self:set_image(icon_hi_widget, "")
				
				if IsGamepadActive() then
					-- revert old selected to grey
					local icon_name = string.format("img_charselect_%s%s_g", class_name, gender)
					self:set_image(icon_widget, icon_name)
				end
			end
		end
	end
	
	if icon_changed then
		if self.activate_ok then
			self:activate_widget("butt_ok")
		end
		return Menu.MESSAGE_HANDLED
	end
	
	return Menu.MESSAGE_NOT_HANDLED -- allow fallthru for other buttons ("Ok", "Cancel", etc)
end

function CreateHeroMenu:CreateNewHero(name, class, portrait, sex)
	assert(name)
	assert(class)
	assert(portrait)
	assert(sex)
	
	print( string.format( "CreateHeroMenu:CreateNewHero( %s, %s, %s, %s )", tostring( name ), tostring( class ), tostring( portrait ), tostring( sex ) ) )

	local HeroData = require("Hero/"..tostring(class))
	
	local hero = CreateStockHero(string.format("Hero.Default%s", class))
	hero:InitialiseCampaign()
	hero:SetAttribute("name", name)
	hero:SetAttribute("class", class)
	hero:SetAttribute("portrait", portrait)
	hero:SetAttribute("sex", sex)
	
	InitialiseNewHeroTutorials(hero)
	
	for _,spellName in ipairs(HeroData.start_data.spells) do
		hero:PushAttribute("knownSpells", spellName)
		hero:PushAttribute("activeSpells", spellName)
	end
	
	hero:InitializeStats()
	hero:CalculateStats()

	-- spam to debug log, often handy to have the hero information here when bug tracking
	hero:PrintDebug()
	
	HeroManager:AddNewHero(hero)
	
	return hero
end


function CreateHeroMenu:OnButton(buttonId, clickX, clickY)
	if self.lockInput then
		return Menu.MESSAGE_HANDLED
	end
	
	local function continue()
		CallScreenSequencer("CreateHeroMenu", function ()
			local portrait = string.format("portrait_%s%s", self.selected_class, self.selected_gender)
			local name = self:get_text("edit_name")
			name = string.gsub(name, "^(.-)%s*$", "%1") -- strip trailing whitespace (leading was stripped in OnEditor)
			local hero = self:CreateNewHero(name, self.selected_class, portrait, self.selected_gender)
			HeroManager:SetActiveHero(hero)
			hero:InitializeStats()
			hero:CalculateStats()
			hero:PrintDebug(true)

			HeroLoadTutorialSystem(hero)	
		
			SCREENS.SaveMenu:Display(function ()
				local saveHandle = HeroManager:GetActiveHeroSaveHandle()
				if saveHandle then
					Settings:Write("last_save", saveHandle:GetName())
					Settings:Save()
				end
				
				SCREENS.SinglePlayerMenu:Open()
				SCREENS.SinglePlayerMenu:StartAnimation("fadein")
			end)
		end)
	end
	
	if buttonId == self:get_widget_id("butt_cancel") then
		self.lockInput = true
		self:Cancel()
		return Menu.MESSAGE_HANDLED
		
	elseif buttonId == self:get_widget_id("butt_ok") then
		self.lockInput = true
		continue()
		return Menu.MESSAGE_HANDLED
		
	elseif buttonId == self:get_widget_id("butt_info") then
		SCREENS.CreateHeroInfoMenu:SetupAndOpen(self.selected_class, self.selected_gender, function() self:ActivateEditor() end)
		return Menu.MESSAGE_HANDLED
	end
	
	local icon_changed = false
	
	local selected_class = self.selected_class
	local selected_gender = self.selected_gender
	
	for class_tag, class_name in pairs(PLAYER_CLASSES) do
		local lowerClass = string.lower(class_name)
		for _, gender in ipairs({"M", "F"}) do
			local lowerGender = string.lower(gender)
			
			local button_widget = string.format("butt_%s%s", lowerClass, lowerGender)
			local is_button = buttonId == self:get_widget_id(button_widget)
			
			if IsGamepadActive() then
				if self.activate_ok and is_button then
					self.lockInput = true
					continue()
					return Menu.MESSAGE_HANDLED
				end
				
			else	-- keyboard/mouse
				local icon_widget = string.format("icon_%s%s", lowerClass, lowerGender)
				
				if is_button then
					if selected_class == class_name and selected_gender == gender then
						-- skip
					else
						icon_changed = true
						self.activate_ok = true
						
						local name_gender = "male"
						if gender == "F" then
							name_gender = "female"
						end
						
						local name = self:GetName(name_gender)
						self:set_text_raw("edit_name", name)
						
						local icon_name = string.format("img_charselect_%s%s", class_name, gender)
						self:set_image(icon_widget, icon_name)
						
						self.selected_class = class_name
						self.selected_gender = gender
					end
				else
					-- revert old selected to grey
					local icon_name_0 = string.format("img_charselect_%s%s_g", class_name, gender)
					self:set_image(icon_widget, icon_name_0)
				end
			end
		end
	end
	
	if icon_changed then
		if self.activate_ok then
			self:activate_widget("butt_ok")
		end
		return Menu.MESSAGE_HANDLED
	end
	
	return Menu.MESSAGE_HANDLED
end

function CreateHeroMenu:OnGamepadButton(user_id, button_id, up)
	if up == 0 or self.lockInput then
		return Menu.MESSAGE_HANDLED
	end
	
	if button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_TOP) then
		SCREENS.CreateHeroInfoMenu:SetupAndOpen(self.selected_class, self.selected_gender, function() self:ActivateEditor() end)	
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CANCEL) then
		self.lockInput = true
		self:Cancel()
	end
	
	return Menu.MESSAGE_HANDLED
end

function CreateHeroMenu:OnEditor(id, name)
	name = string.gsub(name, "^%s*(.-)$", "%1") -- strip leading whitespace (trailing is stripped in OnButton)
	
	if string.len(name) > 0 then
		self:activate_widget("butt_ok")
		self.activate_ok = true
	else
		self:deactivate_widget("butt_ok")
		self.activate_ok = false
	end
	
	self:set_text_raw("edit_name", name)
	
	self:ActivateEditor()
	
	return Menu.OnEditor(self, id, name)
end

function CreateHeroMenu:OnAnimFadeout(data) 
	if (data == 1) then
		CallScreenSequencer("CreateHeroMenu", "MainMenu")
	end
end

function CreateHeroMenu:GetName(gender)
	if not gender or (gender ~= "male" and gender ~= "female") then
		local msg = string.format("%s is not a valid gender, must be male or female", tostring(gender))
		assert(false, msg)
	end

	if gender == "male" then
		local names ={"Sirian", "Jormungard", "Oberron", "Sholto", "Goethe", "Ehlarion", "Artos", "Galad", "Christof", "Nikolas"}
		return table.random(names)

	else
		local names ={"Kiesha", "Artema", "Lycene", "Rycella", "Amarante", "Sekhma", "Saraphine", "Bethriane", "Gwynnyth", "Tempest"}
		return table.random(names)
	end
end

-- Any platform specific stuff in here
dofile("Assets/Scripts/Screens/CreateHeroMenuPlatform.lua")

-- return an instance of CreateHeroMenu
return ExportSingleInstance("CreateHeroMenu")
