use_safeglobals()

class "InventoryMenu" (Menu);

local PAGE_WIDTH = 4
local PAGE_LENGTH = 4
local LIST_SIZE = PAGE_WIDTH * PAGE_LENGTH

local EQUIPSTATUS_EQUIPPABLE   =  1
local EQUIPSTATUS_UNEQUIPPABLE = -1

local Listbox = require("Utils.IconListboxHelpers")
local LuaGamepad = require("Controls.Gamepad")

local gp_mode = { field = 1, area = "items" }
local filters = { weapons = true, armour = true, trade = true }
local show_info

local gp_navigation = {
	filter1		= { widget="butt_all",            left="filter4",      right="filter2",                          down="items"                         },
	filter2		= { widget="butt_weapons",        left="filter1",      right="filter3",                          down="items"                         },
	filter3		= { widget="butt_armour",         left="filter2",      right="filter4",                          down="items"                         },
	filter4		= { widget="butt_misc",           left="filter3",      right="filter1",                          down="items"                         },

	items		= { widget="pad_inventory_item1", --[[L/R are special-cased]]             up="filter1",                                               },
	
	offhand		= { widget="icon_offhand",        left="feet",                            up="armour",                                 highlight=true },
	mainhand	= { widget="icon_mainhand",       left="stat_strength", right="feet",     up="armour",                                 highlight=true },
	feet		= { widget="icon_feet",           left="mainhand",      right="offhand",  up="armour",                                 highlight=true },
	jewellery	= { widget="icon_jewellery",                            right="offhand",  up="helm",              down="armour",       highlight=true },
	armour		= { widget="icon_armour",                               right="offhand",  up="jewellery",         down="feet",         highlight=true },
	helm		= { widget="icon_helm",           left="stat_strength", right="offhand",                          down="jewellery",    highlight=true },

	stat_strength     = { widget="icon_stat_strength",                   right="mainhand", up="stat_morale",       down="stat_agility",      highlight=true },
	stat_agility      = { widget="icon_stat_agility",                    right="mainhand", up="stat_strength",     down="stat_intelligence", highlight=true },
	stat_intelligence = { widget="icon_stat_intelligence",               right="mainhand", up="stat_agility",      down="stat_stamina",      highlight=true },
	stat_stamina      = { widget="icon_stat_stamina",                    right="mainhand", up="stat_intelligence", down="stat_morale",       highlight=true },
	stat_morale       = { widget="icon_stat_morale",                     right="mainhand", up="stat_stamina",      down="stat_strength",     highlight=true },
}                                                                         

local function ApplyFilters(item)
	if type(item) == "userdata" then
		if filters.weapons and item:IsCarriedInHand() then
			return true
		elseif filters.armour and item:IsEquippable() and (not item:IsCarriedInHand()) then
			return true			
		elseif filters.trade and item:IsTrade() then
			return true
		end
	elseif type(item) == "string" and filters.trade then
		return true
	end
	return false
end

function InventoryMenu:__init()
	super()
	self:Initialize("Assets\\Screens\\InventoryMenu.xml")
end

function InventoryMenu:RefreshStats(hero)
	assert(hero)
	
	hero:CalculateStatsDerived()	--CalculateStats()
	
	local stats = { "strength", "agility", "stamina", "intelligence", "morale" }
	
	for _, stat in ipairs(stats) do
		local value, modified = hero:GetStat(stat)
		self:set_text_raw(string.format("str_stat_%s_value", stat), tostring(value))
		if modified then
			self:set_font(string.format("str_stat_%s_value", stat), "font_uitext_green")
		else
			self:set_font(string.format("str_stat_%s_value", stat), "font_uitext")
		end
	end
	
	self:set_text_raw("str_attack_value", tostring(hero:GetAttack()))
	self:set_text_raw("str_def_value", tostring(hero:CalculateStatDefenceDerived()))
	
	local mana = { "red", "green", "blue", "yellow", "black" }
	for _, color in ipairs(mana) do
		local value, modified = hero:GetMaxMana(color)
		self:set_text_raw(string.format("str_%s_value", color), tostring(value))
		if modified then
			self:set_font(string.format("str_%s_value", color), "font_uitext_green")
		else
			self:set_font(string.format("str_%s_value", color), "font_uitext")
		end
	end
end

function InventoryMenu:SetHero(hero)
	self.hero = hero
	
	self:SetupHeroDisplay(hero)
	
	self:SetupListbox(hero)
end

function InventoryMenu:SetupHeroDisplay(hero)
	self:set_text_raw("str_name_value", tostring(hero:GetAttribute("name")))
	self:set_image("icon_portrait", string.format("%s_M", hero:GetAttribute("portrait")))
	self:set_text_raw("str_level_value", tostring(hero:GetAttribute("level")))
	self:set_text_raw("str_exp_value", tostring(hero:GetAttribute("xp")))
	self:set_text_raw("str_health_value", tostring(hero:GetHealth()))
	self:set_text_raw("str_gold_value", tostring(hero:GetAttribute("gold")))
	local attack = hero:GetAttack()
	self:set_text_raw("str_attack_value", tostring(attack))
	local defence = hero:GetDefence()
	self:set_text_raw("str_def_value", tostring(defence))
	
	self:UpdateInventoryQuantities(hero)
	
	self:RefreshStats(hero)
	
	self:SetItems(hero)
end

function InventoryMenu:GetItems()
	
	local list1 = CollectionToTable(self.hero, "inventory", ApplyFilters)
	local list2 = CollectionToTable(self.hero, "questItems", ApplyFilters)
	
	-- could use table.join for this, but it always puts the smaller table
	-- last, which can reorganise the inventory - not desirable.
	local t = { }
	for _, value in ipairs(list1) do
		table.insert(t, value)
	end
	for _, value in ipairs(list2) do
		table.insert(t, value)
	end
	return t
end

function InventoryMenu:OnOpen()
	_G.Joystick.InitJoystick()
	
	--SHOW TUTORIAL ONCE
	local hero = HeroManager:GetActiveHero()
	if not hero:GetAttribute("inventoryTutorialShown") and not is_conversation_menu_open() then
		open_conversation_menu(
			"Assets/Conversations/INVENTORY_TUTORIAL.xml",
			hero:GetName(), 
			hero:GetConversationPortrait(), 
			hero:GetGenderForConversation(), 
			"","","", 
			function ()-- conversation finished, start game
				hero:SetAttribute("inventoryTutorialShown",true)
			end, 
			x or GetScreenWidth()*0.5,
			y or GetScreenHeight()*0.5
		)
	end
	
	return Menu.OnOpen(self)
end

function InventoryMenu:OnAnimOpen(data)
	if (data == 1) then
		if IsGamepadActive() then
			Listbox.DoCallback("items", "SelectEntry", 1)
			
			gp_mode.field = 1
			gp_mode.area  = "items"
			self:SetCursorPosition(1)
			if IsGamepadActive() then
				MenuSystem.SetGamepadWidget(self, "pad_inventory_item1")
			end
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function InventoryMenu:OnRefresh()
	if IsGamepadActive() then
		self:activate_widget("grp_xboxbuttons")
		self:hide_widget("grp_pcbuttons")
		self:hide_widget("icon_lboxhighlight")
		self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
		show_info = false
	else
		self:hide_widget("grp_xboxbuttons")
		self:hide_widget("icon_lboxhighlight")
		self:hide_widget("icon_rboxhighlight")
		self:activate_widget("grp_pcbuttons")
		show_info = true
	end
	
	self:set_image("icon_cursor", "anim_inventory_selector")
	
	filters.weapons = true
	filters.armour = true
	filters.trade = true
	self:set_text("str_rboxheading", "[FILTER_ALL]")
	
	if self.hero then
		self:SetupHeroDisplay(self.hero)
	end
	
	if Listbox.ListExists("items") then
		Listbox.ChangeList("items", self:GetItems())
		self:UpdateScrollbar()
	end
end

function InventoryMenu:OnClose()
	Listbox.ClearList("items")
	
	if self.completedCallback then
		self.completedCallback()
		self.completedCallback = nil
	end
	
	self.hero = nil
	
	return Menu.OnClose(self)
end

function InventoryMenu:SetCompletedCallback(cb)
	self.completedCallback = cb
end

function InventoryMenu:UpdateInventoryQuantities(hero)
	local equip_str = translate_text("[EQUIPMENT_HEADING]")
	local colon = translate_text("[SEPERATOR]")
	local str_items = string.format("%s%s %d/%d", equip_str, colon, hero:GetNumberOfItems(), hero:GetMaxInvItems()-hero:GetMaxResources())
	self:set_text_raw("str_qty_items", str_items)
end

function InventoryMenu:SetItems(hero)
	local slots = { mainHand = "mainhand", 
	                offHand = "offhand", 
	                helm = "helm", 
	                armour = "armour", 
	                jewellery = "jewellery", 
	                feet = "feet" }
	
	for slot_name, widget_name in pairs(slots) do
		local item = hero:GetAttribute(string.format("%sItem", slot_name))
		if item then
			self:activate_widget(string.format("icon_%s", widget_name))
			self:activate_widget(string.format("icon_%s_overlay", widget_name))
			self:set_image(string.format("icon_%s", widget_name), item:GetIconAsset())
			self:set_image(string.format("icon_%s_overlay", widget_name), item:GetIconOverlayAsset())
			if item:IsTwoHanded() then
				self:activate_widget("icon_offhand")
				self:activate_widget("icon_offhand_overlay")
			end
		else
			self:hide_widget(string.format("icon_%s", widget_name))
			self:hide_widget(string.format("icon_%s_overlay", widget_name))
		end
	end
	
	LOGF("%s:  %d, %d", "mainHand", self:get_widget_x("icon_mainhand_overlay"), self:get_widget_y("icon_mainhand_overlay"))
	LOGF("%s:  %d, %d", "offHand",  self:get_widget_x("icon_offhand_overlay"),  self:get_widget_y("icon_offhand_overlay"))
end

function InventoryMenu:OnScrollbar(scrollbar_id, value)
	if scrollbar_id == self:get_widget_id("scroll_items") then
		local currentLine	= math.ceil(Listbox.GetFirstIndex("items")/PAGE_WIDTH)
		if value+1 > currentLine then
			-- navigate down the list (inc)
			local diff = (value+1 - currentLine)*PAGE_WIDTH
			local distanceToEnd = Listbox.GetListSize("items") - Listbox.GetCurrentIndex("items")
			self.suppressTooltip = true
			Listbox.NavigateList("items", math.min(diff, distanceToEnd))
		elseif value+1 < currentLine then
			-- navigate up the list (dec)
			local diff = (currentLine - (value+1))*PAGE_WIDTH
			local distanceToEnd = Listbox.GetCurrentIndex("items")-1
			self.suppressTooltip = true
			Listbox.NavigateList("items", -math.min(diff, distanceToEnd))
		end
		
		Listbox.UpdateList("items")
	end
	return Menu.MESSAGE_HANDLED
end

function InventoryMenu:ShowHighlight(widget)
	assert(widget)
	
	if string.sub(widget, 1, 9) == "icon_stat" then
		self:hide_widget("icon_cursor")
		self:activate_widget("icon_stat_selection")
		self:set_widget_position("icon_stat_selection", self:get_widget_x(widget)-12, self:get_widget_y(widget)-12)
	else
		self:hide_widget("icon_stat_selection")
		self:activate_widget("icon_cursor")
		self:set_widget_position("icon_cursor", self:get_widget_x(widget)-14, self:get_widget_y(widget)-14)
	end
	SoundSystem.Play("pq2audio/ui/itemhover")
end


local function GetHeroToolTipFormatAndData(hero)
	local DEFAULT_WIDTH = 200
	
	local class_base_table = require( "Hero/" .. hero:GetAttribute( "class" ) )
	local hero_stats_constants = require( "Hero/StatsConstants" )
	local healthBonus = class_base_table.hit_points_per_level + hero:GetStatRankBonus( hero_stats_constants.stat_bonus.health_per_level )
	
	local fmt = ""
	local data = ""
	
	fmt = fmt .. "//SHADING/img_TextBackground_Red"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, 3, "font_uitext", translate_text("[BONUSES]"))
	fmt = fmt .. "//SHADING/off"
	
	-- Bonuses
	local str_bonus1 = string.format(translate_text("[SPELLRESISTANCE]"), hero:GetSpellResistChance())
	local str_bonus2 = string.format(translate_text("[SPELLPENETRATION]"), hero:GetSpellPenetration())
	local str_bonus3 = string.format(translate_text("[WEAPONCRITICAL]"), hero:GetWeaponCriticalChance())
	local str_bonus4 = string.format(translate_text("[SHIELDCRITICAL]"), hero:GetShieldCriticalChance())
	local str_bonus5 = substitute(translate_text("[MORALE_RANKEFFECT]"), hero:GetWeaponBonusDamage())
	local str_bonus6 = substitute(translate_text("[ITEM_EFFECT_BLOCKEFFICIENCYBONUS]"), (hero:GetBlockingEffectiveness() * 100) - 50)
	fmt, data = TooltipHelpers.AddTitle(fmt, data, 42,  "font_tooltip", str_bonus1)
	fmt, data = TooltipHelpers.AddTitle(fmt, data, 72,  "font_tooltip", str_bonus2)
	fmt, data = TooltipHelpers.AddTitle(fmt, data, 102,  "font_tooltip", str_bonus3)
	fmt, data = TooltipHelpers.AddTitle(fmt, data, 132,  "font_tooltip", str_bonus4)
	fmt, data = TooltipHelpers.AddTitle(fmt, data, 162,  "font_tooltip", str_bonus5)
	fmt, data = TooltipHelpers.AddTitle(fmt, data, 192,  "font_tooltip", str_bonus6)
	
	fmt = fmt .. "//"
	data = data .. "//"
	
	return fmt,data
end


function InventoryMenu:ShowHeroPopup(x, y, id)
	assert(x)
	assert(y)
	
	if not show_info then
		return
	end
	
	if self.hero then
		local format, data = GetHeroToolTipFormatAndData(self.hero)
		
		open_custompopup_menu(
			format,
			data,
			x, y,
			0,
			200,
			"PlayerInfoMenu",
			id
		)
	end
end

function InventoryMenu:SetCursorPosition(field)
	close_custompopup_menu()
	if type(field) == "number" then -- listbox field
		gp_mode.area = "items"
		if (field > LIST_SIZE) or (field < 1) then
			return
		end
		if Listbox.GetEntryFromField("items", field) then
			Listbox.NavigateTo("items", Listbox.GetIndexFromField("items", field))
		else
			self:ShowHighlight(string.format("icon_inventory_item%d", field))
			gp_mode.field = field
		end
	elseif type(field) == "string" then -- gp_navigation widget entry
		self:ShowHighlight(string.format("icon_%s", field))
		gp_mode.area = field
		
		self:ShowCurrentItemTooltip()
	end
end

function InventoryMenu:OnButton(button_id, x_value, y_value)
	if button_id == self:get_widget_id("butt_close") then
		close_custompopup_menu()
		self:Close()
	elseif button_id == self:get_widget_id("butt_all") then
		self:set_text("str_rboxheading", "[FILTER_ALL]")
		filters.weapons = true
		filters.armour = true
		filters.trade = true
		Listbox.ChangeList("items", self:GetItems())
	elseif button_id == self:get_widget_id("butt_weapons") then
		self:set_text("str_rboxheading", "[FILTER_ACTIVE]")
		filters.weapons = true
		filters.armour = false
		filters.trade = false
		Listbox.ChangeList("items", self:GetItems())
	elseif button_id == self:get_widget_id("butt_armour") then
		self:set_text("str_rboxheading", "[FILTER_PASSIVE]")
		filters.weapons = false
		filters.armour = true
		filters.trade = false
		Listbox.ChangeList("items", self:GetItems())
	elseif button_id == self:get_widget_id("butt_misc") then
		self:set_text("str_rboxheading", "[FILTER_RESOURCES]")
		filters.weapons = false
		filters.armour = false
		filters.trade = true
		Listbox.ChangeList("items", self:GetItems())
	end
	return Menu.MESSAGE_HANDLED
end

function InventoryMenu:OnKey(key)
	if key == Keys.SK_ESCAPE then
		close_custompopup_menu()
		self:Close()
	else
		return Menu.OnKey(self, key)
	end
	
	return Menu.MESSAGE_HANDLED
end

function InventoryMenu:SetupListbox(hero)
	Listbox.AddListbox("items", self:GetItems(), LIST_SIZE, PAGE_WIDTH, true)
	Listbox.SetCallback("items", "SetData",
	function(entry, field)
		local item_icon      = string.format("icon_inventory_item%d", field)
		local item_string    = string.format("str_inventory_item%d", field)
		local item_overlay   = string.format("icon_inventory_item%d_overlay", field)
		local item_equipped  = string.format("icon_inventory_item%d_equipped", field)
		local item_status    = string.format("icon_inventory_item%d_status", field)
		
		self:activate_widget(item_icon)
		
		if type(entry) == "string" then
			-- Quest Items are strings, not GameObjects!
			local item_data = require("QuestItems/"..entry)
			self:set_image(item_icon, item_data.icon)
			self:hide_widget(item_overlay)
			self:hide_widget(item_status)
			self:hide_widget(item_equipped)
			self:hide_widget(item_string)
		else
			self:activate_widget(item_overlay)
			self:set_image(item_icon,    entry:GetIconAsset())
			self:set_image(item_overlay, entry:GetIconOverlayAsset())
		
			if entry:IsTrade() then
				self:hide_widget(item_status)
				self:hide_widget(item_equipped)
				self:activate_widget(item_string)
				self:set_text_raw(item_string, string.format("x%d", entry:GetAttribute("stacksize")))
			else
				self:hide_widget(item_string)
				
				local icon
				entry.equippable = EQUIPSTATUS_EQUIPPABLE
				if self.hero then
					if self.hero:IsItemEquipped(entry) then
						self:activate_widget(item_equipped)
					else
						self:hide_widget(item_equipped)
					end
					if not self.hero:CanEquipItem(entry) then
						self:activate_widget(item_status)
						self:set_image(item_status, "img_Blank")
						self:set_alpha(item_status, 0.65)
						entry.equippable = EQUIPSTATUS_UNEQUIPPABLE
					else
						self:hide_widget(item_status)
					end
				end
			end
		end
	end)
	Listbox.SetCallback("items", "NoData",
	function(field)
		self:hide_widget(string.format("icon_inventory_item%d", field))
		self:hide_widget(string.format("icon_inventory_item%d_overlay", field))
		self:hide_widget(string.format("icon_inventory_item%d_status", field))
		self:hide_widget(string.format("icon_inventory_item%d_equipped", field))
		self:hide_widget(string.format("str_inventory_item%d", field))
	end)
	Listbox.SetCallback("items", "SelectEntry",
	function(entry, field)
		if entry then
			self:ShowHighlight(string.format("icon_inventory_item%d", field))
			gp_mode.field = field
			gp_mode.area = "items"
			
			local lineNumber = math.ceil(Listbox.GetFirstIndex("items") / PAGE_WIDTH)
			self:set_scrollbar_value("scroll_items", lineNumber-1) --scrollbar values are 0 based not 1
			
			self:ShowCurrentItemTooltip()
		end
	end)
	Listbox.SetCallback("items", "DeselectEntry",
	function(entry, field)
		self:hide_widget("icon_cursor")
	end)
	Listbox.SetCallback("items", "UpdateDisplay",
	function()
		
	end)
	Listbox.UpdateList("items")
	self:UpdateScrollbar()
end

function InventoryMenu:UpdateScrollbar()
	local maxLines = math.max(0, math.ceil(Listbox.GetListSize("items")/PAGE_WIDTH) - PAGE_LENGTH)
	if maxLines == 0 then
		self:deactivate_widget("scroll_items")
	else
		self:activate_widget("scroll_items")
		self:set_scrollbar_range("scroll_items", 0, maxLines)
		self:set_scrollbar_value("scroll_items", 0)
	end
end

function InventoryMenu:OnMouseEnter(pad_id, x_value, y_value)
	for i=1, LIST_SIZE do
		if pad_id == self:get_widget_id(string.format("pad_inventory_item%d", i)) then
			if Listbox.GetEntryFromField("items", i) then
				Listbox.NavigateTo("items", Listbox.GetIndexFromField("items", i))
				return Menu.MESSAGE_HANDLED
			end
		end
	end
	
	if pad_id == self:get_widget_id("pad_helm") then
		self:SetCursorPosition("helm")
	elseif pad_id == self:get_widget_id("pad_chest") then
		self:SetCursorPosition("armour")
	elseif pad_id == self:get_widget_id("pad_mainhand") then
		self:SetCursorPosition("mainhand")
	elseif pad_id == self:get_widget_id("pad_offhand") then
		self:SetCursorPosition("offhand")
	elseif pad_id == self:get_widget_id("pad_feet") then
		self:SetCursorPosition("feet")
	elseif pad_id == self:get_widget_id("pad_jewellery") then
		self:SetCursorPosition("jewellery")
	elseif pad_id == self:get_widget_id("pad_stat_strength") then
		self:SetCursorPosition("stat_strength")
	elseif pad_id == self:get_widget_id("pad_stat_agility") then
		self:SetCursorPosition("stat_agility")
	elseif pad_id == self:get_widget_id("pad_stat_intelligence") then
		self:SetCursorPosition("stat_intelligence")
	elseif pad_id == self:get_widget_id("pad_stat_stamina") then
		self:SetCursorPosition("stat_stamina")
	elseif pad_id == self:get_widget_id("pad_stat_morale") then
		self:SetCursorPosition("stat_morale")
	elseif pad_id == self:get_widget_id("pad_portrait") then
		self:ShowHeroPopup(self:get_widget_abs_x("pad_portrait") + (self:get_widget_w("pad_portrait")/2), self:get_widget_abs_y("pad_portrait") + self:get_widget_h("pad_portrait"), pad_id)
	end
	
	return Menu.MESSAGE_NOT_HANDLED
end

function InventoryMenu:OnMouseLeave(pad_id, x_value, y_value)
	close_custompopup_menu()
	return Menu.MESSAGE_HANDLED
end

function InventoryMenu:OnGamepadJoystick(user_id, joystick_id, x_value, y_value)
	if _G.Joystick then
		_G.Joystick.OnGamepadJoystick(user_id, joystick_id, x_value, y_value)
	end
	
	return Menu.MESSAGE_HANDLED
end

function InventoryMenu:OnGamepadDPad(user_id, dpad_id, x_value, y_value)
	if x_value == 0 and y_value == 0 then 
		return Menu.MESSAGE_HANDLED
	end
	-- LOGF("[InvMenu:OnGamepadDPad] X:%d Y:%d GP.area:%s GP.field:%d", x_value, y_value, gp_mode.area, gp_mode.field)
	if (gp_mode.area == "items") then
		local currentIndex = Listbox.GetCurrentIndex("items")
		
		-- special cases to move up to the filter buttons
		if currentIndex > 0 and currentIndex <= PAGE_WIDTH and y_value > 0 then
			-- highlight is in top row and up pushed
			if filters.weapons and filters.armour and filters.trade then
			-- move up to the currently selected filter
				-- move to All
				self:MoveGamepadFocus("filter1")
			elseif filters.weapons then
				self:MoveGamepadFocus("filter2")
			elseif filters.armour then
				self:MoveGamepadFocus("filter3")
			elseif filters.trade then
				self:MoveGamepadFocus("filter4")
			else
				LOG("[InventoryMenu:OnGamepadDPad] couldn't identify active filter:")
				for k,v in pairs(filters) do
					LOGF("%s = %s", tostring(k), tostring(v))
				end
			end

		else
			local currentLine = math.ceil(currentIndex / PAGE_WIDTH)
			local leftEdge = (currentLine-1) * PAGE_WIDTH + 1
			local rightEdge = currentLine * PAGE_WIDTH
			if x_value < 0 then
				if currentIndex == leftEdge then
					Listbox.NavigateList("items", rightEdge - leftEdge)
				else
					Listbox.NavigateList("items", -1)
				end
			elseif x_value > 0 then
				if currentIndex == rightEdge or currentIndex == Listbox.GetListSize("items") then
					Listbox.NavigateList("items", -(currentIndex - leftEdge))
				else
					Listbox.NavigateList("items", 1)
				end
			elseif y_value < 0 then
				Listbox.NavigateList("items", math.min(PAGE_WIDTH, Listbox.GetListSize("items") - currentIndex))
			elseif y_value > 0 then
				Listbox.NavigateList("items", math.max(-PAGE_WIDTH, -currentIndex+1))
			end
		end
	else
		LuaGamepad.NavigateUI(self, x_value, y_value, gp_navigation, gp_mode.area)
	end
	return Menu.MESSAGE_HANDLED
end

function InventoryMenu:OnGamepadButton(user_id, button_id, up)
	if up == 0 then
		return Menu.MESSAGE_HANDLED
	end
	if button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_OKAY) then
		local item, x, y = self:GetSlotData()
		if item and x and y then
			_G.Joystick.InitJoystick()
			self:OpenPopup(item, x, y)
		end
		return Menu.MESSAGE_HANDLED
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CANCEL) then
		close_custompopup_menu()
		self:Close()
		return self:OnButton(self:get_widget_id("butt_close"), self:get_widget_abs_x("butt_close"), self:get_widget_abs_y("butt_close"), true)
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_TOP) then
		show_info = not show_info
		if show_info then
			self:set_text("str_gp_y", "[MORE_INFO_HIDE]")
			self:ShowCurrentItemTooltip()
		else
			self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
			close_custompopup_menu()
		end
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_SHOULDER_LEFT)  or 
		   button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_SHOULDER_RIGHT) then
		if gp_mode.area == "items" or string.sub(gp_mode.area, 1, 6) == "filter" then
			self:hide_widget("icon_rboxhighlight")
			if gp_mode.area ~= "items" then -- ie a filter button
				MenuSystem.SetGamepadWidget(self, "butt_dummy")
			end
			self:MoveGamepadFocus("offhand")
			self:activate_widget("icon_lboxhighlight")
		else
			self:hide_widget("icon_lboxhighlight")
			gp_mode.field = 1
			self:MoveGamepadFocus("items")
			self:activate_widget("icon_rboxhighlight")
		end
	end
	
	return Menu.MESSAGE_NOT_HANDLED
end

function InventoryMenu:OnMouseLeftButton(pad_id, x_value, y_value, up)
	if not up then
		return Menu.MESSAGE_NOT_HANDLED
	end
	
--	for i=1, LIST_SIZE do
--		if pad_id == self:get_widget_id(string.format("pad_inventory_item%d", i)) then
			local item, x, y = self:GetSlotData()
			if item and x and y then
				self:OpenPopup(item, x, y)
			end
			return Menu.MESSAGE_HANDLED
--		end
--	end
--	return Menu.MESSAGE_HANDLED
end

-- This function returns the item, x, and y values for the currently highlighted slot.
-- Nil is returned if a slot is not selected
function InventoryMenu:GetSlotData()
	local widget
	local item
	if gp_mode.area == "items" then
		widget = string.format("pad_inventory_item%d", gp_mode.field)
		item   = Listbox.GetEntryFromField("items", gp_mode.field)
	elseif gp_mode.area == "mainhand" then
		widget = string.format("icon_mainhand")
		item   = self.hero:GetAttribute("mainHandItem")
	elseif gp_mode.area == "offhand" then
		widget = string.format("icon_offhand")
		item   = self.hero:GetAttribute("offHandItem")
	elseif gp_mode.area == "feet"
		 or gp_mode.area == "jewellery"
		 or gp_mode.area == "armour"
		 or gp_mode.area == "helm" then
		widget = string.format("icon_%s", gp_mode.area)
		item   = self.hero:GetAttribute(string.format("%sItem", gp_mode.area))
	elseif gp_mode.area == "stat_strength"
		 or gp_mode.area == "stat_agility"
		 or gp_mode.area == "stat_intelligence"
		 or gp_mode.area == "stat_stamina"
		 or gp_mode.area == "stat_morale" then
		local stats = {["stat_strength"] = 1, ["stat_agility"] = 2, ["stat_intelligence"] = 3, ["stat_stamina"] = 4, ["stat_morale"] = 5}
		widget = gp_navigation[gp_mode.area].widget
		item = stats[gp_mode.area]
	end
	if item and widget then
		return item, self:get_widget_x(widget) + (self:get_widget_w(widget)/2), self:get_widget_y(widget) + self:get_widget_h(widget)
	end
	return nil
end

function InventoryMenu:ShowCurrentItemTooltip()
	if not show_info then
		return
	end
	
	if not IsGamepadActive() and self.suppressTooltip then
		self.suppressTooltip = nil
		return
	end
	
	local item, x, y = self:GetSlotData()
	if item then
		if type(item) == "userdata" then
			local format, data = item:GetToolTipFormatAndData(self.hero, true)
			open_custompopup_menu(format, data, x, y, 4, 256)
		elseif type(item) == "string" then
			local format, data = _G.GetQuestItemToolTipFormatAndData(item)
			open_custompopup_menu(format, data, x, y, 4, 256)
		elseif type(item) == "number" then
			local stat = STATS_ORDER[item]
			local showSpecialStatEffect = false
			if stat == "intelligence" then
				showSpecialStatEffect = true
			end
			
			local format, data = self:GetAttributeToolTipFormatAndData(stat, showSpecialStatEffect)
			open_custompopup_menu(format, data, x, y, 4, 256)
		end
	end
end

function InventoryMenu:GetAttributeToolTipFormatAndData(attr, addSpecialEffect)
	local fmt = ""
	local data = ""
	
	local WIDTH = 416
	
	local HEADING_Y = 3
	local HEADING_HEIGHT = 40
	
	local SUBTITLE_Y = HEADING_Y + HEADING_HEIGHT
	local SUBTITLE_HEIGHT = 30
	
	local SPECIALEFFECT_Y = SUBTITLE_Y + SUBTITLE_HEIGHT
	local SPECIALEFFECT_HEIGHT = 32

	local VALUEEFFECT_Y = SUBTITLE_Y + SUBTITLE_HEIGHT
	local VALUEEFFECT_HEIGHT = 32

	if addSpecialEffect then
		VALUEEFFECT_Y = SPECIALEFFECT_Y + SPECIALEFFECT_HEIGHT
	end
	
	local RANKEFFECT_Y = VALUEEFFECT_Y + VALUEEFFECT_HEIGHT
	local RANKEFFECT_HEIGHT = 32
	
	local STARTMANA_Y = RANKEFFECT_Y + RANKEFFECT_HEIGHT
	local STARTMANA_HEIGHT = 32
	
	local EARNEDMANA_Y = STARTMANA_Y + STARTMANA_HEIGHT
	local EARNEDMANA_HEIGHT = 32
	
	local MAXMANA_Y = EARNEDMANA_Y + EARNEDMANA_HEIGHT
	local MAXMANA_HEIGHT = 32	
	
	local EDGE_OFFSET = 3
	local BULLET_OFFSET = 10
	
	local hero_stats_constants = require( "Hero/StatsConstants" )
	local currentScore = self.hero:GetStat(attr)
	local currentRank = self.hero:GetStatPointsRanking(currentScore).rank
	
	local str_bullet = "img_levelup_" .. attr
	local upr_attr = string.upper(attr)
	local str_name   = translate_text(string.format("[%s]", upr_attr))
	
	-- Minigame bonus turns
	local str_special_effect
	if addSpecialEffect then
		str_special_effect   = translate_text(string.format("[%s_SPECIALEFFECT]", upr_attr))
	end
	
	local str_value_effect   = translate_text(string.format("[%s_VALUEFFECT]", upr_attr))
	local str_rank_effect    = translate_text(string.format("[%s_RANKEFFECT]", upr_attr))
	local str_mana1_effect   = translate_text(string.format("[%s_STARTMANAEFFECT]", upr_attr))
	local str_mana2_effect   = translate_text(string.format("[%s_EARNEDMANAEFFECT]", upr_attr))
	local str_mana3_effect   = translate_text(string.format("[%s_MAXMANAEFFECT]", upr_attr))
	
	local stat_special_effect = currentScore
	local stat_value_effect = currentScore
	local stat_rank_effect = currentRank
	local stat_mana_start_effect = math.floor( stat_rank_effect * hero_stats_constants.mana_start_multiplier )
	local stat_mana_earn_effect = math.floor( stat_rank_effect * hero_stats_constants.mana_earn_multiplier )
	local stat_mana_max_effect = math.floor( stat_rank_effect * hero_stats_constants.mana_max_multiplier )
	
	-- Title is the attribute name
	fmt = fmt .. "//SHADING/img_TextBackground_Red"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, HEADING_Y+EDGE_OFFSET, "font_uitext", str_name)
	fmt = fmt .. "//SHADING/off"
	
	-- Show the current levels amounts
	fmt = fmt .. "//SHADING/img_TextBackground_Purple"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, SUBTITLE_Y, "font_uitext", substitute(translate_text("[LEVELUP_CURRENT]"), tostring(currentScore)))
	fmt = fmt .. "//SHADING/off"
		
	if addSpecialEffect then
		-- spell penetration
		fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, SPECIALEFFECT_Y+4, 24, 24, str_bullet)
		fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, SPECIALEFFECT_Y, WIDTH-32, SPECIALEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_special_effect, stat_special_effect))
	end
	
	-- N% chance for weapon critical / defence / Minigame turns / shield critical / spell resist
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, VALUEEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, VALUEEFFECT_Y, WIDTH-32, VALUEEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_value_effect, stat_value_effect))
	
	-- +N Damage from Skull Gems / + action points / + all max manas / + lifepoints / + weapon damage
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, RANKEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, RANKEFFECT_Y, WIDTH-32, RANKEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_rank_effect, stat_rank_effect))
	
	-- +N Start mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, STARTMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, STARTMANA_Y, 238, STARTMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana1_effect, stat_mana_start_effect))
	
	-- +N Earned Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, EARNEDMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, EARNEDMANA_Y, WIDTH-32, EARNEDMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana2_effect, stat_mana_earn_effect))
	
	-- +N Max Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, MAXMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, MAXMANA_Y, WIDTH-32, MAXMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana3_effect, stat_mana_max_effect))
	
	return fmt .. "//", data .. "//"
end

function InventoryMenu:OpenPopup(item, x, y)
	if item and (type(item) == "userdata") and (not item:IsTrade()) then
		SCREENS.PopupMenu:Open()
		local button_table = { }
		for k,v in ipairs(item:GetEquipCommands(self.hero, function()
			SCREENS.PopupMenu:Close()
			self:SetItems(self.hero)
			Listbox.UpdateList("items")
			self:RefreshStats(self.hero)
		end)) do
			table.insert(button_table, v)
		end
--		table.insert(button_table, { tooltip="Send To Friend", function (params)
--				LOGF("show_custom_message_ui: %s", tostring(XboxCustomMessage.show_custom_message_ui({
--					userIndex = 0,
--					recipient = entry.xuid,
--				})))
--		end})
		table.insert(button_table, { tooltip="[CANCEL]",
		callback = function(params)
			SCREENS.PopupMenu:Close()
		end } )
		SCREENS.PopupMenu:AddActions(button_table, {item = item})
		SCREENS.PopupMenu:SetListPosition(x, 768 - y)
	end
end

function InventoryMenu:MoveGamepadFocus(new_location, dir)
	new_location = new_location or gp_navigation[gp_mode.area][dir]
	assert(gp_navigation[new_location], string.format("Gamepad moved to unknown location %s", tostring(new_location)))
	
	close_custompopup_menu()
	
	if gp_navigation[new_location].enabled == false then
		if dir and gp_navigation[new_location][dir] then
			self:MoveGamepadFocus(gp_navigation[new_location][dir], dir)
		end
		return
	end
	
	if gp_navigation[gp_mode.area].widget and gp_navigation[gp_mode.area].highlight then
		self:hide_widget("icon_cursor")
	end
	
	local data = gp_navigation[new_location]
	if (gp_mode.area == "items") and (new_location ~= "items") then
		Listbox.SwitchOff("items")
		self:hide_widget("icon_cursor")
	end
	
	if (gp_mode.area ~= "items") and (new_location == "items") then
		Listbox.SwitchOn("items", true)
		self.suppressTooltip = true
		self:SetCursorPosition(gp_mode.field)
	end
	
	gp_mode.area = new_location
	if data.widget then
		local widget = data.widget
		MenuSystem.SetGamepadWidget(self, widget)
		if data.highlight then
			self:ShowHighlight(widget)
			self:ShowCurrentItemTooltip()
		end
	end
end

function InventoryMenu:OnTimer(time)
	if _G.Joystick then
		_G.Joystick.OnTimer(self, time)
	end
	return Menu.MESSAGE_HANDLED
end

return ExportSingleInstance("InventoryMenu")