use_safeglobals()

local FX = require("FXContainer")
local Constants = require("Battlegrounds/BattleGroundConstants")
local MenuHelpers = require("Utils/MenuHelpers")

local POOLS					= Constants.MANA_POOL_NAMES
local MAX_SPELLS			= 5
local MAX_ITEMS				= 2
local MAX_STATUS_EFFECTS	= 10
local RUMBLE_INTENSITY		= 0.25
local RUMBLE_TIME			= 1000

local ITEM_DESELECT_ALPHA = 0.6

class "PlayerInfoMenu" (Menu);

function PlayerInfoMenu:__init(num)
	super()
	
	self:Initialize("Assets\\Screens\\PlayerInfoMenu.xml")
	self.current = {}
	self.target = {}
	self.currentManaBarState = {}
	self.currentManaBarTarget = {}
	self.statusEffects = {}
	self.moveID = 1
	self.items = {}
	self.spells = {}
	self.snapshot = nil
	self.canUseItems = 0			-- these are reference counted, because multiple Lesser Stuns could be placed on a hero
	self.canUseSpells = 0
	self.itemCostModifier = 1		-- adjusted as we go in and out of stealth mode
	self.sounds = {}
end

function PlayerInfoMenu:GetItemFromIndex(slot)
	if slot > 0 then
		return self.items[slot]
	elseif slot == Items.MAIN_HAND then
		return self.mainHand
	elseif slot == Items.OFF_HAND then
		return self.offHand
	end
	-- otherwise return nil
end

function PlayerInfoMenu:OnOpen()
	LOG("[PlayerInfoMenu:OnOpen] Started")
	
	for i=1, MAX_SPELLS do
		scheduler:AddChildThread(self, function()
			while true do
				-- if self.canUseSpells == 0 and self:IsInputAllowed() then
				local widgetName = string.format("icon_spell%d_ready", i)
				if self.snapshot and self.canUseSpells == 0 and self.spells and self.spells[i] and self:CheckManaRequirementsForSpell(i) and self:CheckUseCountRequirementsForSpell(i) then
					if self:CheckCooldownForSpell(i) then
						MenuHelpers.FadeUpWidget(self, self.client, widgetName, 0.0, 0.5, 0.5)
						scheduler:Sleep(1000)
						MenuHelpers.FadeDownWidget(self, self.client, widgetName, 0.5, 0.0, 0.5)
						scheduler:Sleep(1000)
					else
						self:set_alpha(widgetName, 0.5)
						coroutine.yield()
					end
				else
					self:set_alpha(widgetName, 0.0)
					coroutine.yield()
				end
			end
		end)
	end
	
	scheduler:AddChildThread(self, function()
		while true do
			-- if self.canUseItems == 0 and self:IsInputAllowed() then
			if self.snapshot and self.canUseItems == 0 and self.mainHand and self.itemCostModifier and self.snapshot.power >= (self.mainHand:GetActionPower() * self.itemCostModifier) then
				local widgetName = "icon_weapon1_ready"
				MenuHelpers.FadeUpWidget(self, self.client, widgetName, 0.0, 0.5, 0.5)
				scheduler:Sleep(1000)
				MenuHelpers.FadeDownWidget(self, self.client, widgetName, 0.5, 0.0, 0.5)
				scheduler:Sleep(1000)
			else
				coroutine.yield()
			end
		end
	end)
	
	scheduler:AddChildThread(self, function()
		while true do
			-- if self.canUseItems == 0 and self:IsInputAllowed() then
			if self.snapshot and self.canUseItems == 0 and self.offHand and self.itemCostModifier and self.snapshot.power >= ((self.offHand:IsStealthy()) and (self.offHand:GetActionPower()) or (self.offHand:GetActionPower() * self.itemCostModifier)) then
				local widgetName = "icon_weapon2_ready"
				MenuHelpers.FadeUpWidget(self, self.client, widgetName, 0.0, 0.5, 0.5)
				scheduler:Sleep(1000)
				MenuHelpers.FadeDownWidget(self, self.client, widgetName, 0.5, 0.0, 0.5)
				scheduler:Sleep(1000)
			else
				coroutine.yield()
			end
		end
	end)
	
	LOG("[PlayerInfoMenu:OnOpen] Ended")

	return Menu.OnOpen(self)
end

function PlayerInfoMenu:OnRefresh()
	-- move the menu first
	SCREENS.BattleGameMenu:RefreshClientMenus()
	
	self.current = {}
	self.target = {}
	self.currentManaBarState = {}
	self.currentManaBarTarget = {}
	self.currentHealthBarState = nil
	
	local targetObject = SCREENS.EffectsMenu:GetWorld()
	
	if self.psys then
		for i, psys in pairs(self.psys) do
			for j, sys in pairs(psys) do
				targetObject:DeleteParticles(self.psys[i][j])
				self.psys[i][j] = nil
			end
		end
		self.psys = nil
	end
	
	for i=1, MAX_SPELLS do
		self:set_alpha(string.format("str_spell%d", i), 0.0)
	end
	
	self:hide_widget("icon_turn")
	self:hide_widget("icon_turn_top")
	self:hide_widget("icon_turn_flame")
	self:hide_widget("icon_portrait_frame_highlight1")
	self:hide_widget("icon_portrait_frame_highlight2")
	self:hide_widget("icon_timer")
	self:hide_widget("icon_item1")
	self:hide_widget("icon_item2")
	self:hide_widget("icon_item3")
	self:hide_widget("icon_item4")
	self:hide_widget("icon_weapon1")
	self:hide_widget("icon_weapon2")
	
	self:hide_widget("icon_weapon1_highlight")
	self:hide_widget("icon_weapon2_highlight")
	self:hide_widget("icon_item1_highlight")
	self:hide_widget("icon_item2_highlight")
	self:hide_widget("icon_item3_highlight")
	self:hide_widget("icon_item4_highlight")
	self:hide_widget("icon_spell1_highlight")
	self:hide_widget("icon_spell2_highlight")
	self:hide_widget("icon_spell3_highlight")
	self:hide_widget("icon_spell4_highlight")
	self:hide_widget("icon_spell5_highlight")
	
	self:set_alpha("icon_spell1_ready", 0.0)
	self:set_alpha("icon_spell2_ready", 0.0)
	self:set_alpha("icon_spell3_ready", 0.0)
	self:set_alpha("icon_spell4_ready", 0.0)
	self:set_alpha("icon_spell5_ready", 0.0)
	self:set_alpha("icon_weapon1_ready", 0.0)
	self:set_alpha("icon_weapon2_ready", 0.0)

	for i=1, MAX_STATUS_EFFECTS do
		self:hide_widget(string.format("icon_statuseffect%d_highlight", i))
	end
	
	self:DisableAllSpellAndItemButtons() -- initially disabled
	self:RedrawStatusEffects() -- initially, turns them all off!
	
	self:set_image("icon_portrait", "")
	
	self:set_image("icon_cost_green1", "img_spellgem_green_off")
	self:set_image("icon_cost_red1", "img_spellgem_red_off")
	self:set_image("icon_cost_yellow1", "img_spellgem_yellow_off")
	self:set_image("icon_cost_blue1", "img_spellgem_blue_off")
	self:set_image("icon_cost_black1", "img_spellgem_black_off")
	self:set_image("icon_cost_green2", "img_spellgem_green_off")
	self:set_image("icon_cost_red2", "img_spellgem_red_off")
	self:set_image("icon_cost_yellow2", "img_spellgem_yellow_off")
	self:set_image("icon_cost_blue2", "img_spellgem_blue_off")
	self:set_image("icon_cost_black2", "img_spellgem_black_off")
	self:set_image("icon_cost_green3", "img_spellgem_green_off")
	self:set_image("icon_cost_red3", "img_spellgem_red_off")
	self:set_image("icon_cost_yellow3", "img_spellgem_yellow_off")
	self:set_image("icon_cost_blue3", "img_spellgem_blue_off")
	self:set_image("icon_cost_black3", "img_spellgem_black_off")
	self:set_image("icon_cost_green4", "img_spellgem_green_off")
	self:set_image("icon_cost_red4", "img_spellgem_red_off")
	self:set_image("icon_cost_yellow4", "img_spellgem_yellow_off")
	self:set_image("icon_cost_blue4", "img_spellgem_blue_off")
	self:set_image("icon_cost_black4", "img_spellgem_black_off")
	self:set_image("icon_cost_green5", "img_spellgem_green_off")
	self:set_image("icon_cost_red5", "img_spellgem_red_off")
	self:set_image("icon_cost_yellow5", "img_spellgem_yellow_off")
	self:set_image("icon_cost_blue5", "img_spellgem_blue_off")
	self:set_image("icon_cost_black5", "img_spellgem_black_off")
	
	if self.client and self.client.hero then
		local offset = GetScreenOffset()
		if offset == 0 then	-- not widescreen
			self.portrait = self.client.hero:GetAttribute("portrait")
		else
			self.portrait = string.format("%s_WS", self.client.hero:GetAttribute("portrait"))
		end
		self:set_image("icon_portrait", self.portrait)
	end
	
	if self.snapshot then
		self:UpdateUI(self.snapshot)
	end
	
	for index, item in pairs(self.items) do
		local widgetName = string.format("icon_item%d", index)
		self:activate_widget(widgetName)
		self:set_image(widgetName, item:GetIconAsset())
		self:set_image(widgetName .. "_overlay", item:GetIconOverlayAsset())
		self:set_alpha(widgetName, ITEM_DESELECT_ALPHA)
	end
	
	if self.mainHand then
		self:set_text_raw("butt_weapon1", self.mainHand:GetName())
		self:set_image("icon_weapon1", self.mainHand:GetIconAsset())
		self:set_image("icon_weapon1_overlay", self.mainHand:GetIconOverlayAsset())
		self:activate_widget("icon_weapon1")
		self:set_alpha("icon_weapon1", ITEM_DESELECT_ALPHA)
	end
	
	if self.offHand then
		self:set_text_raw("butt_weapon2", self.offHand:GetName())
		self:set_image("icon_weapon2", self.offHand:GetIconAsset())
		self:set_image("icon_weapon2_overlay", self.offHand:GetIconOverlayAsset())
		self:activate_widget("icon_weapon2")
		self:set_alpha("icon_weapon2", ITEM_DESELECT_ALPHA)
	end
	
	if self.turn_indicator then
		self:StopTurnIndicator()
		self:StartTurnIndicator()
	end
end

function PlayerInfoMenu:OnClose()
	scheduler:StopAllThreadsForObject(self)
	
	-- just in case it has been left open
	self:CloseSpellTargetMenu()
	
	-- reset bitmap image source rects
	self:ResetBitmapImageRects()
	
	-- Remove any objects created by this screen
	if self.mainHand then
		GameObjectManager:Destroy(self.mainHand)
	end
	
	if self.offHand then
		GameObjectManager:Destroy(self.offHand)
	end
	
	for index, item in pairs(self.items) do
		GameObjectManager:Destroy(item)
	end
	
	self.mainHand = nil
	self.offHand = nil
	self.items = nil
	
	local targetObject = SCREENS.EffectsMenu:GetWorld()
	
	if self.stun_thread then
		scheduler:StopThread(self.stun_thread)
		self.stun_thread = nil
	end
	
	if self.stun_icons then
		for i in ipairs(self.stun_icons) do
			local overlay = string.format("%s_stun_icon%d", tostring(self.client), i)
			if targetObject:GetOverlay(overlay).__ok then
				targetObject:RemoveOverlay(overlay)
			end
		end
		self.stun_icons = nil
	end
	
	if self.psys then
		for i, psys in pairs(self.psys) do
			for j, sys in pairs(psys) do
				targetObject:DeleteParticles(self.psys[i][j])
				self.psys[i][j] = nil
			end
		end
	end
	
	if self.sounds then
		for _, sound in ipairs(self.sounds) do
			SoundSystem.Unload(sound)
		end
	end
	
	return Menu.OnClose(self)
end

function PlayerInfoMenu:FadeDown()
	MenuHelpers.FadeDownWidget(self, self.client, "grp_Board", 1.0, 0.25, 0.75)
end

function PlayerInfoMenu:OnEventImmediateDamageNotify(event)
	local posx = self:GetX() + self:get_widget_x("icon_portrait") + self:get_widget_w("icon_portrait") / 2
	local posy = 768 - (self:GetY() + self:get_widget_y("icon_portrait") + self:get_widget_h("icon_portrait") - 32)
	--local posx = self:GetX() + self:get_widget_x("prog_health") + self:get_widget_w("prog_health") / 2
	--local posy = 768 - (self:GetY() + self:get_widget_y("prog_health") + self:get_widget_h("prog_health") / 2)
	
	local numfx = FX.CreateContainer(1000, 0)
	local numtxt = string.format("-%d", event:GetAttribute("amount"))
	local text = FX.AddText(numfx, "font_tiny_event", numtxt, 0, 0, 1, 1, 0, 1.0, true, 0.0)
	--local text = FX.AddText(numfx, "font_tiny_event", event:GetAttribute("source"), 0, 0, 1, 1, 0, 1.0, true, 0.0)
	
	FX.AddKey(numfx, text, 0, FX.KEY_ALPHA, 1)
	FX.AddKey(numfx, text, 1000, FX.KEY_ALPHA, 1)
	FX.AddKey(numfx, text, 2000, FX.KEY_ALPHA, 0, FX.PUNCH_IN)
	FX.AddKey(numfx, text, 0, FX.KEY_SCALE, 1.2)
	FX.AddKey(numfx, text, 100, FX.KEY_SCALE, 1.0, FX.PUNCH_IN)
	
	FX.Start(SCREENS.EffectsMenu:GetWorld(), numfx, posx-_G.GetScreenOffset(), posy)
	
	local player = self.client:GetAttribute("player_id")
	if player >= 1 and player <= 4 then
		AddRumble(PlayerToUser(player), RUMBLE_INTENSITY, RUMBLE_TIME)
	end

	
	local absorbed = event:GetAttribute("absorbed")
	local amount = event:GetAttribute("amount")
	
	if amount - absorbed > 0 then
		if SCREENS.BattleGameMenu:GetWorld():IsLocalClient(event:GetAttribute("client")) then
			SCREENS.EffectsMenu:PlayDamageFlashEffect(event:GetAttribute("amount"))
		end
	else
		-- no effect if the damage was all absorbed
	end
end

function PlayerInfoMenu:OnEventImmediateRestoreNotify(event)
	local posx = self:GetX() + self:get_widget_x("icon_portrait") + self:get_widget_w("icon_portrait") / 2
	local posy = 768 - (self:GetY() + self:get_widget_y("icon_portrait") + 10)
	
	local numfx = FX.CreateContainer(1000, 0)
	local numtxt = string.format("+%d", event:GetAttribute("amount"))
	local text = FX.AddText(numfx, "font_tiny_event", numtxt, 0, 0, 1, 1, 0, 1.0, true, 0.0)
	
	FX.AddKey(numfx, text, 0, FX.KEY_ALPHA, 1)
	FX.AddKey(numfx, text, 1000, FX.KEY_ALPHA, 1)
	FX.AddKey(numfx, text, 2000, FX.KEY_ALPHA, 0, FX.PUNCH_IN)
	FX.AddKey(numfx, text, 0, FX.KEY_SCALE, 1.2)
	FX.AddKey(numfx, text, 100, FX.KEY_SCALE, 1.0, FX.PUNCH_IN)
	
	FX.Start(SCREENS.EffectsMenu:GetWorld(), numfx, posx-_G.GetScreenOffset(), posy)
	
	if SCREENS.BattleGameMenu:GetWorld():IsLocalClient(event:GetAttribute("client")) then
		SCREENS.EffectsMenu:PlayHealFlashEffect(event:GetAttribute("amount"))
	end
end

function PlayerInfoMenu:OnEventAccumulateDamageNotify(event)
	local amount = event:GetAttribute("amount")
	
	if amount <= 0 then
		return -- we do not need to display no damage
	end
	
	local posx = self:GetX() + self:get_widget_x("icon_portrait") + self:get_widget_w("icon_portrait") / 2
	local posy = 768 - (self:GetY() + self:get_widget_y("icon_portrait") + self:get_widget_h("icon_portrait") / 2)
	
	if not self.accumulatedDamageFXC then
		self.accumulatedDamageFXC = FX.CreateContainer(10000, 0)
		self.accumulatedDamageText = FX.AddText(self.accumulatedDamageFXC, "font_small_event", "0", 0, 0, 1, 1, 0, 1.0, true, 1.0)
		self.accumulatedDamageAmount = 0
		self.accumulatedDamageStartTime = GetGameTime()
		self.accumulatedDamageOwner = {}
		
		FX.Start(SCREENS.EffectsMenu:GetWorld(), self.accumulatedDamageFXC, posx-_G.GetScreenOffset(), posy)
	end
	
	local player = self.client:GetAttribute("player_id")
	if player >= 1 and player <= 4 then
		AddRumble(PlayerToUser(player), RUMBLE_INTENSITY, RUMBLE_TIME)
	end

	scheduler:AddChildThread(self.accumulatedDamageOwner, function ()
		while amount > 0 do
			scheduler:Sleep(25)
			
			self.accumulatedDamageAmount = self.accumulatedDamageAmount + 1
			amount = amount - 1
			
			FX.ChangeText(self.accumulatedDamageFXC, self.accumulatedDamageText, tostring(self.accumulatedDamageAmount))
		end
	end, "OnEventAccumulateDamageNotify")
	
	-- plus a little number
	local numfx = FX.CreateContainer(1000, 0)
	local text = FX.AddText(numfx, "font_tiny_event", tostring(amount), 0, 0, 1, 1, 0, 1.0, true, 0.5)
	
	FX.AddKey(numfx, text, 0, FX.KEY_ALPHA, 1)
	FX.AddKey(numfx, text, 1000, FX.KEY_ALPHA, 0, FX.PUNCH_IN)
	FX.AddKey(numfx, text, 0, FX.KEY_SCALE, 1.2)
	FX.AddKey(numfx, text, 100, FX.KEY_SCALE, 1.0, FX.PUNCH_IN)
	
	FX.Start(SCREENS.EffectsMenu:GetWorld(), numfx, posx-48, posy+48)
	
	if not event:GetAttribute("stealthed") and SCREENS.BattleGameMenu:GetWorld():IsLocalClient(event:GetAttribute("client")) then
		SCREENS.EffectsMenu:PlayDamageFlashEffect(event:GetAttribute("amount"))
	end
end

function PlayerInfoMenu:OnEventApplyAccumulateDamageNotify(event)

	-- [SCF] This assert was firing when the INSTANT WIN cheat was activated
	--       I'm just sidestepping it for the moment
	--       But my dodgy HaX0r code needs to be removed if it accidentally gets committed
	if not self.accumulatedDamageFXC then
		return
	end
	-- End HaX0r
	
	assert(self.accumulatedDamageFXC)	-- apply accumulation event, but no damage was accumulated?
	
	
	
	local amount = event:GetAttribute("amount")
	local blocked = event:GetAttribute("blocked")
	local stealthed = event:GetAttribute("stealthed")
	
	FX.ChangeText(self.accumulatedDamageFXC, self.accumulatedDamageText, tostring(amount))
	
	if stealthed then
		scheduler:StopAllThreadsForObject(self.accumulatedDamageOwner)
		scheduler:AddChildThread(self, function () 
		
			local accumulatedDamageFXC = self.accumulatedDamageFXC
			local accumulatedDamageText = self.accumulatedDamageText
			local accumulatedDamageAmount = self.accumulatedDamageAmount
			local accumulatedDamageStartTime = self.accumulatedDamageStartTime
			local accumulatedDamageOwner = self.accumulatedDamageOwner
			
			-- These are reset immediately, because another accumulate damage could
			-- come through and add to the accumulatedDamageFXC, which would then
			-- fade out, so when the Apply comes through for the earlier accumulate
			-- there might not be an FXC container as the logic expects.
			
			self.accumulatedDamageFXC = nil
			self.accumulatedDamageText = nil
			self.accumulatedDamageAmount = nil
			self.accumulatedDamageStartTime = nil
			self.accumulatedDamageOwner = nil
			
			accumulatedDamageAmount = amount
			
			scheduler:Sleep(1000)

			while accumulatedDamageAmount > 0 do
				scheduler:Sleep(25)
				accumulatedDamageAmount = accumulatedDamageAmount - 1
				FX.ChangeText(accumulatedDamageFXC, accumulatedDamageText, tostring(accumulatedDamageAmount))
			end
			
			scheduler:Sleep(1000)
		
			local startFadeTime = GetGameTime() - accumulatedDamageStartTime
			local endFadeTime = startFadeTime + 1000

			FX.AddKey(accumulatedDamageFXC, accumulatedDamageText, startFadeTime, FX.KEY_ALPHA, 1)
			FX.AddKey(accumulatedDamageFXC, accumulatedDamageText, endFadeTime, FX.KEY_ALPHA, 0)
			
			scheduler:Sleep(1000)
			
			FX.Stop(accumulatedDamageFXC, false)
		end, "OnEventApplyAccumulateDamageNotify")
		
		local posx = self:GetX() + self:get_widget_x("icon_portrait") + self:get_widget_w("icon_portrait") / 2
		local posy = 768 - (self:GetY() + self:get_widget_y("icon_portrait") + self:get_widget_h("icon_portrait") / 2)
	
		local numfx = FX.CreateContainer(amount*25+2000, 0)
		local text = FX.AddText(numfx, "font_tiny_event", "[STEALTH_IGNORED_DAMAGE]", 0, 0, 1, 1, 0, 1.0, true, 0.0)
		
		FX.AddKey(numfx, text, 0, FX.KEY_ALPHA, 1)
		FX.AddKey(numfx, text, amount*25, FX.KEY_ALPHA, 1)
		FX.AddKey(numfx, text, amount*25+2000, FX.KEY_ALPHA, 0, FX.PUNCH_IN)
		FX.AddKey(numfx, text, 0, FX.KEY_SCALE, 1.2)
		FX.AddKey(numfx, text, 100, FX.KEY_SCALE, 1.0, FX.PUNCH_IN)
		
		FX.Start(SCREENS.EffectsMenu:GetWorld(), numfx, posx, posy)
	elseif event:GetAttribute("blocked") == 0 then
		local startFadeTime = GetGameTime() - self.accumulatedDamageStartTime
		local endFadeTime = startFadeTime + 1000

		FX.AddKey(self.accumulatedDamageFXC, self.accumulatedDamageText, startFadeTime, FX.KEY_ALPHA, 1)
		FX.AddKey(self.accumulatedDamageFXC, self.accumulatedDamageText, endFadeTime, FX.KEY_ALPHA, 0)
		
		local accumulatedDamageFXC = self.accumulatedDamageFXC
		local accumulatedDamageOwner = self.accumulatedDamageOwner
		
		-- These are reset immediately, because another accumulate damage could
		-- come through and add to the accumulatedDamageFXC, which would then
		-- fade out, so when the Apply comes through for the earlier accumulate
		-- there might not be an FXC container as the logic expects.
		
		self.accumulatedDamageFXC = nil
		self.accumulatedDamageText = nil
		self.accumulatedDamageAmount = nil
		self.accumulatedDamageStartTime = nil
		self.accumulatedDamageOwner = nil
		
		scheduler:StopAllThreadsForObject(accumulatedDamageOwner)
		scheduler:AddChildThread(self, function () 
			scheduler:Sleep(1000)
			
			FX.Stop(accumulatedDamageFXC, false)
		end, "OnEventApplyAccumulateDamageNotify")
	else -- player blocked some damage
		scheduler:StopAllThreadsForObject(self.accumulatedDamageOwner)
		scheduler:AddChildThread(self, function () 
			
			SoundSystem.Play("pq2audio/board/fx_block")
			
			local accumulatedDamageFXC = self.accumulatedDamageFXC
			local accumulatedDamageText = self.accumulatedDamageText
			local accumulatedDamageAmount = self.accumulatedDamageAmount
			local accumulatedDamageStartTime = self.accumulatedDamageStartTime
			local accumulatedDamageOwner = self.accumulatedDamageOwner
			
			-- These are reset immediately, because another accumulate damage could
			-- come through and add to the accumulatedDamageFXC, which would then
			-- fade out, so when the Apply comes through for the earlier accumulate
			-- there might not be an FXC container as the logic expects.
			
			self.accumulatedDamageFXC = nil
			self.accumulatedDamageText = nil
			self.accumulatedDamageAmount = nil
			self.accumulatedDamageStartTime = nil
			self.accumulatedDamageOwner = nil
			
			local startShakeTime = GetGameTime() - accumulatedDamageStartTime
		
			for i=0,500,25 do
				FX.AddKey(accumulatedDamageFXC, accumulatedDamageText, startShakeTime+i, FX.KEY_XY, math.random(-2,2), math.random(-2,2))
			end
			
			accumulatedDamageAmount = amount
			
			scheduler:Sleep(1000)

			while accumulatedDamageAmount > (amount - blocked) do
				scheduler:Sleep(25)
				accumulatedDamageAmount = accumulatedDamageAmount - 1
				FX.ChangeText(accumulatedDamageFXC, accumulatedDamageText, tostring(accumulatedDamageAmount))
			end
			
			scheduler:Sleep(1000)
		
			local startFadeTime = GetGameTime() - accumulatedDamageStartTime
			local endFadeTime = startFadeTime + 1000

			FX.AddKey(accumulatedDamageFXC, accumulatedDamageText, startFadeTime, FX.KEY_ALPHA, 1)
			FX.AddKey(accumulatedDamageFXC, accumulatedDamageText, endFadeTime, FX.KEY_ALPHA, 0)
			
			scheduler:Sleep(1000)
			
			FX.Stop(accumulatedDamageFXC, false)
		end, "OnEventApplyAccumulateDamageNotify")
		
		local posx = self:GetX() + self:get_widget_x("icon_portrait") + self:get_widget_w("icon_portrait") / 2
		local posy = 768 - (self:GetY() + self:get_widget_y("icon_portrait") + self:get_widget_h("icon_portrait") / 2)
	
		local numfx = FX.CreateContainer(1000, 0)
		local text = FX.AddText(numfx, "font_tiny_event", "[BLOCKED]", 0, 0, 1, 1, 0, 1.0, true, 0.0)
		
		FX.AddKey(numfx, text, 0, FX.KEY_ALPHA, 1)
		FX.AddKey(numfx, text, 1000, FX.KEY_ALPHA, 1)
		FX.AddKey(numfx, text, 2000, FX.KEY_ALPHA, 0, FX.PUNCH_IN)
		FX.AddKey(numfx, text, 0, FX.KEY_SCALE, 1.2)
		FX.AddKey(numfx, text, 100, FX.KEY_SCALE, 1.0, FX.PUNCH_IN)
		
		FX.Start(SCREENS.EffectsMenu:GetWorld(), numfx, posx-_G.GetScreenOffset(), posy)
		
		if not stealthed and SCREENS.BattleGameMenu:GetWorld():IsLocalClient(event:GetAttribute("client")) then
			SCREENS.EffectsMenu:PlayBlockFlashEffect(amount)
		end
	end
end

function PlayerInfoMenu:OnEventAbsorbedDamageNotify(event)
	-- [NEOBEN] This could have been nicer, like the BLOCKED! effect, but its the best I can do for now
	local posx = self:GetX() + self:get_widget_x("icon_portrait") + self:get_widget_w("icon_portrait") / 2
	local posy = 768 - (self:GetY() + self:get_widget_y("icon_portrait") + self:get_widget_h("icon_portrait") / 2)
	
	local numfx = FX.CreateContainer(1000, 0)
	local text = FX.AddText(numfx, "font_tiny_event", "[ABSORBED]", 0, 0, 1, 1, 0, 1.0, true, 0.0)
	
	FX.AddKey(numfx, text, 0, FX.KEY_ALPHA, 1)
	FX.AddKey(numfx, text, 1000, FX.KEY_ALPHA, 1)
	FX.AddKey(numfx, text, 2000, FX.KEY_ALPHA, 0, FX.PUNCH_IN)
	FX.AddKey(numfx, text, 0, FX.KEY_SCALE, 1.2)
	FX.AddKey(numfx, text, 100, FX.KEY_SCALE, 1.0, FX.PUNCH_IN)
	
	FX.Start(SCREENS.EffectsMenu:GetWorld(), numfx, posx-_G.GetScreenOffset(), posy)
end

function PlayerInfoMenu:OnEventAddStatusEffect(event)
	local id = event:GetAttribute("id")
	local effect = {
		id = id,
		name = event:GetAttribute("name"),
		prettyName = event:GetAttribute("prettyName"),
--		turns = event:GetAttribute("turns"),
		def = require(string.format("StatusEffects.%s", event:GetAttribute("name"))),
		reason = event:GetAttribute("reason"),
	}
	
	local turns = event:GetAttribute("turns")
	if turns >= 0 then
		effect.turns = turns
	end
	
	-- some status effects change the UI
	local method = string.format("OnAddStatusEffect%s", effect.name)
	
	if self[method] then
		self[method](self, effect)
	end
	
	self.statusEffects[id] = effect
	
	if effect.def.LoadParams then
		effect.params = self.statusEffects[id].def.LoadParams(event)
	else
		effect.params = {}
	end
	
	effect.params.turnsLeft = effect.turns
	
	self:RedrawStatusEffects()
	
	-- spawn an FX container to highlight the fact that a new status effect has appeared
	local fxc = FX.CreateContainer(2400, 1, "img_blank", 1000)
	local img = FX.AddImage(fxc, STATUS[effect.name].icon, 0, 0, 1, 1, 0, 1, 1)
	
	FX.AddKey(fxc, img, 0, FX.KEY_ALPHA, 0)
	FX.AddKey(fxc, img, 0, FX.KEY_SCALE, 0)
	FX.AddKey(fxc, img, 2000, FX.KEY_ALPHA, 0)
	FX.AddKey(fxc, img, 2000, FX.KEY_SCALE, 1)
	FX.AddKey(fxc, img, 2200, FX.KEY_SCALE, 2, FX.BOUNCE)
	FX.AddKey(fxc, img, 2200, FX.KEY_ALPHA, 0.9, FX.BOUNCE)
	FX.AddKey(fxc, img, 2400, FX.KEY_ALPHA, 0, FX.BOUNCE)
	FX.AddKey(fxc, img, 2400, FX.KEY_SCALE, 1, FX.BOUNCE)
	
	local widgetName = string.format("icon_statuseffect%d", effect.id)
	
	local x = self:get_widget_x(widgetName) + math.ceil(self:get_widget_w(widgetName) / 2) + self:GetX()
	local y = self:get_widget_y(widgetName) + math.ceil(self:get_widget_h(widgetName) / 2) + self:GetY()
	
	FX.Start(SCREENS.EffectsMenu:GetWorld(), fxc, x, 768-y)
end

function PlayerInfoMenu:OnEventRemoveStatusEffect(event)
	local effect = self.statusEffects[event:GetAttribute("id")]
	
	-- some status effects change the UI
	local method = string.format("OnRemoveStatusEffect%s", effect.name)
	
	if self[method] then
		self[method](self, effect)
	end
	
	self.statusEffects[event:GetAttribute("id")] = nil
	self:RedrawStatusEffects()
	
	-- was there a custom popup open with this status effect?
end

function PlayerInfoMenu:OnEventPingStatusEffect(event)
--[[ DISABLED FOR PRESS DEMO, NEED PROPER EFFECT ANYWAY
	local effect = self.statusEffects[event:GetAttribute("id")]
	
	if effect then -- this was a ping from a hidden status effect, ignore it
		-- CHEESY TEMP EFFECT
		local fxc = FX.CreateContainer(1000, 1, "img_black", 1000)
		local img = FX.AddImage(fxc, STATUS[effect.name].icon, 0, 0, 1, 1, 0, 1, 1)
		
		FX.AddKey(fxc, img, 1000, FX.KEY_SCALE, 25, FX.BOUNCE)
		FX.AddKey(fxc, img, 1000, FX.KEY_ALPHA, 0, FX.BOUNCE)
		FX.Start(SCREENS.BattleGameMenu:GetWorld(), fxc, 256, 256)
	end
]]
end

function PlayerInfoMenu:RedrawStatusEffects()
	for i=1,MAX_STATUS_EFFECTS do 
		self:hide_widget(string.format("icon_statuseffect%d", i))
	end
	
	self.displayedEffects = GroupStatusEffects(self.statusEffects)
	
	local i = 1
	for name, effects in pairs(self.displayedEffects) do
		local widget = string.format("icon_statuseffect%d", i)
		self:activate_widget(widget)
		self:set_image(widget, STATUS[name].icon)
		for _, effect in ipairs(effects) do
			effect.index = i
		end
		i = i + 1
	end
	
end

function PlayerInfoMenu:ShowStatusEffectPopup(slot, x, y, id)
	assert(slot)
	assert(x)
	assert(y)
	
	if not self.displayedEffects then
		return
	end
	
	local statusEffectArray = {}
	
	for k,v in pairs(self.displayedEffects) do
		table.insert(statusEffectArray, v)
	end
	
	local effects = statusEffectArray[slot]
	
	
	if not effects then
		return
	end
	
	local format, data = GetStatusEffectToolTip(effects)
	
	open_custompopup_menu(format, data, x, y, 4, 256, "PlayerInfoMenu", id)
end

function PlayerInfoMenu:UpdateString(name, value, label, fmt, ...)
	local extra = {...}
	
	self.target[name] = value
	
	if value ~= self.current[name] then
		scheduler:AddChildThread(self, function ()
			while self.target[name] ~= self.current[name] do
				if not self.current[name] then
					self.current[name] = self.target[name]
				elseif self.current[name] < self.target[name] then
					self.current[name] = self.current[name] + 1
				else
					self.current[name] = self.current[name] - 1
				end
				
				self:set_text_raw(label, string.format(fmt, self.current[name], unpack(extra)))
				
				-- different sleep periods depending on how much to go
				local diff = math.abs(self.target[name] - self.current[name])
				if diff > 0 then
					scheduler:Sleep(250 / diff)
				end
			end
		end, string.format("UpdateString thread: %s -> %d into %s", name, value, label))
	end
end

-- Converts a value between 0-1 into the range 84-31
local function ConvertNormalisedValueToManaGraphic(v)
	assert(v >= 0.0)
	assert(v <= 1.0)
	
	if v == 0 then
		return 0
	elseif v == 1 then
		return 52
	else
		return math.floor(v * 50 + 1)
	end
end

local function hdtv_to_virtual(x)
	return math.floor(x / (1080 / 768) + 0.5)
end

-- Tick up the mana bar over time
--  "name" is the mana pool name
--  "value" is the value between 0-1 representing how full the bar is
--  "label" is the widget to change
--  "asset_base" is the first part of the asset name (which has the number appended to it for the full asset name)

-- How the mana bars are updated
-- ---------------------------------
--
-- Each mana bar has the following assets:
--     * img_green_mana_filling is an asset of a full mana bar, but this is shown while the bar is filling
--     * img_green_mana_full_0 is an asset of THE full mana bar
--     * img_green_mana_full_n (32<n<82) is an asset that covers the filling image to show half full bars
--     * img_green_mana_full_115 is an asset that shows a fully empty mana bar
--
-- There is 50 levels of fill in the assets, so the bars can be in 52 states:
--     * 0 			- fully empty
--     * 1-51 		- filling
--     * 52			- completely full
--

function PlayerInfoMenu:UpdateManaBar(name, value, label, asset_base)
	if value ~= self.currentManaBarState[name] then
		self.currentManaBarTarget[name] = ConvertNormalisedValueToManaGraphic(value)
		
		scheduler:AddChildThread(self, function ()
			while self.currentManaBarTarget[name] ~= self.currentManaBarState[name] do
				if not self.currentManaBarState[name] then
					self.currentManaBarState[name] = self.currentManaBarTarget[name]
				elseif self.currentManaBarState[name] < self.currentManaBarTarget[name] then
					self.currentManaBarState[name] = self.currentManaBarState[name] + 1
				elseif self.currentManaBarState[name] > self.currentManaBarTarget[name] then
					self.currentManaBarState[name] = self.currentManaBarState[name] - 1
				end
				
				local current = self.currentManaBarState[name]
				local image = nil -- string.format("%s%d", asset_base, self.currentManaBarState[name])
				local height = 0
				
				if current == 0 then
					image = string.format("%s115", asset_base)
					
					local offset = GetScreenOffset()
					if offset == 0 then	-- not widescreen
						height = hdtv_to_virtual(115)
					else
						height = hdtv_to_virtual(190)
					end
				elseif current == 52 then
					image = string.format("%s0", asset_base)
					
					local offset = GetScreenOffset()
					if offset == 0 then	-- not widescreen
						height = hdtv_to_virtual(115)
					else
						height = hdtv_to_virtual(190)
					end
				else
					local offset = GetScreenOffset()
					if offset == 0 then	-- not widescreen
						local number = (52-self.currentManaBarState[name])+32
						image = string.format("%s%d", asset_base, number)
						height = hdtv_to_virtual(number)
					else
						local number = (48-self.currentManaBarState[name])+13
						image = string.format("%s%d", asset_base, number)
						height = hdtv_to_virtual(2 * number)
					end
				end
				
--				LOGF("Set %s to image %s and height %d (from state %d)", label, tostring(image), height, current)
				
				if not image then
					self:hide_widget(label)
				else
					self:activate_widget(label)
					self:set_image(label, image)
					self:set_widget_h(label, height)
				end
				
				-- different sleep periods depending on how much to go
				local diff = math.abs(self.currentManaBarTarget[name] - self.currentManaBarState[name])
				if diff > 0 and value < 1 then
					scheduler:Sleep(250 / diff)
				end
			end
		end, string.format("UpdateManaBar thread: %s -> %f into %s", name, value, label))
	end
end

function PlayerInfoMenu:UpdateHealthBar(value)
	if value ~= self.currentHealthBarState then
		self.currentHealthBarTarget = value
		
		scheduler:AddChildThread(self, function ()
			while self.currentHealthBarTarget ~= self.currentHealthBarState do
				if not self.currentHealthBarState then
					self.currentHealthBarState = self.currentHealthBarTarget
				elseif self.currentHealthBarState < self.currentHealthBarTarget then
					self.currentHealthBarState = self.currentHealthBarState + 1
				elseif self.currentHealthBarState > self.currentHealthBarTarget then
					self.currentHealthBarState = self.currentHealthBarState - 1
				end
				
				self:set_progress("prog_health", self.currentHealthBarState)
				
				-- different sleep periods depending on how much to go
				local diff = math.abs(self.currentHealthBarTarget - self.currentHealthBarState)
				if diff > 0 then
					scheduler:Sleep(250 / diff)
				end
			end
		end, string.format("UpdateHealthBar thread: health -> %d into prog_health", value))
	end
end

function PlayerInfoMenu:InitUI(event)
	assert(event)
	
	local offset = GetScreenOffset()
	if offset == 0 then	-- not widescreen
		self.portrait = event:GetAttribute("portrait")
	else
		self.portrait = string.format("%s_WS", event:GetAttribute("portrait"))
	end
	self:set_image("icon_portrait", self.portrait)
	self.heroParams = {
		weaponBonusDamage = event:GetAttribute("weapon_bonus_damage"),
		shieldBonusDefense = event:GetAttribute("shield_bonus_defence"),
	}
	
	for i, spell in IterateCollection(event, "spells") do
		self.spells[i] = spell
		
		local spellData = SPELLS[spell]
		if spellData.sound then
			local bank = string.format("%s_bank", spellData.sound)
			table.insert(self.sounds, bank)
			SoundSystem.Load(bank)
		end
	end
	
	self:ReformatSpellWidgets(1)
end

function PlayerInfoMenu:ReformatSpellWidgets(costModifier)
	assert(costModifier)

	for i=1,MAX_SPELLS do
		for _, pool in ipairs(Constants.MANA_POOL_NAMES) do
			if pool ~= "power" then
				self:hide_widget(string.format("str_cost_%s%d", pool, i))
				self:hide_widget(string.format("icon_cost_%s%d", pool, i))
			end
		end
	end
	
	if not self.psys then
		self.psys = {}
	end
	
	for i, name in pairs(self.spells) do
		if not self.psys[i] then
			self.psys[i] = {}
		end
		
		local spell = SPELLS[name]
		
		self:set_text(string.format("butt_spell%d", i), string.format("[%s_NAME]", spell.id))
		
		-- is this spell castable?
		local spellManaRequirementsMade = false
		
		if self.snapshot then -- we do refresh spell buttons before any player state arrives
			spellManaRequirementsMade = self:CheckManaRequirementsForSpell(i)
		end
		
		-- configure the little mana cost indicators
		for j, pool in ipairs(Constants.MANA_POOL_NAMES) do
			if spell.cost[pool] and spell.cost[pool] > 0 and pool ~= "power" then
				local poolCost = math.floor(spell.cost[pool] * costModifier)
				local labelName = string.format("str_cost_%s%d", pool, i)
				local iconName = string.format("icon_cost_%s%d", pool, i)
				self:activate_widget(labelName)
				self:activate_widget(iconName)
				self:set_text_raw(labelName, tostring(poolCost))
				
				if costModifier == 1 then
					self:set_font(labelName, "font_uitext")
				elseif costModifier > 1 then
					self:set_font(labelName, "font_uitext_red")
				elseif costModifier < 1 then
					self:set_font(labelName, "font_uitext_green")
				end
				
				-- change the mana image with 3 states, "not enough mana", "enough mana", "spell ready to cast"
				--[[if spellManaRequirementsMade then
					self:set_image(iconName, string.format("img_spellgem_%s_full", string.lower(pool)))
				elseif ]]
				
				--Tim: Changed to 2 states at artist's request, 'not enough mana' and 'enough mana'
				local targetObject = SCREENS.EffectsMenu:GetWorld()
				local player_x = self:GetX()
				local player_y = self:GetY()
				local icon_x = self:get_widget_x(iconName)
				local icon_y = self:get_widget_y(iconName)
				local icon_w = self:get_widget_w(iconName)
				local icon_h = self:get_widget_h(iconName)
				local x = player_x + icon_x + (icon_w/2)	-- widget position in screen space
				local y = player_y + icon_y + (icon_h/2)	-- widget position in screen space				
				local point = SCREENS.EffectsMenu:ConvertScreenCoordsToWorldCoords(x, y)
				x = point.x
				y = point.y
				
				if self.snapshot and (poolCost <= self.snapshot[pool]) then
					self:set_image(iconName, string.format("img_spellgem_%s_on", string.lower(pool)))
					if not self.psys[i][j] then
						self.psys[i][j] = AttachParticles(targetObject, string.format("Mana_%s", pool), x, y)
						
						local lineEffect = GameObjectManager:ConstructLocal("EFCT")
						local source_widget = string.format("str_%s_value", string.lower(pool))
						local source_x = self:get_widget_x(source_widget) + self:get_widget_w(source_widget) / 2 + self:GetX() - GetScreenOffset()
						local source_y = 768 - (self:get_widget_y(source_widget) + self:get_widget_h(source_widget) / 2 + self:GetY())
						
						lineEffect:SetInfo(pool)
						lineEffect:Fire(source_x, source_y, x, y, 1000)
						
						SCREENS.EffectsMenu:GetWorld():AddChild(lineEffect)
					end
				else
					self:set_image(iconName, string.format("img_spellgem_%s_off", string.lower(pool)))
					if self.psys[i][j] then
						targetObject:DeleteParticles(self.psys[i][j])
						self.psys[i][j] = nil
					end
				end
				
				if spellManaRequirementsMade then
					if not self.psys[i][j+10] then
						self.psys[i][j+10] = AttachParticles(targetObject, string.format("ManaReq_%s", pool), x, y)
					end
				else
					if self.psys[i][j+10] then
						targetObject:DeleteParticles(self.psys[i][j+10])
						self.psys[i][j+10] = nil
					end
				end
			end
		end
	end
end

function PlayerInfoMenu:SlotIndexToItem(slot)
	assert(slot)
	if slot > 0 then
		return self.items[slot]
	elseif slot == Items.MAIN_HAND then
		return self.mainHand
	elseif slot == Items.OFF_HAND then
		return self.offHand
	end
end

function PlayerInfoMenu:InitItemUI(event)
	assert(event)
	
	LOGF("NICK: InitItemUI(%d, %s, %s, %s)", event:GetAttribute("slot"), event:GetAttribute("baseType"), event:GetAttribute("race"), event:GetAttribute("rarity"))
	
	local slot = event:GetAttribute("slot")
	local item
	
	if event:GetAttribute("baseType") == "Custom" then
		item = CreateUniqueItem(event:GetAttribute("unique"))
	else
		item = CreateEquippableItem(event:GetAttribute("baseType"), event:GetAttribute("race"), event:GetAttribute("rarity"))
	end
	
	if slot > 0 then
		local itemIndex = slot
		local widgetName = string.format("icon_item%d", itemIndex)
		
		self:activate_widget(widgetName)
		self:set_image(widgetName, item:GetIconAsset())
		self:set_image(widgetName .. "_overlay", item:GetIconOverlayAsset())
		self:set_alpha(widgetName, ITEM_DESELECT_ALPHA)
		self.items[slot] = item
	elseif slot == Items.MAIN_HAND then
		self:set_text_raw("butt_weapon1", item:GetName())
		self:set_image("icon_weapon1", item:GetIconAsset())
		self:set_image("icon_weapon1_overlay", item:GetIconOverlayAsset())
		self:activate_widget("icon_weapon1")
		self:set_alpha("icon_weapon1", ITEM_DESELECT_ALPHA)
		self.mainHand = item
		
		local sound = item:GetActivationSound()
		if sound then
			local bank = string.format("%s_bank", sound)
			table.insert(self.sounds, bank)
			SoundSystem.Load(bank)
		end
	elseif slot == Items.OFF_HAND then
		self:set_text_raw("butt_weapon2", item:GetName())
		self:set_image("icon_weapon2", item:GetIconAsset())
		self:set_image("icon_weapon2_overlay", item:GetIconOverlayAsset())
		self:activate_widget("icon_weapon2")
		self:set_alpha("icon_weapon2", ITEM_DESELECT_ALPHA)
		self.offHand = item
		
		local sound = item:GetActivationSound()
		if sound then
			local bank = string.format("%s_bank", sound)
			table.insert(self.sounds, bank)
			SoundSystem.Load(bank)
		end
	else
		error(string.format("weird slot index: %d", slot))
	end
end

function PlayerInfoMenu:UpdateUI(snapshot)
	if self:get_progress_max("prog_health") ~= snapshot.max_health then
		self:set_progress_max("prog_health", snapshot.max_health)
	end
	
	if mp_is_in_session and mp_is_in_session() then
		if self.client:GetAttribute("local") then
			self:set_text_raw("str_name_value", mp_state:GetPlayer1Name())
		else
			self:set_text_raw("str_name_value", mp_state:GetPlayer2Name())
		end
	else
		self:set_text_raw("str_name_value", snapshot.name)
	end
	
	self:UpdateString("health", snapshot.health, "str_health_value", "%d/%d", snapshot.max_health)
	self:UpdateHealthBar(snapshot.health)
	
	self:UpdateString("red", snapshot.red, "str_red_value", "%d")
	self:UpdateString("green", snapshot.green, "str_green_value", "%d")
	self:UpdateString("blue", snapshot.blue, "str_blue_value", "%d")
	self:UpdateString("yellow", snapshot.yellow, "str_yellow_value", "%d")
	self:UpdateString("black", snapshot.black, "str_black_value", "%d")
	self:UpdateString("power", snapshot.power, "str_power_value", "%d")
	
	local offset = GetScreenOffset()
	if offset == 0 then	-- not widescreen
		self:UpdateManaBar("red", snapshot.red / snapshot.actual_max_red, "icon_red", "img_red_mana_full_")
		self:UpdateManaBar("green", snapshot.green / snapshot.actual_max_green, "icon_green", "img_green_mana_full_")
		self:UpdateManaBar("blue", snapshot.blue / snapshot.actual_max_blue, "icon_blue", "img_blue_mana_full_")
		self:UpdateManaBar("yellow", snapshot.yellow / snapshot.actual_max_yellow, "icon_yellow", "img_yellow_mana_full_")
		self:UpdateManaBar("black", snapshot.black / snapshot.actual_max_black, "icon_black", "img_black_mana_full_")
	else
		self:UpdateManaBar("red", snapshot.red / snapshot.actual_max_red, "icon_red", "img_red_mana_full_ws_")
		self:UpdateManaBar("green", snapshot.green / snapshot.actual_max_green, "icon_green", "img_green_mana_full_ws_")
		self:UpdateManaBar("blue", snapshot.blue / snapshot.actual_max_blue, "icon_blue", "img_blue_mana_full_ws_")
		self:UpdateManaBar("yellow", snapshot.yellow / snapshot.actual_max_yellow, "icon_yellow", "img_yellow_mana_full_ws_")
		self:UpdateManaBar("black", snapshot.black / snapshot.actual_max_black, "icon_black", "img_black_mana_full_ws_")
	end

	if snapshot.actual_max_red > snapshot.base_max_red then
		self:set_font("str_red_value", "font_uitext_green")
	elseif snapshot.actual_max_red < snapshot.base_max_red then
		self:set_font("str_red_value", "font_uitext_red")
	else
		self:set_font("str_red_value", "font_uitext")
	end

	if snapshot.actual_max_green > snapshot.base_max_green then
		self:set_font("str_green_value", "font_uitext_green")
	elseif snapshot.actual_max_green < snapshot.base_max_green then
		self:set_font("str_green_value", "font_uitext_red")
	else
		self:set_font("str_green_value", "font_uitext")
	end

	if snapshot.actual_max_blue > snapshot.base_max_blue then
		self:set_font("str_blue_value", "font_uitext_green")
	elseif snapshot.actual_max_blue < snapshot.base_max_blue then
		self:set_font("str_blue_value", "font_uitext_red")
	else
		self:set_font("str_blue_value", "font_uitext")
	end

	if snapshot.actual_max_yellow > snapshot.base_max_yellow then
		self:set_font("str_yellow_value", "font_uitext_green")
	elseif snapshot.actual_max_yellow < snapshot.base_max_yellow then
		self:set_font("str_yellow_value", "font_uitext_red")
	else
		self:set_font("str_yellow_value", "font_uitext")
	end

	if snapshot.actual_max_black > snapshot.base_max_black then
		self:set_font("str_black_value", "font_uitext_green")
	elseif snapshot.actual_max_black < snapshot.base_max_black then
		self:set_font("str_black_value", "font_uitext_red")
	else
		self:set_font("str_black_value", "font_uitext")
	end

	if snapshot.actual_attack_rating > snapshot.base_attack_rating then
		self:set_font("str_attack_value", "font_uitext_green")
	elseif snapshot.actual_attack_rating < snapshot.base_attack_rating then
		self:set_font("str_attack_value", "font_uitext_red")
	elseif snapshot.actual_attack_rating == snapshot.base_attack_rating then
		self:set_font("str_attack_value", "font_uitext")
	end
	
	if snapshot.actual_defence_rating > snapshot.base_defence_rating then
		self:set_font("str_def_value", "font_uitext_green")
	elseif snapshot.actual_defence_rating < snapshot.base_defence_rating then
		self:set_font("str_def_value", "font_uitext_red")
	elseif snapshot.actual_defence_rating == snapshot.base_defence_rating then
		self:set_font("str_def_value", "font_uitext")
	end
	
	self:UpdateString("attack_rating", snapshot.actual_attack_rating, "str_attack_value", "%d")
	self:UpdateString("defence_rating", snapshot.actual_defence_rating, "str_def_value", "%d")
	
	self.snapshot = snapshot

	-- mana has updated, maybe some spell is now available?
	self:DisableAllSpellAndItemButtons()
	self:EnableAvailableSpellButtons()
	self:EnableAvailableItemButtons()
	self:ReformatSpellWidgets(snapshot.actual_spell_cost_modifier / 100)
end

function PlayerInfoMenu:IsInputAllowed()
	-- Input can be made on this player info menu, iff:
	--    * there is a turn active (activeClient)
	--    * it is the players turn (activeClient == self.client)
	--    * this is not an NPC
	--    * this is a local client (meaning, it is controlled by THIS PC)
	
	return self.activeClient and self.activeClient == self.client and not self.activeClient:GetAttribute("npc") and self.activeClient:GetAttribute("local")
end

function PlayerInfoMenu:StartTurn(event)
	self.activeClient = event:GetAttribute("client")
	
	if event:GetAttribute("client") == self.client then
		self:activate_widget("icon_turn")
		self:activate_widget("icon_turn_top")
		self:activate_widget("icon_turn_flame")
		
		self:StartTurnIndicator()
	end
	
	-- turn off all spell and item buttons, then turn on the ones that could be
	-- used (ie, meet all mana requirements)
	self:DisableAllSpellAndItemButtons()
	self:EnableAvailableSpellButtons()
	self:EnableAvailableItemButtons()
	
	-- set the turns left
	local view = SCREENS.BattleGameMenu:GetWorld()
	if view.turnLimit > 0 then
		self.turnsRemaining = 1 + view.turnLimit - event:GetAttribute("turn")
	end
	
	-- decrement "turnsLeft" attribute on all status effects
	for _, effect in pairs(self.statusEffects) do
		if effect.params.turnsLeft then
			effect.params.turnsLeft = math.max(0, effect.params.turnsLeft - 1)
		end
	end
end

function PlayerInfoMenu:StartTurnIndicator()
	--Turn Indicator Particle
	if not self.turn_indicator then
		self.turn_indicator = GameObjectManager:ConstructLocal("SPFX")
		self.turn_indicator.menuOffsets = {x=self:GetX(),y=self:GetY()}
		self.turn_indicator.height = self:get_widget_h("icon_portrait")
		self.turn_indicator.width = self:get_widget_w("icon_portrait")
		self.turn_indicator.start_x = self:get_widget_x("icon_portrait")
		self.turn_indicator.start_y = self:get_widget_y("icon_portrait")
		local function newCycle(fragment)
			local menuOffsets = fragment.menuOffsets
			local start_x = menuOffsets.x + fragment.start_x
			local start_y = menuOffsets.y + fragment.start_y
			local startpt = SCREENS.EffectsMenu:ConvertScreenCoordsToWorldCoords(start_x, start_y)
			start_x = startpt.x
			start_y = startpt.y
			local points = {{0,0},{1,0},{1,1},{0,1},{0,0}}
			local path = MovementManager:Construct("RoundedNonuniformSpline")
			path:SetAttribute("Duration", 2000)
			local dir = 1
			if start_x > GetScreenWidth()/2 then
				dir = -1
				start_x = start_x + fragment.width
			end
			for _,point in ipairs(points) do--Creates path
				local x = start_x + point[1]*fragment.width*dir
				local y = start_y - point[2]*fragment.height
				path:PushAttribute("PointX",x)
				path:PushAttribute("PointY",y)
				path:PushAttribute("VelocityX",0)
				path:PushAttribute("VelocityY",0)
			end
			
			fragment:SetPos(path:GetAttributeAt("PointX",1),path:GetAttributeAt("PointY",1))
			fragment:SetMovementController(path)
			SCREENS.EffectsMenu:GetWorld():AddChild(fragment)
		end
		self.turn_indicator:SetMovementFinishedCallback(newCycle)
		newCycle(self.turn_indicator)
		--Particle needs to be added after Position of fragment is set.
		local particle = "Red"
		if self.turn_indicator.menuOffsets.x < GetScreenWidth()/2 then--Player 1 == Blue/ Player2 == Red
			particle = "Blue"
		end
		particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription(string.format("LightningPath%s",particle)))
		particle:SetLifeCycle(3000)
		particle:SetReleaseInterval(1)
		particle:SetNumParticlesToRelease(3)
		local r,g,b,a = particle:GetColor()
		particle:SetColor(r,g,b,0.4)
		
		self.turn_indicator.particles = ParticleLibrary.CreateParticleSystem(particle)
		self.turn_indicator:AddParticlesPos(self.turn_indicator.particles, 0,0, 0)
	end
end

function PlayerInfoMenu:StopTurnIndicator()
	if self.turn_indicator then
		GameObjectManager:Destroy(self.turn_indicator)
		self.turn_indicator = nil
	end
end

function PlayerInfoMenu:EndTurn()
	self:EndMove()
	self:hide_widget("icon_turn")
	self:hide_widget("icon_turn_top")
	self:hide_widget("icon_turn_flame")
	
	self:StopTurnIndicator()
	
	self.activeClient = nil
end

function PlayerInfoMenu:DisableAllSpellAndItemButtons()
	self:deactivate_widget("butt_weapon1")
	self:deactivate_widget("butt_weapon2")
	for i=1,MAX_SPELLS do
		self:deactivate_widget(string.format("butt_spell%d", i))
		if not self.spells[i] then
			self:hide_widget(string.format("butt_spell%d", i))
		end
	end
end

function PlayerInfoMenu:EnableAvailableSpellButtons()
	if self.canUseSpells == 0 and self:IsInputAllowed() then
		for i=1,MAX_SPELLS do
			if self.spells[i] and self:CheckAllRequirementsForSpell(i) then
				self:activate_widget(string.format("butt_spell%d", i))
			end
		end
	end
end

function PlayerInfoMenu:EnableAvailableItemButtons()
	if self.canUseItems == 0 and self:IsInputAllowed() then
		if self.mainHand then
			if self.snapshot.power >= (self.mainHand:GetActionPower() * self.itemCostModifier) then
				self:activate_widget(string.format("butt_weapon1", i))
			end
		end
		if self.offHand then
			if self.snapshot.power >= ((self.offHand:IsStealthy()) and (self.offHand:GetActionPower()) or (self.offHand:GetActionPower() * self.itemCostModifier)) then
				self:activate_widget(string.format("butt_weapon2", i))
			end
		end
	end
end

function PlayerInfoMenu:CheckAllRequirementsForSpell(idx)
	assert(idx)
	assert(self.spells[idx])
	
	local manareqs = self:CheckManaRequirementsForSpell(idx)
	local countreqs = self:CheckUseCountRequirementsForSpell(idx)
	local cooldown = self:CheckCooldownForSpell(idx)
	
	return manareqs and countreqs and cooldown
end

function PlayerInfoMenu:CheckManaRequirementsForSpell(idx)
	assert(idx)
	assert(self.spells[idx])
	
	local name = self.spells[idx]
	local spellInfo = SPELLS[name]
	
	for _, pool in pairs(POOLS) do
		if spellInfo.cost[pool] and spellInfo.cost[pool] > self.snapshot[pool] then
			return false
		end
	end
	
	if spellInfo.givesTrait then
		-- this has no cost, assume this is a spell that gives a trait, so
		-- we leave this unclickable for now
		return false
	end
	
	return true
end

function PlayerInfoMenu:CheckUseCountRequirementsForSpell(idx)
	assert(idx)
	assert(self.spells[idx])
	
	if self.client:IsSpellCastLimitExceeded(idx, self.spells[idx]) then
		return false
	end
	
	return true
end

function PlayerInfoMenu:CheckCooldownForSpell(idx)
	assert(idx)
	assert(self.spells[idx])
	
	if self.client:IsSpellInCooldown(idx, self.spells[idx]) then
		return false
	end
	
	return true
end

function PlayerInfoMenu:StartMove()
--	self:activate_widget("str_move")
	self.moveID = self.moveID + 1
	self.moveActive = true
	
	-- start the turn timer
	local view = SCREENS.BattleGameMenu:GetWorld()
	if view.turnTimer > 0 then
		local currentMove = self.moveID
		local time = math.floor(view.turnTimer / 1000)
		
		self:activate_widget("icon_timer")
		self:activate_widget("str_timer_value")
		self:set_text_raw("str_timer_value", tostring(time))
		
		-- this thread wakes up once a second to decrement the timer, note the thread
		-- will abort when the moveID is changed, which it is, at the start of each 
		-- move.
		scheduler:AddChildThread(self, function ()
			scheduler:Sleep(1000)
			while currentMove == self.moveID and time > 0 do
				time = time - 1
				self:set_text_raw("str_timer_value", tostring(time))
				scheduler:Sleep(1000)
			end
		end, "Update Turn Timer")
	elseif view.turnLimit > 0 and self.turnsRemaining then	-- show the turns left
		self:activate_widget("icon_timer")
		self:activate_widget("str_timer_value")
		self:set_text_raw("str_timer_value", tostring(self.turnsRemaining))
	end
end

function PlayerInfoMenu:RestartMove()
	-- renable timer widgets, if this is a timer based game
	if SCREENS.BattleGameMenu:GetWorld().turnTimer > 0 then
		self:activate_widget("icon_timer")
		self:activate_widget("str_timer_value")
	end
	
	self.moveActive = true
end

function PlayerInfoMenu:EndMove()
--	self:hide_widget("str_move")
	local view = SCREENS.BattleGameMenu:GetWorld()
	if view.turnLimit == 0 then
		self:hide_widget("icon_timer")
		self:hide_widget("str_timer_value")
	end

	-- just in case it has been left open
	self:CloseSpellTargetMenu()
	
	-- flag that no move is active, this is to stop players from attempting
	-- to cast a spell if they just cast one
	self.moveActive = nil
end

function PlayerInfoMenu:GetSpellIndex(spell)
	for i=1, MAX_SPELLS do
		if spell == self.spells[i] then
			return i
		end
	end
end

function PlayerInfoMenu:SendSpellRequest(idx)
	if self.targettingLockdown then
		self:DisableTargettingLockdown()
		
		return
	end
	
	local spell = self.spells[idx]
	
	if spell then
		local spellInfo = SPELLS[spell]
		
		if spellInfo.requiresTarget and not self.activeClient:IsSpellInCooldown(idx, self.spells[idx]) then
			self.targettingLockdown = true
			self:DisableAllSpellAndItemButtons()
			
			if spellInfo.targetText then
				local x, y = self:CalculateSpellTargetMenuPos()
				
				SCREENS.SpellTargetMenu:SetupAndOpen(spellInfo.targetText, self.activeClient, x, y)
			end
			
			-- need to move the view into targetting mode
			SCREENS.BattleGameMenu:GetWorld():SetTargettingOnly(spellInfo.targetOnly)
			SCREENS.BattleGameMenu:GetWorld():StartTargettingMode(function (x, y)
				self.targettingLockdown = false
				
				self:CloseSpellTargetMenu()
				
				self.client:SendSpellRequest(idx, x, y)
				self:EndMove()
			end, spellInfo.icon)
		else
			self.client:SendSpellRequest(idx)
			self:EndMove()
		end
	end
end

function PlayerInfoMenu:CalculateSpellTargetMenuPos()
	local x, y
	local offset = GetScreenOffset()
	if offset > 0 then	-- widescreen
		x = -102	-- see #3859
	else
		x = self:GetX() + self:get_widget_x("butt_spell1")
	end
	y = self:GetY() + self:get_widget_y("butt_spell1")
	
	return x, y
end

function PlayerInfoMenu:CloseSpellTargetMenu()
	SCREENS.SpellTargetMenu:Close()
end

function PlayerInfoMenu:DisableTargettingLockdown()
	self.targettingLockdown = false
	
	SCREENS.BattleGameMenu:GetWorld():EndTargettingMode(true)
	
	self:EnableAvailableSpellButtons()
	self:EnableAvailableItemButtons()
	self:CloseSpellTargetMenu()
end

function PlayerInfoMenu:OnMouseRightButton()
	if self.targettingLockdown then
		self:DisableTargettingLockdown()
		
		return Menu.MESSAGE_HANDLED
	end
	
	return Menu.MESSAGE_HANDLED
end

function PlayerInfoMenu:SendItemRequest(idx)
	self.client:SendItemActivationRequest(idx)
end

function PlayerInfoMenu:ShowItemPopup(slot, x, y, id)
	assert(slot)
	assert(x)
	assert(y)
	
	local item = self:GetItemFromIndex(slot)
	
	if not item then
		return
	end
	
	local format, data = item:GetToolTipFormatAndDataWithParams(self.heroParams, true)
	
	open_custompopup_menu(format, data, x, y, 0, 256, "PlayerInfoMenu", id)
end

function PlayerInfoMenu:ShowHint(msg, x, y, id)
	open_custompopup_menu("//HELPEX/0/6/font_system/center/top//", string.format("//%s//", msg), x, y, 4, 256, "PlayerInfoMenu", id)
end

function PlayerInfoMenu:ShowHeroPopup(x, y, id)
	assert(x)
	assert(y)
	
	-- [SCF] Trying a bit more info
	if self.snapshot then
		local format, data = GetPlayerToolTipFormatAndData(self.snapshot)
		
		open_custompopup_menu(
			format,
			data,
			x, y,
			4,
			200,
			"PlayerInfoMenu",
			id
		)
	end
end

function PlayerInfoMenu:ShowSpellPopup(slot, x, y, id)
	assert(slot)
	assert(x)
	assert(y)
	
	local spell = self.spells[slot]
	if spell then
		local format, data = _G.GetSpellToolTipFormatAndData(spell)
		open_custompopup_menu(format, data, x, y, 0, 256, "PlayerInfoMenu", id)
	end
end

function PlayerInfoMenu:HideItemPopup(id)
	close_custompopup_menu(id)
end

function PlayerInfoMenu:OnMouseEnter(id, x, y)
	LOGF("PlayerInfoMenu:OnMouseEnter(%d, x=%d, y=%d)", id, x, y)

	if not SCREENS.BattleGameMenu:IsPopupsEnabled() then
		close_custompopup_menu(id)
		return Menu.MESSAGE_HANDLED
	end
	
	for i=1,4 do
		local pad = string.format("pad_item%d", i)
		if (id == self:get_widget_id(pad)) then
			self:ShowItemPopup(i, self:get_widget_abs_x(pad) + (self:get_widget_w(pad)/2), self:get_widget_abs_y(pad), id)
			self:set_alpha(string.format("icon_item%d", i), 1.0)
			return Menu.MESSAGE_HANDLED
		end
	end
	
	if id == self:get_widget_id("pad_portrait") then
		self:ShowHeroPopup(self:get_widget_abs_x("pad_portrait") + (self:get_widget_w("pad_portrait")/2), self:get_widget_abs_y("pad_portrait") + self:get_widget_h("pad_portrait"), id)
		return Menu.MESSAGE_HANDLED
	elseif id == self:get_widget_id("pad_attackrating") then
		self:ShowHint("[ATTACKRATING_TIP0]", self:get_widget_abs_x("pad_attackrating") + (self:get_widget_w("pad_attackrating")/2), self:get_widget_abs_y("pad_attackrating") + self:get_widget_h("pad_attackrating"), id)
		return Menu.MESSAGE_HANDLED
	elseif id == self:get_widget_id("pad_defrating") then
		self:ShowHint("[DEFRATING_TIP0]", self:get_widget_abs_x("pad_defrating") + (self:get_widget_w("pad_defrating")/2), self:get_widget_abs_y("pad_defrating") + self:get_widget_h("pad_defrating"), id)
		return Menu.MESSAGE_HANDLED
	end
		
	for i=1,7 do
		local pad = string.format("pad_statuseffect%d", i)
		if id == self:get_widget_id(pad) then
			self:ShowStatusEffectPopup(i, self:get_widget_abs_x(pad) + (self:get_widget_w(pad)/2), self:get_widget_abs_y(pad) + self:get_widget_h(pad), id)
			return Menu.MESSAGE_HANDLED
		end
	end
	
	return Menu.OnMouseEnter(self, id, x, y)
end

function PlayerInfoMenu:OnMouseLeave(id, x, y)
	LOGF("PlayerInfoMenu:OnMouseLeave(%d, x=%d, y=%d)", id, x, y)

	for i=1,4 do
		if id == self:get_widget_id(string.format("pad_item%d", i)) then
			self:HideItemPopup(id)
			self:set_alpha(string.format("icon_item%d", i), ITEM_DESELECT_ALPHA)
		end
	end
	
	close_custompopup_menu(id)
	
	return Menu.MESSAGE_HANDLED
end

function PlayerInfoMenu:OnButton(buttonId, clickX, clickY)
	--[[ Uncomment this if NAMCO want the selection ring to dissapear as soon as any button is pressed
	local view = SCREENS.BattleGameMenu:GetWorld()
	if view then
		view:Unselect()
	end]]
	
	close_custompopup_menu(buttonId)
	
	if self.moveActive then -- only process button requests if the player is able to make a move
		if buttonId == self:get_widget_id("butt_spell1") then
			self:SendSpellRequest(1)
		elseif buttonId == self:get_widget_id("butt_spell2") then
			self:SendSpellRequest(2)
		elseif buttonId == self:get_widget_id("butt_spell3") then
			self:SendSpellRequest(3)
		elseif buttonId == self:get_widget_id("butt_spell4") then
			self:SendSpellRequest(4)
		elseif buttonId == self:get_widget_id("butt_spell5") then
			self:SendSpellRequest(5)
		elseif buttonId == self:get_widget_id("butt_weapon1") then
			self:SendItemRequest(Items.MAIN_HAND)
		elseif buttonId == self:get_widget_id("butt_weapon2") then
			self:SendItemRequest(Items.OFF_HAND)
		else
			return Menu.OnButton(self, buttonId, clickX, clickY)
		end
	end
	
	return Menu.OnButton(self, buttonId, clickX, clickY)
--	return Menu.MESSAGE_HANDLED
end

function PlayerInfoMenu:OnButtonAvailable(button_id, x_value, y_value, on)
	LOGF("PlayerInfoMenu:OnButtonAvailable(button_id=%d, x_value=%d, y_value=%d, on=%s)", button_id, x_value, y_value, tostring(on))
	
	if button_id == self:get_widget_id("butt_weapon1") then
		if on then
			self:set_alpha("icon_weapon1", 1.0)
		else
			self:set_alpha("icon_weapon1", ITEM_DESELECT_ALPHA)
		end
	elseif button_id == self:get_widget_id("butt_weapon2") then
		if on then
			self:set_alpha("icon_weapon2", 1.0)
		else
			self:set_alpha("icon_weapon2", ITEM_DESELECT_ALPHA)
		end
	end
	
	if on and SCREENS.BattleGameMenu:IsPopupsEnabled() then
		for i=1,5 do
			local button = string.format("butt_spell%d", i)
			if (button_id == self:get_widget_id(button)) then
				self:ShowSpellPopup(i, self:get_widget_abs_x(button) + (self:get_widget_w(button)/2), self:get_widget_abs_y(button), button_id)
			end
		end

		if button_id == self:get_widget_id("butt_weapon1") then
			self:ShowItemPopup(Items.MAIN_HAND, 
				self:get_widget_abs_x("butt_weapon1") + (self:get_widget_w("butt_weapon1")/2), 
				self:get_widget_abs_y("butt_weapon1"), 
				button_id)
		elseif button_id == self:get_widget_id("butt_weapon2") then
			self:ShowItemPopup(Items.OFF_HAND, 
				self:get_widget_abs_x("butt_weapon2") + (self:get_widget_w("butt_weapon2")/2), 
				self:get_widget_abs_y("butt_weapon2"),
				button_id)
		end
	else
		close_custompopup_menu(button_id)
	end
	
	return Menu.MESSAGE_HANDLED
end

function PlayerInfoMenu:OnItemActivationNotify(event)
end

function PlayerInfoMenu:OnSpellRejected(event)
	assert(self.client)
	if event:GetAttribute("pending") then
		self:StartMove()
	end
end

function PlayerInfoMenu:UpdateSpellAndItemButtons()
	self:DisableAllSpellAndItemButtons()
	self:EnableAvailableSpellButtons()
	self:EnableAvailableItemButtons()
end

-- Function to get center (x,y) coords of widget
function PlayerInfoMenu:GetWidgetPosition(widgetName)
	local x = self:get_widget_x(widgetName) + self:get_widget_w(widgetName) / 2 + self:GetX() - GetScreenOffset()
	local y = 768 - (self:get_widget_y(widgetName) + self:get_widget_h(widgetName) / 2 + self:GetY())
	return x, y
end

function PlayerInfoMenu:AddStunIcons()
	-- remove any previous icons
	self:RemoveStunIcons()
	
	self.stun_icons = {}
	local stun_index = 1
	local view = SCREENS.EffectsMenu:GetWorld()
	
	if not (self.canUseSpells == 0) then
		for i=1,MAX_SPELLS do
			if self.spells[i] then
				local x, y = self:GetWidgetPosition(string.format("butt_spell%d", i))
				
				local stunView = GameObjectViewManager:Construct("GraphicAsset", "img_status_stun")
				stunView:SetSortingValue(-5)
				stunView:SetAlpha(0.0)
				
				view:AddOverlay(string.format("%s_stun_icon%d", tostring(self.client), stun_index), stunView, x, y)
				table.insert(self.stun_icons, stunView)
				stun_index = stun_index + 1
			end
		end
	end
	
	if not (self.canUseItems == 0) then
		if self.mainHand then
			local x, y = self:GetWidgetPosition("butt_weapon1")
			
			local stunView = GameObjectViewManager:Construct("GraphicAsset", "img_status_stun")
			stunView:SetSortingValue(-5)
			stunView:SetAlpha(0.0)
			
			view:AddOverlay(string.format("%s_stun_icon%d", tostring(self.client), stun_index), stunView, x, y)
			table.insert(self.stun_icons, stunView)
			stun_index = stun_index + 1
		end
		if self.offHand then
			local x, y = self:GetWidgetPosition("butt_weapon2")
			
			local stunView = GameObjectViewManager:Construct("GraphicAsset", "img_status_stun")
			stunView:SetSortingValue(-5)
			stunView:SetAlpha(0.0)
			
			view:AddOverlay(string.format("%s_stun_icon%d", tostring(self.client), stun_index), stunView, x, y)
			table.insert(self.stun_icons, stunView)
			stun_index = stun_index + 1
		end
	end
	
	if next(self.stun_icons) then
		self.stun_thread = scheduler:AddChildThread(self, function ()
			while (self.canUseSpells ~= 0) or (self.canUseItems ~= 0) do
				local delay_in  = 1000
				local delay_out = 1000
				for i, stun_icon in ipairs(self.stun_icons) do
					continuous_blend(stun_icon, 0.0, 1.0, GetGameTime(), delay_in)
				end
				scheduler:Sleep(delay_in)
				for i, stun_icon in ipairs(self.stun_icons) do
					continuous_blend(stun_icon, 1.0, 0.0, GetGameTime(), delay_out)
				end
				scheduler:Sleep(delay_out)
			end
		end)
	end
end

function PlayerInfoMenu:RemoveStunIcons()
	local targetObject = SCREENS.EffectsMenu:GetWorld()
	
	if self.stun_thread then
		scheduler:StopThread(self.stun_thread)
		self.stun_thread = nil
	end
	
	if self.stun_icons then
		for i in ipairs(self.stun_icons) do
			local overlay = string.format("%s_stun_icon%d", tostring(self.client), i)
			if targetObject:GetOverlay(overlay).__ok then
				targetObject:RemoveOverlay(overlay)
			end
		end
		self.stun_icons = nil
	end
end

function PlayerInfoMenu:OnAddStatusEffectStunLesser(effect)
	self.canUseSpells = self.canUseSpells + 1
	self.canUseItems = self.canUseItems + 1
	self:UpdateSpellAndItemButtons()
	
	self:AddStunIcons()
end

function PlayerInfoMenu:OnRemoveStatusEffectStunLesser(effect)
	self.canUseSpells = self.canUseSpells - 1
	self.canUseItems = self.canUseItems - 1
	self:UpdateSpellAndItemButtons()
	
	self:RemoveStunIcons()
end

function PlayerInfoMenu:OnAddStatusEffectStunLesserUntilSkullMatch(effect)
	self.canUseSpells = self.canUseSpells + 1
	self.canUseItems = self.canUseItems + 1
	self:UpdateSpellAndItemButtons()
	
	self:AddStunIcons()
end

function PlayerInfoMenu:OnRemoveStatusEffectStunLesserUntilSkullMatch(effect)
	self.canUseSpells = self.canUseSpells - 1
	self.canUseItems = self.canUseItems - 1
	self:UpdateSpellAndItemButtons()
	
	self:RemoveStunIcons()
end

function PlayerInfoMenu:OnAddStatusEffectNoItems(effect)
	self.canUseItems = self.canUseItems + 1
	self:UpdateSpellAndItemButtons()
	
	self:AddStunIcons()
end

function PlayerInfoMenu:OnRemoveStatusEffectNoItems(effect)
	self.canUseItems = self.canUseItems - 1
	self:UpdateSpellAndItemButtons()
	
	self:RemoveStunIcons()
end

function PlayerInfoMenu:OnAddStatusEffectNoSpells(effect)
	self.canUseSpells = self.canUseSpells + 1
	self:UpdateSpellAndItemButtons()
	
	self:AddStunIcons()
end

function PlayerInfoMenu:OnRemoveStatusEffectNoSpells(effect)
	self.canUseSpells = self.canUseSpells - 1
	self:UpdateSpellAndItemButtons()
	
	self:RemoveStunIcons()
end

function PlayerInfoMenu:OnEventEnterStealth(event)
	assert(not self.accumulatedStealthBonusFXC, "entering stealth mode, but we are already in stealth!")
	
	self.itemCostModifier = event:GetAttribute("itemCostModifier") / 100
	self.stealthed = true

	-- lets alpha out the portrait a little
	MenuHelpers.FadeDownWidget(self, event:GetAttribute("client"), "icon_portrait", 1.0, 0.4, 0.3)
	
	local client = event:GetAttribute("client")
	for i=1,MAX_SPELLS do
		if self.spells[i] then
			local butt_widget = string.format("butt_spell%d", i)
			local str_widget = string.format("str_spell%d", i)
			self:set_text(str_widget, "[STEALTH]")
			
			scheduler:AddChildThread(self, function () 
				while self.stealthed do
					MenuHelpers.CrossFadeOverTime(self, client, butt_widget, 0.0, str_widget, 1.0, 2500)
					scheduler:Sleep(2500)
					MenuHelpers.CrossFadeOverTime(self, client, str_widget, 0.0, butt_widget, 1.0, 2500)
					scheduler:Sleep(2500)
				end
			end)
		end
	end
	
	local posx = self:GetX() + self:get_widget_x("icon_weapon1") + ((2/3) * self:get_widget_w("icon_weapon1"))
	local posy = 768 - (self:GetY() + self:get_widget_y("icon_weapon1") + ((2/3) * self:get_widget_h("icon_weapon1")))
	
	self.accumulatedStealthBonusFXC = FX.CreateContainer(10*60*60*1000, 0) -- 10 hours should be long enough *famous last words*
	self.accumulatedStealthBonusText = FX.AddText(self.accumulatedStealthBonusFXC, "font_button", "+0", 0, 0, 1, 1, 0, 1.0, true, 1.0)
	self.accumulatedStealthBonusAmount = 0
	self.accumulatedStealthBonusStartTime = GetGameTime()
	self.accumulatedStealthBonusOwner = {}
	
	FX.Start(SCREENS.EffectsMenu:GetWorld(), self.accumulatedStealthBonusFXC, posx-_G.GetScreenOffset(), posy)
end

function PlayerInfoMenu:OnEventExitStealth(event)
	MenuHelpers.FadeUpWidget(self, event:GetAttribute("client"), "icon_portrait", 0.4, 1.0, 0.3)
	
	local startFadeTime = GetGameTime() - self.accumulatedStealthBonusStartTime
	local endFadeTime = startFadeTime + 1000

	FX.AddKey(self.accumulatedStealthBonusFXC, self.accumulatedStealthBonusText, startFadeTime, FX.KEY_ALPHA, 1)
	FX.AddKey(self.accumulatedStealthBonusFXC, self.accumulatedStealthBonusText, endFadeTime, FX.KEY_ALPHA, 0)
	
	local accumulatedStealthBonusFXC = self.accumulatedStealthBonusFXC
	local accumulatedStealthBonusOwner = self.accumulatedStealthBonusOwner
	
	-- These are reset immediately, because another accumulate damage could
	-- come through and add to the accumulatedDamageFXC, which would then
	-- fade out, so when the Apply comes through for the earlier accumulate
	-- there might not be an FXC container as the logic expects.
	
	self.accumulatedStealthBonusFXC = nil
	self.accumulatedStealthBonusText = nil
	self.accumulatedStealthBonusAmount = nil
	self.accumulatedStealthBonusStartTime = nil
	self.accumulatedStealthBonusOwner = nil
	
	scheduler:StopAllThreadsForObject(accumulatedStealthBonusOwner)
	scheduler:AddChildThread(self, function () 
		scheduler:Sleep(1000)
		
		FX.Stop(accumulatedStealthBonusFXC, false)
	end, "OnEventApplyAccumulateDamageNotify")

	self.itemCostModifier = 1
	self.stealthed = nil
end

function PlayerInfoMenu:OnEventChangeStealthBonus(event)
	self.accumulatedStealthBonusAmount = event:GetAttribute("bonus")
	
	FX.ChangeText(self.accumulatedStealthBonusFXC, self.accumulatedStealthBonusText, string.format("+%d", self.accumulatedStealthBonusAmount))

	-- ping this number with another FX container
	local posx = self:GetX() + self:get_widget_x("icon_weapon1") + self:get_widget_w("icon_weapon1") / 2
	local posy = 768 - (self:GetY() + self:get_widget_y("icon_weapon1") + self:get_widget_h("icon_weapon1") / 2)
	
	local numfx = FX.CreateContainer(1000, 0)
	local text = FX.AddText(numfx, "font_tiny_event", string.format("+%d", self.accumulatedStealthBonusAmount), 0, 0, 1, 1, 0, 1.0, true, 0.5)
	
	FX.AddKey(numfx, text, 0, FX.KEY_ALPHA, 1)
	FX.AddKey(numfx, text, 1000, FX.KEY_ALPHA, 0, FX.PUNCH_IN)
	FX.AddKey(numfx, text, 0, FX.KEY_SCALE, 1.2)
	FX.AddKey(numfx, text, 100, FX.KEY_SCALE, 1.0, FX.PUNCH_IN)
	FX.Start(SCREENS.EffectsMenu:GetWorld(), numfx, posx, posy)
end

-- data from Assets.xml
local BitmapImageRects = {
	img_spellbutton1_ready_player1 = {x = 283,	y = 0,		w = 281,	h = 75},
	img_spellbutton2_ready_player1 = {x = 283,	y = 75,		w = 281,	h = 74},
	img_spellbutton3_ready_player1 = {x = 283,	y = 149,	w = 281,	h = 75},
	img_spellbutton4_ready_player1 = {x = 283,	y = 224,	w = 281,	h = 74},
	img_spellbutton5_ready_player1 = {x = 283,	y = 298,	w = 281,	h = 75},
	img_spellbutton1_ready_player2 = {x = 0,	y = 0,		w = 281,	h = 75},
	img_spellbutton2_ready_player2 = {x = 0,	y = 75,		w = 281,	h = 74},
	img_spellbutton3_ready_player2 = {x = 0,	y = 149,	w = 281,	h = 75},
	img_spellbutton4_ready_player2 = {x = 0,	y = 224,	w = 281,	h = 74},
	img_spellbutton5_ready_player2 = {x = 0,	y = 298,	w = 281,	h = 75},
	img_spellbutton1_ready_player1_ws = {x = 0,		y = 0,		w = 427,	h = 75},
	img_spellbutton2_ready_player1_ws = {x = 0,		y = 75,		w = 427,	h = 74},
	img_spellbutton3_ready_player1_ws = {x = 0,		y = 149,	w = 427,	h = 75},
	img_spellbutton4_ready_player1_ws = {x = 0,		y = 224,	w = 427,	h = 74},
	img_spellbutton5_ready_player1_ws = {x = 0,		y = 298,	w = 427,	h = 75},
	img_spellbutton1_ready_player2_ws = {x = 429,	y = 0,		w = 427,	h = 75},
	img_spellbutton2_ready_player2_ws = {x = 429,	y = 75,		w = 427,	h = 74},
	img_spellbutton3_ready_player2_ws = {x = 429,	y = 149,	w = 427,	h = 75},
	img_spellbutton4_ready_player2_ws = {x = 429,	y = 224,	w = 427,	h = 74},
	img_spellbutton5_ready_player2_ws = {x = 429,	y = 298,	w = 427,	h = 75},
}

function PlayerInfoMenu:GetBitmapImageRects(name)
	assert(name, "no name provided")
	assert(BitmapImageRects[name], "BitmapImage not found for BitmapImageRects")
	
	return BitmapImageRects[name].x, BitmapImageRects[name].y, BitmapImageRects[name].w, BitmapImageRects[name].h
end

function PlayerInfoMenu:ResetBitmapImageRects()
	local offset = GetScreenOffset()
	for i=1, MAX_SPELLS do
		for j=1, 2 do
			local bitmapimage
			if offset == 0 then	-- not widescreen
				bitmapimage	= string.format("img_spellbutton%d_ready_player%d", i, j)
			else
				bitmapimage	= string.format("img_spellbutton%d_ready_player%d_ws", i, j)
			end
			local x, y, w, h = self:GetBitmapImageRects(bitmapimage)
			set_bitmap_image_source_rect(bitmapimage, x, y, w, h)
		end
	end
end

function PlayerInfoMenu:OnEventSpellCooldownNotify(event)
	local client = event:GetAttribute("client")
	local player = client:GetAttribute("team")
	
	local spell = event:GetAttribute("spell")
	local value = event:GetAttribute("cooldown")
	
	local index = self:GetSpellIndex(spell)
	local spellInfo = SPELLS[spell]
	local cooldown = spellInfo.cooldown
	
	if cooldown then	-- only modify widget width and bitmapimage source rect if neccessary
	local widget = string.format("icon_spell%d_ready", index)
	local multiplier = (cooldown - value) / cooldown
		self:set_widget_w(widget, math.ceil(Constants.SPELL_WIDTH * multiplier))
		
		local bitmapimage
		local offset = GetScreenOffset()
		if offset == 0 then	-- not widescreen
			bitmapimage	= string.format("img_spellbutton%d_ready_player%d", index, player)
		else
			bitmapimage	= string.format("img_spellbutton%d_ready_player%d_ws", index, player)
		end
		local x, y, w, h = self:GetBitmapImageRects(bitmapimage)
		set_bitmap_image_source_rect(bitmapimage, x, y, math.ceil(w * multiplier), h)
	end
end

function PlayerInfoMenu:QueryAvailableCommands()
	local commands = { }
	if self.mainHand then
		commands.weapon1 = true
	end
	if self.offHand then
		commands.weapon2 = true
	end
	for i=1,5 do
		if self.spells[i] then
			commands[string.format("spell%d", i)] = true
		end
	end
	
	for i=1,4 do
		if self.items[i] then
			commands[string.format("item%d", i)] = true
		end
	end
	
	return commands
end

-- Any platform specific stuff in here
dofile("Assets/Scripts/Screens/PlayerInfoMenuPlatform.lua")

return ExportClass("PlayerInfoMenu")
