use_safeglobals()

local MenuHelpers = require("Utils/MenuHelpers")

class "SinglePlayerMenu" (Menu);

function SinglePlayerMenu:__init()
	super()
	
	self:Initialize("Assets\\Screens\\SinglePlayerMenu.xml")
end

-- The SinglePlayerMenu assumes responsibility of deleting the objects passed into
-- SetHeroData.  It deletes the objects in the OnClose() method provided that the
-- objects haven't been transferred to something elses control, ie, the campaign
-- system.

function SinglePlayerMenu:SetHeroData(hero, campaign)
	self.hero = hero
	self.campaign = campaign
end

function SinglePlayerMenu:OnOpen()
	ShowBackdropMenu()
	
	return Menu.OnOpen(self)
end

function SinglePlayerMenu:OnRefresh()
	local activeHero = HeroManager:GetActiveHero()
	
	self:set_image("icon_portrait", string.format("%s_M", activeHero:GetAttribute("portrait")))
	self:set_text_raw("str_name_value", activeHero:GetAttribute("name"))
	self:set_text_raw("str_level_value", tostring(activeHero:GetAttribute("level")))
	self:set_text_raw("str_health_value", tostring(activeHero:GetHealth()))
	
	local gold_val = activeHero:GetAttribute("gold")
	local text_val, real_gold_val = MenuHelpers.ShortenValue(gold_val)
	
	if real_gold_val and text_val then
		self:set_text_raw("str_gold_value", substitute(translate_text(text_val), tostring(real_gold_val)))
	else
		self:set_text_raw("str_gold_value", tostring(gold_val))
	end
	
	-- setup the difficulty droplist
	self:reset_list("drop_difficulty")
	self:set_list_option("drop_difficulty", "[DIFFICULT_EASY]")
	self:set_list_option("drop_difficulty", "[DIFFICULT_NORMAL]")
	self:set_list_option("drop_difficulty", "[DIFFICULT_HARD]")
	-- set to the current difficulty
	local difficulty = activeHero:GetGameDifficulty()
	self:set_list_value("drop_difficulty", difficulty + 1)
end

function SinglePlayerMenu:OnClose()
	return Menu.OnClose(self)
end

function SinglePlayerMenu:OnTimer(time)
	return Menu.OnTimer(self, time)
end

function SinglePlayerMenu:OnKey(key)
	if key == Keys.SK_ESCAPE then
		HeroManager:SetHeroSaveEnabled(true)
		CallScreenSequencer("SinglePlayerMenu", "MainMenu")
	elseif key == Keys.SK_F12 then -- DEBUG
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local hero = HeroManager:GetActiveHero()
			local level = hero:GetAttribute("level")
			SCREENS.LevelUpMenu:Open()
			SCREENS.LevelUpMenu:SetLevelGainData(hero, math.min(level+1, 50))
		end
	elseif key == string.byte("V") or key == string.byte("v") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local hero = HeroManager:GetActiveHero()
			
			for room, nodeList in pairs(require("LevelData.LevelsRoomList")) do
				hero:SetRoomVisited(room)
			end
		end
	elseif key == Keys.SK_F11 then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			SCREENS.ShopMenu:Open()
			SCREENS.ShopMenu:FillHeroItems(HeroManager:GetActiveHero())
			SCREENS.ShopMenu:FillShopItems( { name = "The Cosmos" } )
		end
	elseif key == Keys.SK_F9 then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			SCREENS.QuestRewardsMenu:Open()
			SCREENS.QuestRewardsMenu:SetData(HeroManager:GetActiveHero(),
			{	{	choiceType = "Gold", count = 500, },
				{   choiceType = "Standard",  baseType = "Axe" },
			})
		end
	elseif key == Keys.SK_F8 then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			CallScreenSequencer("SinglePlayerMenu", "TournamentMenu")
		end
	elseif key == Keys.SK_F5 then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			_G.ShowTutorialSingleElement(3, "SinglePlayerMenu", "butt_back", 1)
		end
	end
	
	return Menu.OnKey(self, key)
end

function SinglePlayerMenu:OnButton(buttonId, clickX, clickY)
	if buttonId == self:get_widget_id("butt_back") then
		self:Quit()
	elseif buttonId == self:get_widget_id("butt_quest") then
		local function StartGame()
			HideBackdropMenu()
			
			local activeHero = HeroManager:GetActiveHero()
			activeHero:SetRequireSave(false)
			activeHero:PrintDebug(true)
			
			local function StartWorldMap()
				MusicManager:Resume()
				MusicManager:SetState( "LOADING" )
				SCREENS.CustomLoadingMenu:SetupAndOpen(activeHero:GetAttribute("currentLocation"), function ()
					local screen = StartWorldMapAt(activeHero:GetAttribute("currentLocation"), activeHero:GetAttribute("currentNode"), activeHero:GetAttribute("currentDir"))
					assert(type(screen) == 'function')
					screen()
					SCREENS.CustomLoadingMenu:Close()
				end)
			end	
			
			if activeHero:GetAttribute("introductionCutsceneShown") then
				StartWorldMap()
			else
				activeHero:SetAttribute("introductionCutsceneShown", true)
				self:Close()
				local hero_voice = 0
				if activeHero:GetAttribute("sex") == "M" then
					hero_voice = 1
				end
				SoundSystem.Load("amb_bank")
				SoundSystem.SetParameter( "pq2audio/amb/amb_n0aa", "hero_gender", hero_voice )
				MusicManager:Stop()
				open_nis_menu("N0AA", StartWorldMap)
			end
		end
		
		CallScreenSequencer("SinglePlayerMenu", StartGame)

	elseif buttonId == self:get_widget_id("butt_quickbattle") then
		self:StartQuickBattle()
	elseif buttonId == self:get_widget_id("butt_inventory") then
		SCREENS.InventoryMenu:Open()
		SCREENS.InventoryMenu:SetCompletedCallback(function () SCREENS.SaveMenu:Display() end)
		SCREENS.InventoryMenu:SetHero(HeroManager:GetActiveHero())
	elseif buttonId == self:get_widget_id("butt_spellbook") then
		SCREENS.SpellbookMenu:SetHero(HeroManager:GetActiveHero())
		SCREENS.SpellbookMenu:SetCompletedCallback(function () SCREENS.SaveMenu:Display() end)
		SCREENS.SpellbookMenu:Open()
	--elseif buttonId == self:get_widget_id("butt_tournament") then
		--CallScreenSequencer("SinglePlayerMenu", "TournamentMenu")
	elseif buttonId == self:get_widget_id("butt_endurance") then
		CallScreenSequencer("SinglePlayerMenu", "EnduranceMenu")		
	elseif buttonId == self:get_widget_id("butt_leaderboards") then
		if IsValidPlayerOnline(1) then --MUST BE SIGNED INTO XBOX LIVE
			if IsValidPlayerSignedIn(1) then
				SCREENS.LeaderboardHolderMenu:Open()
				SCREENS.LeaderboardHolderMenu:SetMode("sp_overall")
			else
				open_message_menu("[WARNING]","[MUST_BE_SIGNED_IN_PROFILE]",nil)
			end
		else
			open_message_menu("[WARNING]","[MUST_BE_SIGNED_IN_LIVE]",nil)
		end
	elseif buttonId == self:get_widget_id("butt_minigames") then
		CallScreenSequencer("MainMenu", "MinigameSelectionMenu") 
		SCREENS.SinglePlayerMenu:Close()
	else
		return Menu.OnButton(self, buttonId, clickX, clickY)
	end
	
	return Menu.MESSAGE_HANDLED   
end

function SinglePlayerMenu:Quit()
	HeroManager:ResetActiveHero()
	HeroManager:SetHeroSaveEnabled(true)
	if _G.IsDemo() then
		CallScreenSequencer("SinglePlayerMenu", "CreateHeroMenu")
	else
		CallScreenSequencer("SinglePlayerMenu", function ()
			SCREENS.CustomLoadingMenu:SetupAndOpen(nil, function()
				local function Continue()
					if HeroManager:GetNumHeroes() > 0 then
						SCREENS.CustomLoadingMenu:Close()
						SCREENS.SelectHeroMenu:SetupAndOpen(function (selected)
							if selected then
								CallScreenSequencer("SelectHeroMenu", "SinglePlayerMenu")
							else
								CallScreenSequencer("SelectHeroMenu", "MainMenu")
							end
						end)
						SCREENS.SelectHeroMenu:StartAnimation("fadein")
					else
						CallScreenSequencer("CustomLoadingMenu", "CreateHeroMenu")
					end
				end
				local function Shutdown(heading, message) 
					_G.ResetToSplashWithError(heading, message)
				end
				HeroManager:LoadHeroes(Continue, Shutdown)
				MusicManager:SetState( "MENU" )
			end, "[LOADING]", 2000)
		end)
	end
end

function SinglePlayerMenu:OnDroplist(droplist_id, item)
	if droplist_id == self:get_widget_id("drop_difficulty") then
		local hero = HeroManager:GetActiveHero()
		hero:SetGameDifficulty(item)
		
	else
		return Menu.MESSAGE_NOT_HANDLED
	end
	
	return Menu.MESSAGE_HANDLED
end

function SinglePlayerMenu:StartQuickBattle()
	local hero1 = HeroManager:GetActiveHero()
	local hero2 = CreateRandomMonster(hero1, true) -- true indicates it's a quick battle, take quest predicates into account
	
	local def = hero2.def
	if def and def.voice then
		SoundSystem.Load(string.format("vox_%s_bank", def.voice))
	end
	
	local function OnFinished()
		ShowBackdropMenu()
		
		-- Restore presence
		Presence:SetPresenceMode("NAVIGATING_MENUS")
			
		DestroyHero(hero2)
		
		if def and def.voice then
			SoundSystem.Unload(string.format("vox_%s_bank", def.voice))
		end
		
		SCREENS.SinglePlayerMenu:Open()
		
		-- handle level up and saving
		local requireSave = true
		
		if hero1:HasQuestFeedback() then
			local message = hero1:PopQuestFeedback()
			if message.title == "[LEVELUP_HEAD]" then
				message.action()
				
				requireSave = false
				
				SCREENS.LevelUpMenu:SetCompletedCallback(function () SCREENS.SaveMenu:Display() end)
			end
		end
		
		if requireSave then
			SCREENS.SaveMenu:Display()
		end
	end
	
	local function OnFlee()
		ShowBackdropMenu()
		
		-- Restore presence
		Presence:SetPresenceMode("NAVIGATING_MENUS")
			
		if def and def.voice then
			SoundSystem.Unload(string.format("vox_%s_bank", def.voice))
		end
		
		SCREENS.SinglePlayerMenu:Open()
	end
	
	local function OnFight(arguments)
		hero2 = assert(arguments.monsterHero)	-- update the monster hero as it is selectable
		
		Presence:SetPresenceMode("SINGLEPLAYER_BATTLE")
		Presence:SetPresenceContext("MONSTER", string.upper(hero2.def.name))
	end
		
	local params = {}
	
	params.heroes = {}
	params.heroes[1] = {hero1}
	params.heroes[2] = {hero2}
	
	SCREENS.PreBattleMenu:SetupAndOpen({
		transitionFrom = "SinglePlayerMenu", 
		playerHero = hero1, 
		monsterHero = hero2, 
		onFinished = OnFinished, 
		onFlee = OnFlee, 
		onFight = OnFight,
		params = params,
		selectableMonster = true
	})
end

dofile("Assets/Scripts/Screens/SinglePlayerMenuPlatform.lua")

-- return an instance of SinglePlayerMenu
return ExportSingleInstance("SinglePlayerMenu")
