local BatSwarm = {}
local Spell = require("Spells/Spell")

--[[

[Damage Over Time: Causes 2% damage to your enemy per turn for their next 3 turns.]
[Defense Penalty: -50% to your opponent's Defense for 3 turns.]
[Fear: Opponent loses their next turn due to Fear.]

]]

BatSwarm.id		= "BATSWARM"
BatSwarm.cost	= {
	red 		= 6,
}
BatSwarm.icon	= "img_spell_bat_swarm"
BatSwarm.short_name	= "BATSWM"
BatSwarm.sound	= "sp_batswarm"
BatSwarm.sound_ds = "snd_sp_cursedark"
BatSwarm.sound_bank_ds = "BANK_SP_CURSEDARK"
BatSwarm.dietfxParticle	= "magicfireball"
BatSwarm.dietfxTexture	= "Skull"
BatSwarm.cooldownForAI = 3
BatSwarm.effects = {
	{"[BATSWARM_EFFECT0_HEAD]", "[BATSWARM_EFFECT0_BODY]"},
	{"[BATSWARM_EFFECT1_HEAD]", "[BATSWARM_EFFECT1_BODY]"},
}
BatSwarm.title	= Spell:GetTitleTag(BatSwarm)
BatSwarm.subtitles = {
	Spell:GetEffectTag(BatSwarm, 0),
}
BatSwarm.cooldown = 4
BatSwarm.cooldownForAI = 4

function BatSwarm:Apply(params)
	return function ()
		local damage = 5
		
		Spell:Pause(Spell.SPELL_INTRO_PAUSE)
		Spell:ApplyStatusEffectTo(params.target, params, self, "DamageOverTime", 6, damage)
		Spell:ApplyStatusEffectTo(params.target, params, self, "Fear", 6, -50, 0)
		Spell:Pause(Spell.SPELL_MIN_DURATION)
		Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	end
end

function BatSwarm:Notify(grid,notify)
	local heading, text, startTime = Spell:ShowStandardSpellHeader(self.title)
	
	
	local client = notify:GetAttribute("client")
	local start_x = Spell:GetWidgetTargetPosition(client, Spell:GetSpellButtonWidgetName(notify))
	local boardOffset = SCREENS.BattleGameMenu:get_widget_w("icon_board")/3
	local p_name = "Bats"
	
	if start_x > GetScreenWidth()/2 then
		p_name = "Bats2"
	end
	
	local particle1 = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription(p_name))	
	particle1:SetNumParticlesToRelease(50)
	particle1:SetMaxParticles(200)
	particle1:SetGravity(0,4,0)
	for i=1,10 do		
		local widgetsList = {
			{1,Spell:GetSpellButtonWidgetName(notify),0,10};
			{2,"icon_portrait",math.random(-80,80),math.random(-50,50)};
		};
		local path = Spell:WidgetPath(nil,notify,widgetsList,2,math.random(5,15)/10,toggle)
		path:SetAttribute("Duration",1000)
		Spell:AttachParticleMotionFragment(path, {particle1},nil, 100*(i-1))
	end
	
	
	Spell:Pause(Spell.SPELL_INTRO_PAUSE)
	Spell:ShowStandardEffectSubtitle(self.subtitles[1], Spell.SPELL_MIN_DURATION)
	Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	Spell:RemoveStandardSpellHeader(heading, text, startTime)
end

return BatSwarm
