local FearsomeHowl = {}
local Spell = require("Spells/Spell")

FearsomeHowl.id = "FEARSOMEHOWL"
FearsomeHowl.cost = {
	green = 9,
	black = 8,
}
FearsomeHowl.icon = "img_spell_fearsome_howl"
FearsomeHowl.short_name		= "FRHOWL"
FearsomeHowl.sound	= "sp_fearsomehowl"
FearsomeHowl.sound_ds = "snd_sp_howl"
FearsomeHowl.sound_bank_ds = "BANK_SP_HOWL"
FearsomeHowl.dietfxParticle	= "magicfireball"
FearsomeHowl.dietfxTexture	= "Skull"
FearsomeHowl.cooldownForAI = 5
FearsomeHowl.cooldown = 5

FearsomeHowl.effects = {
	{"[FEARSOMEHOWL_EFFECT0_HEAD]", "[FEARSOMEHOWL_EFFECT0_BODY]" },
	{"[FEARSOMEHOWL_EFFECT1_HEAD]", "[FEARSOMEHOWL_EFFECT1_BODY]" }
}

FearsomeHowl.title = Spell:GetTitleTag(FearsomeHowl)
FearsomeHowl.subtitles = {
	Spell:GetEffectTag(FearsomeHowl, 0),
	Spell:GetEffectTag(FearsomeHowl, 1),
}

function FearsomeHowl:Apply(params)
	return function()
		Spell:Pause(Spell.SPELL_INTRO_PAUSE)
		Spell:ApplyStatusEffectTo(params.target, params, self, "Stun", 3)
		Spell:Pause(Spell.SPELL_EFFECT_PAUSE)
		Spell:ApplyStatusEffectTo(params.target, params, self, "Fear", 7, -100, 0)
		Spell:Pause(Spell.SPELL_EFFECT_PAUSE)
		Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	end
end

function FearsomeHowl:Notify(grid,notify)
	local heading, text, startTime = Spell:ShowStandardSpellHeader(self.title)
	
	
	-- get widget info
	local client = notify:GetAttribute("client")
	local opClient = Spell:GetOpposingClient(client)
	local button = Spell:GetSpellButtonWidgetName(notify)
	
	-- get positions
	local start_x, start_y = Spell:GetWidgetTargetPosition(client, button)
	local end_x, end_y = Spell:GetWidgetTargetPosition(opClient, "icon_portrait")
	
	-- calculate velocity
	local d_x = end_x - start_x
	local d_y = end_y - start_y
	local theta = math.atan(d_y/d_x)
	local vel = 1
	local vel_x = math.cos(theta) * vel
	local vel_y = math.sin(theta) * vel
	
	local pathCount = 3
	for i=1,pathCount do
		scheduler:AddChildThread(self, function ()
			scheduler:Sleep((i-1) * 100)
			
			-- Create path
			local path = MovementManager:Construct("RoundedNonuniformSpline")
			
			Spell:PushPosition(path, start_x, start_y)
			Spell:PushVelocity(path, vel_x + math.random(0, 1),vel_y + math.random(-1, 1))
			
			Spell:PushPosition(path, end_x, end_y)
			Spell:PushVelocity(path, vel_x + math.random(0, 1),vel_y  + math.random(-1, 1))
			
			path:SetAttribute("Duration", 2500)
			path:SetAttribute("Smoothing", true)
			
			-- create particle def's
			local def1 = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("FearSmoke"))
			local def2 = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("FearSkulls"))
			
			-- define update function
			local function updatef(pdef_a, d_t)
			local spin_vel = 10
			local vel = 1
			local angle_step = 2 * math.pi / table.getn(pdef_a)
			local offset = 2 * (math.pi - angle_step)
			for i, def in ipairs(pdef_a) do
				--if def == def1 then
					spin_vel = math.random(-8, 8)
					vel = math.random(-2, 2)
					local v_x = math.cos(-(offset + (i - 1) * angle_step + spin_vel * d_t / 1000)) * vel
					local v_y = math.sin(-(offset + (i - 1) * angle_step + spin_vel * d_t / 1000)) * vel
					def:SetVelocity(v_x, v_y, 0)
				--else
					--def:SetVelocity(0, math.random(-1, 1), 0)
				--end
			end
		end
		
		-- dowiiittttt
		Spell:AttachParticleMotionFragment(path, { def1, def2 }, updatef)
		end, "FX")
	end
	
	
	Spell:ShowStandardEffectSubtitle(self.subtitles[1], Spell.SPELL_EFFECT_PAUSE)
	Spell:ShowStandardEffectSubtitle(self.subtitles[2], Spell.SPELL_EFFECT_PAUSE)
	Spell:RemoveStandardSpellHeader(heading, text, startTime)
end

return FearsomeHowl
