local Weaken = {}
local Spell = require("Spells/Spell")

--[[

Halves selected mana total.

]]

Weaken.id		= "WEAKEN"
Weaken.icon	= "img_spell_weaken"
Weaken.short_name = "WEAKEN"
Weaken.sound	= "sp_weaken"
Weaken.sound_ds = "snd_sp_curselight"
Weaken.sound_bank_ds = "BANK_SP_CURSELIGHT"
Weaken.dietfxParticle	= "magicexplosion"
Weaken.dietfxTexture	= "storm"
Weaken.requiresTarget = true
Weaken.is_stat_logged  = true
Weaken.targetText = "[WEAKEN_TARGETTEXT]"
Weaken.targetOnly = {"Red", "Green", "Blue", "Yellow", "Black"}
Weaken.cooldownForAI = 2
Weaken.cooldown = 2
Weaken.cost	= {
	yellow	= 5,
	green   = 4,
}
Weaken.effects = {
	{"[WEAKEN_EFFECT0_HEAD]", "[WEAKEN_EFFECT0_BODY]"},
}
Weaken.title = Spell:GetTitleTag(Weaken)
Weaken.subtitles = {
	Spell:GetEffectTag(Weaken, 0),
	Spell:GetEffectTag(Weaken, 1),
}

function Weaken:Apply(params,notify)
	local pool = params.grid[params.targetX][params.targetY].def.name
	local manaDamage = math.floor(params.target.state[string.lower(pool)]/2)
	LOG("Weaken x="..params.targetX..", y="..params.targetY)
	notify:PushAttribute("gemx",params.targetX)
	notify:PushAttribute("gemy",params.targetY)
	notify:PushAttribute("sparams",pool)
	
	return function ()
		Spell:Pause(Spell.SPELL_INTRO_PAUSE)
		Spell:ExplodeGemByPos(params, params.targetX, params.targetY, false)
		Spell:DamageMana(params, pool, manaDamage)
		Spell:Pause(Spell.SPELL_MIN_DURATION)
		Spell:AwardBonusMove(params)		
		Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	end
end

function Weaken:Notify(grid,notify)
	local heading, text, startTime = Spell:ShowStandardSpellHeader(self.title)
	
	local x = notify:GetAttributeAt("gemx",1)
	local y = notify:GetAttributeAt("gemy",1)
	local color = notify:GetAttributeAt("sparams",1)
	LOG("Weaken x="..x..", y="..y)
	local gem = grid:GemAt(x,y)
	--local color = gem.name
	LOG("Color "..tostring(color))
	
	--Mana Jets
	local icon = string.format("icon_%s",string.lower(color))
	local widgetsList = {
		  {2,icon,0,0};
		  {2,icon,0,1};
		};
	local path = Spell:WidgetPath(nil,notify,widgetsList,0,0)
	
	Spell:AttachParticleMotionFragment(path, {ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription(string.format("Jet%s",color)))}, nil)
	
	
	
	--GemGlow/Explode
	local width = gem:GetView():GetExtentsX()
	local duration = 1500
	local size = 1.3 * (width/72)
	local offset = SCREENS.BattleGameMenu:get_widget_w("icon_board")
	x = gem:GetX()
	y = offset - gem:GetY()
	offset = offset/2
	widgetsList = {
		  {3,"icon_board",x-offset,y-offset};
		  {3,"icon_board",x-offset,y-offset+1};
		};
	path = Spell:WidgetPath(nil,notify,widgetsList,0,0)
	
	if color == "Black" then
		color = "Purple"
	end
	local particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription(string.format("Message%s",color)))
	particle:SetAlpha(0.0)
	particle:SetTargetAlpha(0.3)
	particle:SetNumParticlesToRelease(10)
	particle:SetMaxParticles(10)
	particle:SetLifetime(duration)
	particle:SetLifeCycle(duration)
	particle:SetVelocityVariation(0.05)
	particle:SetSize(size)
	particle:SetAnimateSize(false)
	
	Spell:AttachParticleMotionFragment(path, {particle}, nil)
	
	--Gem Explosion
	path = Spell:WidgetPath(nil,notify,widgetsList,0,0)	
	Spell:AttachParticleMotionFragment(path, {ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription(string.format("%sExplosion",color)))}, nil,duration)
	
	
	
	Spell:Pause(Spell.SPELL_INTRO_PAUSE)
	Spell:ShowStandardEffectSubtitle(self.subtitles[1], Spell.SPELL_MIN_DURATION)
	Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	Spell:RemoveStandardSpellHeader(heading, text, startTime)
end

function Weaken:GetScore(params)
	local gem = table.random(params.host.grid:GetAllGemsByAttribute("givesMana"))
	
	if not gem then
		return 0
	end
	
	return 50, gem.x, gem.y
end

return Weaken
