
local STATS = Stats.GetSinglePlayerStats()

STATS.ASSASSIN.COMPLETIONS_MALE.steam_name = "COMPLETIONS_M_ASSASSIN"
STATS.ASSASSIN.COMPLETIONS_FEMALE.steam_name = "COMPLETIONS_F_ASSASSIN"
STATS.INQUISITOR.COMPLETIONS_MALE.steam_name = "COMPLETIONS_M_TEMPLAR"
STATS.INQUISITOR.COMPLETIONS_FEMALE.steam_name = "COMPLETIONS_F_TEMPLAR"
STATS.WARMAGE.COMPLETIONS_MALE.steam_name = "COMPLETIONS_M_SORCERER"
STATS.WARMAGE.COMPLETIONS_FEMALE.steam_name = "COMPLETIONS_F_SORCERER"
STATS.BARBARIAN.COMPLETIONS_MALE.steam_name = "COMPLETIONS_M_BARBARIAN"
STATS.BARBARIAN.COMPLETIONS_FEMALE.steam_name = "COMPLETIONS_F_BARBARIAN"
STATS.OVERALL.YETITURNS.steam_name = "YETI_TURN_COUNTER"
STATS.ONLINE.CURRENT_STREAK.steam_name = "ONLINE_CURRENT_STREAK"

local function members_empty(t)
	for k,v in pairs(t) do
		if #v > 0 then
			return false
		end
	end
	return true
end

local function reset(t)
	for i=1,#t do
		t[i] = nil
	end
end

local function aggregate(t, agg, current)
	local v
	
	if agg == "agg_min" then
		v = math.min(current, unpack(t))
	elseif agg == "agg_max" then
		v = math.max(current, unpack(t))
	elseif agg == "agg_bestturn" then
		-- #4024: last minute hack to fix problem with the minimum turns for steam stats.  
		local lmin = math.min(unpack(t))
		
		if current == 0 then
			v = lmin
		else
			v = math.min(lmin, current)
		end
	elseif agg == "agg_sum" then
		v = current
		for i, s in ipairs(t) do
			v = v + s
		end
	elseif agg == "agg_last" then
		v = t[#t]
	elseif agg == "agg_streak" then
		v = current
		for i, s in ipairs(t) do
			if v < 0 and s < 0 then
				v = v - 1
			elseif v > 0 and s > 0 then
				v = v + 1
			else
				v = s
			end
		end
	else
		error("unknown aggregator: " .. tostring(agg))
	end
	
	return v
end

function Stats.GetStatPlatform(SP_STATS, category, name)
	local steam_name = SP_STATS[category][name].steam_name
	assert(steam_name, string.format("The stat '%s' '%s' has no steam stat name associated with it", tostring(category), tostring(name)))
	local success, value = Steam.GetIntegerValue(steam_name)
	assert(success)
	return value
end

local function increment_stat(values, agg_method, steam_name)
	assert(values)
	assert(agg_method)
	assert(steam_name)
	if #values > 0 then
		local res, old_value = Steam.GetIntegerValue(steam_name)
		if not res then
			LOGF("UNKNOWN STAT '%s'", tostring(steam_name))
		end
		local res2 = Steam.SetIntegerValue(steam_name, aggregate(values, agg_method, old_value))
		LOGF("Steam.SetIntegerValue(%s,%s) : %s : (was %s)", tostring(steam_name), tostring(aggregate(values, agg_method, old_value)), tostring(res2), tostring(old_value))
		reset(values)
	end
end

local CLASSES = {
	["ASSASSIN"] = true,
	["BARBARIAN"] = true,
	["SORCERER"] = true,
	["TEMPLAR"] = true, 
}

function Stats.PushSinglePlayerStatsPlatform(SP_STATS, cb)
	LOGF("PushSinglePlayerStatsPlatform()")

	increment_stat(SP_STATS.OVERALL.GAMESWON, 					"agg_sum", 		"VICTORY_COUNTER")
	increment_stat(SP_STATS.OVERALL.ROOMS_VISITED,				"agg_max", 		"ROOM_COUNTER")
	increment_stat(SP_STATS.OVERALL.YETITURNS,					"agg_max", 		"YETI_TURN_COUNTER")
	increment_stat(SP_STATS.OVERALL.TOURNAMENT_BATTLES,			"agg_sum", 		"TOURNAMENT_BATTLE_COUNT")
	increment_stat(SP_STATS.OVERALL.TOURNAMENT_VICTORIES,		"agg_sum", 		"TOURNAMENT_VICTORY_COUNT")
	increment_stat(SP_STATS.OVERALL.HOARD_WINS,					"agg_sum", 		"LOOT_COUNTER")
	increment_stat(SP_STATS.OVERALL.SEARCH_AMBUSHES,			"agg_sum", 		"SEARCH_AMBUSH_COUNTER")
	increment_stat(SP_STATS.OVERALL.SEARCH_SECRETS,				"agg_sum", 		"SEARCH_SECRET_COUNTER")
	increment_stat(SP_STATS.OVERALL.SEARCH_TRAPS,				"agg_sum", 		"SEARCH_TRAP_COUNTER")
	increment_stat(SP_STATS.OVERALL.XP,							"agg_sum", 		"TOTAL_XP")
	increment_stat(SP_STATS.OVERALL.GOLD,						"agg_sum", 		"TOTAL_GOLD")
	increment_stat(SP_STATS.OVERALL.BATTLE_TIME,				"agg_sum", 		"BATTLE_TIME")
	increment_stat(SP_STATS.OVERALL.ENDURANCE_BATTLES,			"agg_sum", 		"ENDURANCE_BATTLE_COUNT")
	increment_stat(SP_STATS.OVERALL.ENDURANCE_VICTORIES,		"agg_sum", 		"ENDURANCE_VICTORY_COUNT")
	increment_stat(SP_STATS.OVERALL.CLASSES_COMPLETED,			"agg_max", 		"CLASS_COMPLETION_COUNTER")
	
	increment_stat(SP_STATS.ASSASSIN.COMPLETIONS_MALE, 			"agg_sum", 		"COMPLETIONS_M_ASSASSIN")
	increment_stat(SP_STATS.ASSASSIN.COMPLETIONS_FEMALE, 		"agg_sum", 		"COMPLETIONS_F_ASSASSIN")
	increment_stat(SP_STATS.ASSASSIN.VICTORIES_MALE, 			"agg_sum", 		"ASSASSIN_M_VICTORIES")
	increment_stat(SP_STATS.ASSASSIN.VICTORIES_FEMALE, 			"agg_sum", 		"ASSASSIN_F_VICTORIES")
	increment_stat(SP_STATS.ASSASSIN.BATTLE_COUNT, 				"agg_sum",		"ASSASSIN_BATTLE_COUNT")
	increment_stat(SP_STATS.ASSASSIN.TOTAL_DAMAGE, 				"agg_sum",		"ASSASSIN_TOTAL_DAMAGE")
	increment_stat(SP_STATS.ASSASSIN.BEST_DAMAGE, 				"agg_max",		"ASSASSIN_BEST_DAMAGE")
	increment_stat(SP_STATS.ASSASSIN.TOTAL_ABSORBED, 			"agg_sum",		"ASSASSIN_TOTAL_ABSORBTION")
	increment_stat(SP_STATS.ASSASSIN.BEST_PLAY_TIME,		 	"agg_min",		"COMPLETION_TIME_ASSASSIN")
	increment_stat(SP_STATS.ASSASSIN.BEST_ENDURANCE_RESULT,	 	"agg_max",		"ENDURANCE_ASSASSIN_BEST")
	increment_stat(SP_STATS.ASSASSIN.ONLINE_BATTLES,	 		"agg_sum",		"ASSASSIN_ONLINE_BATTLES")
	increment_stat(SP_STATS.ASSASSIN.ONLINE_VICTORIES_MALE,	 	"agg_sum",		"ASSASSIN_M_ONLINE_VICTORIES")
	increment_stat(SP_STATS.ASSASSIN.ONLINE_VICTORIES_FEMALE,	"agg_sum",		"ASSASSIN_F_ONLINE_VICTORIES")

	increment_stat(SP_STATS.BARBARIAN.COMPLETIONS_MALE, 		"agg_sum", 		"COMPLETIONS_M_BARBARIAN")
	increment_stat(SP_STATS.BARBARIAN.COMPLETIONS_FEMALE, 		"agg_sum", 		"COMPLETIONS_F_BARBARIAN")
	increment_stat(SP_STATS.BARBARIAN.VICTORIES_MALE, 			"agg_sum", 		"BARBARIAN_M_VICTORIES")
	increment_stat(SP_STATS.BARBARIAN.VICTORIES_FEMALE, 		"agg_sum", 		"BARBARIAN_F_VICTORIES")
	increment_stat(SP_STATS.BARBARIAN.BATTLE_COUNT, 			"agg_sum",		"BARBARIAN_BATTLE_COUNT")
	increment_stat(SP_STATS.BARBARIAN.TOTAL_DAMAGE, 			"agg_sum",		"BARBARIAN_TOTAL_DAMAGE")
	increment_stat(SP_STATS.BARBARIAN.BEST_DAMAGE, 				"agg_max",		"BARBARIAN_BEST_DAMAGE")
	increment_stat(SP_STATS.BARBARIAN.BEST_ATTACK_RATING, 		"agg_max",		"BARBARIAN_BEST_ATTACK_RATING")
	increment_stat(SP_STATS.BARBARIAN.BEST_PLAY_TIME,		 	"agg_min",		"COMPLETION_TIME_BARBARIAN")
	increment_stat(SP_STATS.BARBARIAN.BEST_ENDURANCE_RESULT,	"agg_max",		"ENDURANCE_BARBARIAN_BEST")
	increment_stat(SP_STATS.BARBARIAN.ONLINE_BATTLES,	 		"agg_sum",		"BARBARIAN_ONLINE_BATTLES")
	increment_stat(SP_STATS.BARBARIAN.ONLINE_VICTORIES_MALE,	"agg_sum",		"BARBARIAN_M_ONLINE_VICTORIES")
	increment_stat(SP_STATS.BARBARIAN.ONLINE_VICTORIES_FEMALE,	"agg_sum",		"BARBARIAN_F_ONLINE_VICTORIES")

	increment_stat(SP_STATS.WARMAGE.COMPLETIONS_MALE, 			"agg_sum", 		"COMPLETIONS_M_SORCERER")
	increment_stat(SP_STATS.WARMAGE.COMPLETIONS_FEMALE, 		"agg_sum", 		"COMPLETIONS_F_SORCERER")
	increment_stat(SP_STATS.WARMAGE.VICTORIES_MALE, 			"agg_sum", 		"SORCERER_M_VICTORIES")
	increment_stat(SP_STATS.WARMAGE.VICTORIES_FEMALE, 			"agg_sum", 		"SORCERER_F_VICTORIES")
	increment_stat(SP_STATS.WARMAGE.BATTLE_COUNT, 				"agg_sum",		"SORCERER_BATTLE_COUNT")
	increment_stat(SP_STATS.WARMAGE.TOTAL_DAMAGE, 				"agg_sum",		"SORCERER_TOTAL_DAMAGE")
	increment_stat(SP_STATS.WARMAGE.BEST_DAMAGE, 				"agg_max",		"SORCERER_BEST_DAMAGE")
	increment_stat(SP_STATS.WARMAGE.BEST_MANA_GAIN, 			"agg_max",		"SORCERER_BEST_MANA_GAIN")
	increment_stat(SP_STATS.WARMAGE.BEST_PLAY_TIME,		 		"agg_min",		"COMPLETION_TIME_SORCERER")
	increment_stat(SP_STATS.WARMAGE.BEST_ENDURANCE_RESULT,	 	"agg_max",		"ENDURANCE_SORCERER_BEST")
	increment_stat(SP_STATS.WARMAGE.ONLINE_BATTLES,	 			"agg_sum",		"SORCERER_ONLINE_BATTLES")
	increment_stat(SP_STATS.WARMAGE.ONLINE_VICTORIES_MALE,	 	"agg_sum",		"SORCERER_M_ONLINE_VICTORIES")
	increment_stat(SP_STATS.WARMAGE.ONLINE_VICTORIES_FEMALE,	"agg_sum",		"SORCERER_F_ONLINE_VICTORIES")

	increment_stat(SP_STATS.INQUISITOR.COMPLETIONS_MALE, 		"agg_sum", 		"COMPLETIONS_M_TEMPLAR")
	increment_stat(SP_STATS.INQUISITOR.COMPLETIONS_FEMALE, 		"agg_sum", 		"COMPLETIONS_F_TEMPLAR")
	increment_stat(SP_STATS.INQUISITOR.VICTORIES_MALE, 			"agg_sum", 		"TEMPLAR_M_VICTORIES")
	increment_stat(SP_STATS.INQUISITOR.VICTORIES_FEMALE, 		"agg_sum", 		"TEMPLAR_F_VICTORIES")
	increment_stat(SP_STATS.INQUISITOR.BATTLE_COUNT, 			"agg_sum",		"TEMPLAR_BATTLE_COUNT")
	increment_stat(SP_STATS.INQUISITOR.TOTAL_DAMAGE, 			"agg_sum",		"TEMPLAR_TOTAL_DAMAGE")
	increment_stat(SP_STATS.INQUISITOR.BEST_DAMAGE, 			"agg_max",		"TEMPLAR_BEST_DAMAGE")
	increment_stat(SP_STATS.INQUISITOR.BEST_BLOCK, 				"agg_max",		"TEMPLAR_BEST_BLOCK")
	increment_stat(SP_STATS.INQUISITOR.BEST_PLAY_TIME,		 	"agg_min",		"COMPLETION_TIME_TEMPLAR")
	increment_stat(SP_STATS.INQUISITOR.BEST_ENDURANCE_RESULT,	"agg_max",		"ENDURANCE_TEMPLAR_BEST")
	increment_stat(SP_STATS.INQUISITOR.ONLINE_BATTLES,	 		"agg_sum",		"TEMPLAR_ONLINE_BATTLES")
	increment_stat(SP_STATS.INQUISITOR.ONLINE_VICTORIES_MALE,	"agg_sum",		"TEMPLAR_M_ONLINE_VICTORIES")
	increment_stat(SP_STATS.INQUISITOR.ONLINE_VICTORIES_FEMALE,	"agg_sum",		"TEMPLAR_F_ONLINE_VICTORIES")
	
	increment_stat(SP_STATS.CHALLENGE.MIN_GREEN_DRAGON_TURNS, 	"agg_bestturn",	"GREEN_DRAGON_BEST_TURN_COUNT")
	increment_stat(SP_STATS.CHALLENGE.MIN_TWO_HEADED_OGRE_TURNS,"agg_bestturn",	"TWO_HEADED_OGRE_BEST_TURN_COUNT")
	increment_stat(SP_STATS.CHALLENGE.MIN_GORGON_TURNS, 		"agg_bestturn",	"GORGON_BEST_TURN_COUNT")
	increment_stat(SP_STATS.CHALLENGE.MIN_ARCH_LICH_TURNS,		"agg_bestturn",	"ARCH_LICH_BEST_TURN_COUNT")
	increment_stat(SP_STATS.CHALLENGE.MIN_SHADOWBRINGER_TURNS,	"agg_bestturn",	"SHADOWBRINGER_BEST_TURN_COUNT")
	increment_stat(SP_STATS.CHALLENGE.MIN_YETI_TURNS,			"agg_bestturn",	"YETI_BEST_TURN_COUNT")
	
	increment_stat(SP_STATS.ONLINE.TOTAL_DAMAGE,				"agg_sum",		"ONLINE_TOTAL_DAMAGE")
	increment_stat(SP_STATS.ONLINE.BEST_DAMAGE,					"agg_max",		"ONLINE_BEST_DAMAGE")
	increment_stat(SP_STATS.ONLINE.BEST_BLOCK,					"agg_max",		"ONLINE_BEST_BLOCK")
	increment_stat(SP_STATS.ONLINE.SHORTEST_BATTLE,				"agg_min",		"ONLINE_SHORTEST_BATTLE")
	increment_stat(SP_STATS.ONLINE.LONGEST_BATTLE,				"agg_max",		"ONLINE_LONGEST_BATTLE")
	increment_stat(SP_STATS.ONLINE.TOTAL_TURNS,					"agg_sum",		"ONLINE_TOTAL_TURN_COUNT")
	increment_stat(SP_STATS.ONLINE.TOTAL_ENEMY_TURNS,			"agg_sum",		"ONLINE_TOTAL_ENEMY_TURN_COUNT")
	increment_stat(SP_STATS.ONLINE.TOTAL_MOVES,					"agg_sum",		"ONLINE_TOTAL_MOVE_COUNT")
	increment_stat(SP_STATS.ONLINE.TOTAL_ENEMY_MOVES,			"agg_sum",		"ONLINE_TOTAL_ENEMY_MOVE_COUNT")
	increment_stat(SP_STATS.ONLINE.BONUS_MOVES,					"agg_sum",		"ONLINE_TOTAL_BONUS_MOVES")
	increment_stat(SP_STATS.ONLINE.BEST_BONUS_MOVES_STREAK,		"agg_max",		"ONLINE_BEST_MOVE_STREAK")
	increment_stat(SP_STATS.ONLINE.BEST_DAMAGE_PER_TURN,		"agg_max",		"ONLINE_BEST_DAMAGE_PER_TURN")
	increment_stat(SP_STATS.ONLINE.CURRENT_STREAK,				"agg_streak",	"ONLINE_CURRENT_STREAK")
	increment_stat(SP_STATS.ONLINE.BEST_STREAK,					"agg_max",		"ONLINE_BEST_STREAK")
	
	for my_class in pairs(CLASSES) do
		for enemy_class in pairs(CLASSES) do
			local local_name = string.format("%s_V_%s", my_class, enemy_class)
			local remote_name = string.format("%s_VS_%s_VICTORIES", my_class, enemy_class)
			
			increment_stat(SP_STATS.ONLINE[local_name], 		"agg_sum",	remote_name)
		end
	end

	for k,v in pairs(SP_STATS.SPELLS) do
		increment_stat(v, "agg_sum", string.format("%s_CAST_COUNT", string.upper(k)))
	end
	
	for k,v in pairs(SP_STATS.TOURNAMENT) do
		increment_stat(v, "agg_sum", string.format("TOURNAMENT_%s_VICTORY_COUNT", string.upper(v.rename or k)))
	end
	
	for k,v in pairs(SP_STATS.ENDURANCE) do
		increment_stat(v, "agg_sum", string.format("ENDURANCE_REWARD_%s", string.upper(k)))
	end
	
	LOGF("FlushUserStats in platform stats returned: %s", tostring(Steam.FlushUserStats(function () 
		LOGF("Stats have been stored") 
		
		Achievements.OnStatsStored()
		
		if cb then
			cb()
		end
	end, function () --[[ achievement awarded]] end)))
end
