local DraggingSupport = {}

function DraggingSupport:AddToClass(cls)
	cls.OnDragInit = self.OnDragInit
	cls.OnDragStart = self.OnDragStart
	cls.OnDragEnd = self.OnDragEnd
	cls.OnDragUpdate = self.OnDragUpdate
	cls.OnDragReset = self.OnDragReset
end

function DraggingSupport:OnDragInit(sorting_value, scale)
	assert(sorting_value)
	assert(scale)
	
	self.dragStartObj = nil
	self.dragHoverObj = nil
	
	self.OverlayDrag1 = GameObjectViewManager:Construct("GraphicAsset", "img_target_select_b")
	self.OverlayDrag1:SetScale(scale)
	self.OverlayDrag1:SetSortingValue(sorting_value)
	self.OverlayDrag1:SetAutoDir(true)

	self.OverlayDrag2 = GameObjectViewManager:Construct("GraphicAsset", "img_target_select_b")
	self.OverlayDrag2:SetScale(scale)
	self.OverlayDrag2:SetSortingValue(sorting_value) 
	self.OverlayDrag2:SetAutoDir(true)
	
end

function DraggingSupport:OnDragStart(obj)
	if self.dragStartObj then
		self:OnDragReset()
	end
	if obj.IS_GEM and obj.def.canSwap then
		self.dragStartObj = obj
		self.dragStartObj:AddOverlay("drag_start", self.OverlayDrag1, 0, 0)
	end
end

function DraggingSupport:OnDragEnd(end_obj, x, y)
	if self.dragStartObj and end_obj and end_obj.IS_GEM and end_obj.def.canSwap then
		local gem1 = self.dragStartObj
		local gem2 = end_obj
		local valid = false
		
		if (gem1.x == gem2.x + 1 or gem1.x == gem2.x - 1) and gem1.y == gem2.y then
			valid = true
		elseif (gem1.y == gem2.y + 1 or gem1.y == gem2.y - 1) and gem1.x == gem2.x then
			valid = true
		end
		
		if valid then
			self:SendSwapRequest(self.dragStartObj.x, self.dragStartObj.y, end_obj.x, end_obj.y)
		end

		self:OnDragReset()
		
		return valid
	else
		self:OnDragReset()
	end
end

function DraggingSupport:OnDragUpdate(hover_object)
	if self.dragStartObj then
		local valid = false
		
		if hover_object and hover_object.IS_GEM and hover_object.def.canSwap then
			if (hover_object.x == self.dragStartObj.x + 1 or hover_object.x == self.dragStartObj.x - 1) and hover_object.y == self.dragStartObj.y then
				valid = true
			elseif (hover_object.y == self.dragStartObj.y + 1 or hover_object.y == self.dragStartObj.y - 1) and hover_object.x == self.dragStartObj.x then
				valid = true
			end
		end
		
		if not valid then
			hover_object = nil
		end
		
		if hover_object ~= self.dragHoverObj then
			if self.dragHoverObj then
				self.dragHoverObj:RemoveOverlay("drag_target")
				SoundSystem.Play("pq2audio/board/select1")
			end
			
			self.dragHoverObj = hover_object
			
			if self.dragHoverObj then
				self.dragHoverObj:AddOverlay("drag_target", self.OverlayDrag2, 0, 0)
				SoundSystem.Play("pq2audio/board/select1")
			end
		end
	end
end

function DraggingSupport:OnDragReset()
	if self.dragHoverObj then
		self.dragHoverObj:RemoveOverlay("drag_target")
	end
	if self.dragStartObj then
		self.dragStartObj:RemoveOverlay("drag_start")
	end
	self.dragHoverObj = nil
	self.dragStartObj = nil
end

return DraggingSupport
